<?PHP
	/*
		Проверяем валидность кассы
	*/
	if(empty($_POST['merchant'])) {
		AlertError(getLang('alerts.market.buy.merchant.not_exists'));
	}
	
	$Merchants = Merchants();
	
	if(empty($Merchants[$_POST['merchant']])) {
		AlertError(getLang('alerts.market.buy.merchant.not_exists'));
	}
	
	if(!$Merchants[$_POST['merchant']]['enable']) {
		AlertError(getLang('alerts.market.buy.merchant.not_enable'));
	}
	
	/*
		Проверяем валидность товара
	*/
	if(empty($_POST['product_id'])) {
		AlertError(getLang('alerts.market.buy.not_select'));
	}
	
	$dataProduct = Product()->Get($_POST['product_id']);
	if(empty($dataProduct)) {
		AlertError(getLang('alerts.market.buy.not_exists'));
	}
	
	$dataProduct = (object)$dataProduct['product'];
	
	if(!$dataProduct->enable) {
		AlertError(getLang('alerts.market.buy.disabled'));
	}
	
	/*
		Настраиваем итоговую цену
	*/
	$count = preg_replace('/[^\d]/', '', $_POST['count']);
	$count = (int)$count;
	
	if(empty($count)) {
		$count = 1;
	}
	
	$price = formatPrice($_POST['product_id']);
	$endPrice = reFormatPrice($price->total);
	
	if($count > 1) {
		$endPrice = reFormatPrice($endPrice * $count);
	}

	if($endPrice < conf()->min_price) {
		Result([
			'Alert' => 'Alert',
			'Message' => getLang('buy.alert.min_price', conf()->min_price . ' ' . conf()->currency->html)
		]);
	}

	$count_quantity = Product()->getQuantity($_POST['product_id']);
	if($count_quantity < $count) {
		Result([
			'Alert' => 'Alert',
			'Message' => getLang('buy.alert.limit_count', [
				is_null($count_quantity) ? 0 : $count_quantity
			])
		]);
	}

	/*
		Работаем непосредственно с платежными решениями
	*/
	switch($_POST['merchant']) {
		/*
			Локальная покупка
		*/
		case 'local': {
			if(empty($_SESSION['id'])) {
				AlertError(getLang('alerts.not_auth'));
			}
			
			$UsersBalance = new UsersBalance;
			if(!$UsersBalance->isBalance($_SESSION['id'], $endPrice)) {
				AlertError(getLang('alerts.is_money'));
			}
			
			$UsersTransactions = new UsersTransactions;
			
			if($UsersBalance->minBalance($_SESSION['id'], $endPrice)) {
				$type_delivery = Product()->getTypeDelivery($dataProduct->id);
				
				if(is_null($type_delivery)) {
					AlertError(getLang('alerts.market.buy.error'));
				}

				switch($type_delivery) {
					case 'auto': {
						$deliveryArray = json_decode($dataProduct->data_auto, true);
						$deliveryItems = [];

						for($i = 0; $i < $count; $i++) {
							$deliveryRandId = array_rand($deliveryArray);
							$deliveryItems[] = $deliveryArray[$deliveryRandId];
							unset($deliveryArray[$deliveryRandId]);
						}

						if(empty($deliveryArray)) {
							$deliveryArray = null;
						}

						$delivery = $deliveryItems;
						break;
					}

					default: {
						$deliveryCount = $dataProduct->quantity - $count;
					}
				}

				$transParams = [
					'amount' => $endPrice,
					'data' => [
						'product_id' => $dataProduct->id,
						'count' => $count
					],
					'type' => $UsersTransactions->getTypeFromCode('purchase')->id,
					'status' => 1
				];

				if(isset($delivery)) {
					$transParams['data']['delivery'] = $delivery;
				}

				$trans_id = $UsersTransactions->Add($_SESSION['id'], $transParams);
				$UsersBalance->addFrozen($dataProduct->author_id, $trans_id, $endPrice);

				if(isset($delivery)) {
					$updateData = [
						'data_auto' => is_null($deliveryArray) ? null : json_encode($deliveryArray)
					];

					if(is_null($deliveryArray)) {
						$updateData['enable'] = 2;
					}
				}
				else {
					$updateData = [
						'quantity' => $deliveryCount
					];

					if($deliveryCount == 0) {
						$updateData['enable'] = 2;
					}
				}

				Product()->Update($dataProduct->id, $updateData);
				
				CallBack()->Call('onBuy', [
					'merchant' => $_POST['merchant'],
					'user_id' => $_SESSION['id'],
					'product_id' => $dataProduct->id,
					'price' => $endPrice,
					'count' => $count,
					'delivery' => isset($delivery) ? $delivery : null,
					'date' => time()
				]);
				
				Result([
					'Alert' => 'Success',
					'Message' => getLang('alerts.redirect_chat'),
					'SellerId' => $dataProduct->author_id
				]);
			}
			else {
				$UsersTransactions->Add($_SESSION['id'], [
					'amount' => $endPrice,
					'data' => [
						'product_id' => $dataProduct->id,
						'count' => $count
					],
					'type' => $UsersTransactions->getTypeFromCode('purchase')->id,
					'status' => 3
				]);
				
				AlertError(getLang('alerts.market.buy.error'));
			}
			
			break;
		}
	}