<?PHP
	$nextChangeTime = 60; // Через сколько можно будет изменить Почтовый E-mail или отправить повторный запрос на смену.
	
	function GetLastChange($user_id) {
		try {
			$sth = pdo()->prepare('SELECT * FROM `users__change-email` WHERE `user_id`=:user_id LIMIT 1');
			$sth->execute([':user_id' => $user_id]);
			
			if(!$sth->rowCount()) {
				return null;
			}
			
			return $sth->fetch(PDO::FETCH_OBJ);
		}
		catch(Exception $e) {
			return null;
		}
	}
	
	try {
		switch($_POST['onType']) {
			/* Смена пароля */
			case 'changePassword': {
				if($_POST['new_password'] != $_POST['confirm_password']) {
					AlertError(getLang('alerts.password_dont_match'));
				}

				$data = users()->Get($_SESSION['id']);

				if(!password_verify($_POST['old_password'], $data->password)) {
					AlertError(getLang('alerts.invalid_old_password'));
				}

				users()->Update($_SESSION['id'], ['password' => password_hash($_POST['new_password'], PASSWORD_DEFAULT)]);
				AlertSuccess(getLang('alerts.edit_save'));

				break;
			}

			/* Просмотр покупок */
			case 'view_orders': {
				users()->updateSettings($_SESSION['id'], ['view_orders' => $_POST['value']]);
				AlertSuccess(getLang('alerts.edit_save'));

				break;
			}

			/* Сообщения */
			case 'messages': {
				users()->updateSettings($_SESSION['id'], ['messages' => $_POST['value']]);
				AlertSuccess(getLang('alerts.edit_save'));

				break;
			}
			
			case 'emailChange': {
				$dataLastChange = GetLastChange($_SESSION['id']);

				if(!is_null($dataLastChange)) {
					$checkTime = $dataLastChange->date + $nextChangeTime;
					$time = time();
					
					if($checkTime > $time) {
						AlertWarning(getLang('alerts.settings.email.not_time', [($checkTime - $time)]));
					}
				}
				
				if(users()->isEmail($_POST['email'])) {
					AlertError(getLang('alerts.settings.email.exists'));
				}
				
				/* Удаляем старые попытки смены почты */
				$sth = pdo()->prepare('DELETE FROM `users__change-email` WHERE `user_id`=:user_id');
				$sth->execute([':user_id' => $_SESSION['id']]);
				
				/* Отправляем письмо на старый почтовый адрес, для подтверждения */
				$code = generateConfirmationCode(4);
				$dataUser = users()->Get($_SESSION['id']);
				
				try {
					$mail = PHPMailer();
					$mail->addAddress($dataUser->email);
					
					$tpl = new Template('system');
					$body = $tpl->Execute($tpl->Set(['{code}' => $code], $tpl->Get('mailer/email_very')));
					$mail->setContent(getLang('alerts.settings.email.code', [$code]), $body);
					$mail->Send();
				}
				catch(Exception $e) {
					AlertError($e->getMessage());
				}
				
				$sth = pdo()->prepare('INSERT INTO `users__change-email`(`user_id`, `code`, `email`, `date`) VALUES (:user_id, :code, :email, :date)');
				$sth->execute([
					':user_id' => $_SESSION['id'],
					':code' => $code,
					':email' => $_POST['email'],
					':date' => time()
				]);
				
				AlertSuccess(getLang('alerts.settings.email.send'));
				break;
			}
			
			case 'emailChangeConfirm': {
				$dataLastChange = GetLastChange($_SESSION['id']);
				$code = $_POST['check_1'] . $_POST['check_2'] . $_POST['check_3'] . $_POST['check_4'];
				
				if($dataLastChange->code == $code) {
					try {
						/* Меняем почтовый адрес */
						$sth = pdo()->prepare('UPDATE `users` SET `email`=:email WHERE `id`=:id LIMIT 1');
						$sth->execute([
							':email' => $dataLastChange->email,
							':id' => $_SESSION['id']
						]);
						
						/* Удаляем старые попытки смены почты */
						$sth = pdo()->prepare('DELETE FROM `users__change-email` WHERE `user_id`=:user_id');
						$sth->execute([
							':user_id' => $_SESSION['id']
						]);
						
						Result([
							'Alert' => 'Success',
							'Message' => getLang('alerts.settings.email.confirm'),
							'Email' => $dataLastChange->email
						]);
					}
					catch(Exception $e) {
						AlertError($e->getMessage());
					}
				}
				else {
					AlertError(getLang('alerts.settings.email.error'));
				}
				
				break;
			}
		}
	}
	catch(Exception $e) {
		AlertError($e->getMessage());
	}