<?PHP
	if(!isset($_SERVER['HTTP_X_TELEGRAM_BOT_API_SECRET_TOKEN'])) {
		http_response_code(400);
		die('Missing Secret Token');
	}

	$request = json_decode(file_get_contents("php://input"), true);
	
	if(!$request || !$apiTelegram->isValid($_SERVER['HTTP_X_TELEGRAM_BOT_API_SECRET_TOKEN'])) {
		http_response_code(403);
		die('Invalid Secret Token');
	}
	
	if(isset($request['callback_query'])) {
		$data = json_decode($request['callback_query']['data'], true);
		$chatId = $request['callback_query']['message']['chat']['id'];

		switch($data['action']) {
			/* Прикрепляем профиль */
			case 'onBind': {
				if(!$apiTelegram->isData($chatId)) {
					$apiTelegram->onCreate($chatId);
				}
				
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->setLastCommand($chatId, $data['action']);
				$apiTelegram->onSend($chatId, 'Введите полученный Token ID');

				break;
			}
			
			/* Открываем настройки */
			case 'onSettings': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->onSend($chatId, 'Выберите действие', [
					'inline_keyboard' => [
						[
							[
								'text' => '🔓 Отвязать профиль',
								'callback_data' => json_encode([
									'action' => 'unBind'
								])
							],
							[
								'text' => '🔙 Назад',
								'callback_data' => json_encode([
									'action' => 'onMenu'
								])
							]
						]
					]
				]);
				
				break;
			}
			
			/* Снятие привязки учетной записи */
			case 'unBind': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->onSend($chatId, 'Вы действительно хотите отвязать профиль?', [
					'inline_keyboard' => [
						[
							[
								'text' => '✅ Да',
								'callback_data' => json_encode([
									'action' => 'unBind_access'
								])
							],
							[
								'text' => '❌ Нет',
								'callback_data' => json_encode([
									'action' => 'unBind_cancel'
								])
							]
						]
					]
				]);
				
				break;
			}
			
			/* Подтверждение отвязки профиля */
			case 'unBind_access': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->onDelete($chatId);
				$apiTelegram->onSend($chatId, 'Ваш профиль успешно отвязан!', $apiTelegram->inlineKeyboardMenu($chatId));
				
				break;
			}
			
			/* Отмена отвязки профиля */
			case 'unBind_cancel': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->onSend($chatId, 'Выберите действие', $apiTelegram->inlineKeyboardMenu($chatId));
				
				break;
			}
			
			/* Возвращение в меню */
			case 'onMenu': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->onSend($chatId, 'Выберите действие', $apiTelegram->inlineKeyboardMenu($chatId));
				
				break;
			}
			
			/* Выводим список доступных товаров */
			case 'onMyProduct': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->getMyProduct($chatId);
				
				break;
			}
			
			/* Выбор товара */
			case 'onSelectProduct': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				
				$apiTelegram->onSend($chatId, 'Выберите действие', ['inline_keyboard' => $apiTelegram->createInlineKeyboard([
					[
						['text' => '📝 Изменить', 'callback_data' => json_encode(['action' => 'onEditProduct', 'productId' => $data['productId']])],
						['text' => '🗑 Удалить', 'callback_data' => json_encode(['action' => 'onRemoveProduct', 'productId' => $data['productId']])]
					],
					[
						'text' => '🔙 Назад', 'callback_data' => json_encode(['action' => 'onMyProduct'])
					]
				])]);
				
				break;
			}
			
			/* История продаж */
			case 'onHistorySells': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				$apiTelegram->getMySells($chatId);
				
				break;
			}
			
			/* Выбераем транзакцию */
			case 'onSelectTransaction': {
				$apiTelegram->removeMessage($chatId, $request['callback_query']['message']['message_id']);
				
				$classUsersTransactions = new UsersTransactions;
				$dataTransaction = $classUsersTransactions->Get($data['id']);
				
				if(is_null($dataTransaction)) {
					$apiTelegram->onSend($chatId, 'Транзакция не найдена', [
						[
							[
								'text' => '🔙 Назад',
								'callback_data' => json_encode([
									'action' => 'onHistorySells'
								])
							]
						]
					]);
					
					die;
				}
				
				$d = json_decode($dataTransaction->data);
				$dataProduct = Product()->Get($d->product_id);
				$dataProduct = $dataProduct['product'];
				
				$message = "Транзакция: #" . $dataTransaction->id;
				$message .= "\nДата выполнения: " . date('d.m.Y в H:i', $dataTransaction->date);
				$message .= "\n\nКлиент: " . getName($dataTransaction->user_id);
				$message .= "\nТовар: " . $dataProduct['name'];
				$message .= "\nВ количестве: " . $d->count;
				
				$apiTelegram->onSend($chatId, $message, [
					[
						[
							'text' => '🌐 Открыть товар',
							'url' => 'https://' . $_SERVER['SERVER_NAME'] . '/market/index?id=' . $dataProduct['id']
						],
						[
							'text' => '💬 Чат с клиентом',
							'url' => 'https://' . $_SERVER['SERVER_NAME'] . '/account/messages?id=' . $dataTransaction->user_id
						],
					]
				]);
				
				$apiTelegram->onSend($chatId, $message, ['inline_keyboard' => $apiTelegram->createInlineKeyboard([
					[
						'text' => '🌐 Открыть товар',
						'url' => 'https://' . $_SERVER['SERVER_NAME'] . '/market/index?id=' . $dataProduct['id']
					],
					[
						[
							'text' => '💬 Чат с клиентом',
							'url' => 'https://' . $_SERVER['SERVER_NAME'] . '/account/messages?id=' . $dataTransaction->user_id
						],
						[
							'text' => '👨 Профиль клиента',
							'url' => 'https://' . $_SERVER['SERVER_NAME'] . '/account/home?id=' . $dataTransaction->user_id
						]
					],
					[
						'text' => '🔙 Назад',
						'callback_data' => json_encode([
							'action' => 'onHistorySells'
						])
					]
				])]);
				
				break;
			}
		}

		die;
	}

	if(isset($request['message']) && $request['message']) {
		$chatId = $request['message']['chat']['id'];

		if(isset($request['message']['text'])) {
			switch($request['message']['text']) {
				case '/start': {
					if($apiTelegram->isData($chatId)) {
						$apiTelegram->onSend($chatId, $apiTelegram->onFormateMessage("С возвращением, {profile_name}!\n\nМы помним Вас.", $request), $apiTelegram->inlineKeyboardMenu($chatId));
					}
					else {
						$apiTelegram->onSend($chatId, $apiTelegram->onFormateMessage("Привет, {profile_name}!\n\nВы открыли официальный Бот {site_name}.\n\nЧтобы продолжить работу, привяжите свой профиль по Token ID из профиля на сайте.", $request), $apiTelegram->inlineKeyboardMenu($chatId));
					}

					break;
				}
				
				case '/menu': {
					$apiTelegram->onSend($chatId, 'Выберите действие', $apiTelegram->inlineKeyboardMenu($chatId));
					
					break;
				}
				
				default: {
					$dataLast = $apiTelegram->getData($chatId);

					if(empty($dataLast)) {
						$apiTelegram->onSend($chatId, 'Неизвестная команда, попробуйте снова.');
						die;
					}

					switch($dataLast->last_command) {
						case 'onBind':
							$sth = pdo()->prepare('SELECT * FROM users__token WHERE hash=:hash LIMIT 1');
							$sth->execute([
								':hash' => $request['message']['text']
							]);

							if(!$sth->rowCount()) {
								$apiTelegram->onSend($chatId, 'Не удалось найти Token ID, попробуйте  снова.');
								die;
							}

							$userId = $sth->fetch(PDO::FETCH_OBJ)->user_id;
							$dataUser = users()->Get($userId);

							$sth = pdo()->prepare('UPDATE users SET telegram_id=:telegram_id WHERE id=:id LIMIT 1');

							$sth->execute([
								':telegram_id' => $chatId,
								':id' => $userId
							]);

							$apiTelegram->setUserId($chatId, $userId);
							$apiTelegram->setLastCommand($chatId, null);

							$classUsersRole = new UsersRole;
							$dataRole = $classUsersRole->Get($dataUser->roleid);

							$apiTelegram->onSend($chatId, "Ваш профиль привязан! Ваша роль - {$dataRole->name}", $apiTelegram->inlineKeyboardMenu($chatId));
							
							break;

						default:
							$apiTelegram->onSend($chatId, 'Неизвестная команда, попробуйте снова.');
					}
				}
			}
		}

		die;
	}
