<?PHP
	// Защита от path traversal
	function getSafePath($basePath, $relativePath) {
		// Нормализуем путь для предотвращения path traversal
		$realBasePath = realpath($basePath);
		$realRelativePath = realpath($basePath . '/' . $relativePath);
		
		if ($realRelativePath === false || strpos($realRelativePath, $realBasePath) !== 0) {
			// Если путь выходит за пределы базовой директории, это потенциальная угроза
			AlertError(getLang('api.errors.invalid_path'));
		}
		
		return $realRelativePath;
	}

	// Получаем путь для версии
	$version = $_PAGE['params'][1] ?? '';
	$path = $_SERVER['DOCUMENT_ROOT'] . '/../application/api/v' . $version;

	// Проверяем, существует ли версия
	if(!is_dir($path)) {
		AlertError(getLang('api.errors.version'));
	}

	// Загружаем preload.php, если он существует
	$preloadPath = getSafePath($path, 'preload.php');
	if(file_exists($preloadPath)) {
		require($preloadPath);
	}

	// Получаем путь для метода
	$method = $_PAGE['params'][2] ?? '';
	$path = getSafePath($path, $method);

	// Проверяем, существует ли метод
	if(!is_dir($path)) {
		AlertError(getLang('api.errors.method'));
	}

	// Получаем путь для модуля
	$module = $_PAGE['params'][3] ?? '';
	$modulePath = getSafePath($path, $module . '.php');

	// Проверяем, существует ли модуль
	if(!file_exists($modulePath)) {
		AlertError(getLang('api.errors.module'));
	}

	// Подключаем модуль
	require($modulePath);