<?PHP
	/*
		Создаем PDO
	*/
	function CreatePDO($base) {
		if(empty($base)) {
			return false;
		}
		
		try {
			$pdo = new PDO('mysql:host=' . $base['hostname'] . ';dbname=' . $base['dataname'], $base['username'], $base['password'], [
				PDO::ATTR_ERRMODE => PDO::ERRMODE_EXCEPTION,
				PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES 'utf8mb4'"
			]);
			
			return $pdo;
		}
		catch(PDOException $e) {
			die(
				'Error: ' . $e->getMessage()
			);
		}
	}
	
	/*
		Товары
	*/
	function Product() {
		global $product;
		
		if(empty($product)) {
			$product = new Product;
		}
		
		return $product;
	}
	
	/*
		Отправляет активный PDO
	*/
	function pdo() {
		global $pdo;
		
		if(empty($pdo)) {
			return null;
		}
		
		return $pdo;
	}
	
	/*
		Страницы
	*/
	function Pages() {
		global $Pages;
		
		if(empty($Pages)) {
			return new Pages;
		}
		
		return $Pages;
	}
	
	/*
		Шаблонизатор
	*/
	function tpl() {
		global $tpl;
		
		if(empty($tpl)) {
			return new Template;
		}
		
		return $tpl;
	}
	
	/*
		Генерация рандомных строк
		@ Len = количество символов
	*/
	function RandomString($Len) {
		return substr(
			str_shuffle(
				'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789'
			), 0, $Len
		);
	}
	
	/*
		Возвращает результат в JSON
	*/
	function Result($_ARRAY = []) {
		die(
			json_encode(
				$_ARRAY
			)
		);
	}
	
	/*
		Возвращает результат Успеха в JSON
	*/
	function AlertSuccess($_MESSAGE = '') {
		Result([
			'Alert' => 'Success',
			'Message' => $_MESSAGE
		]);
	}
	
	/*
		Возвращает результат Ошибки в JSON
	*/
	function AlertError($_MESSAGE = '') {
		Result([
			'Alert' => 'Error',
			'Message' => $_MESSAGE
		]);
	}
	
	/*
		Возвращает результат Предупреждения в JSON
	*/
	function AlertWarning($_MESSAGE = '') {
		Result([
			'Alert' => 'Warning',
			'Message' => $_MESSAGE
		]);
	}
	
	/*
		Возвращает результат Информации в JSON
	*/
	function AlertInfo($_MESSAGE = '') {
		Result([
			'Alert' => 'Info',
			'Message' => $_MESSAGE
		]);
	}
	
	/*
		Проверяет Валидность запроса
	*/
	function IsValidActions() {
		if(empty($_POST['phpaction'])) {
			AlertError(getLang('alerts.errors.action'));
		}
		
		if(!csrf()->Is($_POST['csrf_token'])) {
			AlertError(getLang('alerts.errors.token'));
		}
	}
	/*
		Записывает сведения в Логи
		@ File - имя файла, для записи
		@ Message - сообщение для записи
	*/
	function AddLogs($File, $Message) {
		tpl()
		->SetAppearance('system')
		->Start('logs/sample')
		->Set([
			'{system_date}' => date('d.m.Y'),
			'{system_time}' => date('H:i:s'),
			'{user_id}' => isset($_SESSION['id']) ? $_SESSION['id'] : getLang('system.unknown'),
			'{user_ip}' => users()->getRemote(),
			'{user_agent}' => $_SERVER['HTTP_USER_AGENT'],
			'{json_post}' => print_r($_POST, true),
			'{json_get}' => print_r($_GET, true),
			'{other_data}' => $Message
		]);
		
		$path = $_SERVER['DOCUMENT_ROOT'] . '/../application/system/logs/' . $File;

		if(createPathForFile($path)) {
			return file_put_contents($path, tpl()->getTemp(), FILE_APPEND);
		}
		
		return false;
	}
	
	/*
		Получение конфигураций
	*/
	function conf() {
		global $conf;
		
		if(isset($conf)) {
			return $conf;
		}
	}
	
	/*
		Редирект
	*/
	function Redirect($uri, $timeout = 300) {
		die('<script>setTimeout(\'location.href = "' . $uri . '";\', ' . $timeout . ');</script>');
	}
	
	/*
		CSRF защита
	*/
	function csrf() {
		global $csrf;
		
		if(isset($csrf)) {
			return $csrf;
		}
		
		return new CSRF;
	}
	
	/*
		Очистка текста
	*/
	function TextClear($string) {
		if(empty($string)) {
			return null;
		}
		
		$string = strip_tags($string);
		$string = htmlspecialchars($string);
		
		if(!mb_strlen($string)) {
			return null;
		}
		
		return $string;
	}
	
	/*
		Библиотка пользователи
	*/
	function users() {
		global $usr;
		
		if(isset($usr)) {
			return $usr;
		}
		
		return new Users;
	}
	
	/*
		Проверка валидности номера телефона
	*/
	function isValidPhoneNumber($phoneNumber) {
		$phoneNumber = preg_replace('/[^0-9]/', '', $phoneNumber);
		
		if(strlen($phoneNumber) === 11) {
			return $phoneNumber;
		}
		
		return false;
	}
	
	/*
		Получение сокращенной даты на русском
	*/
	function dateRussian($timestamp) {
		if(date('d') == date('d', $timestamp)) {
			return date('H:i', $timestamp);
		}
		
		$month = monthRussian($timestamp);
		return $month . date(' d', $timestamp);
	}
	
	/*
		Месяц на русском
	*/
	function monthRussian($timestamp) {
		$month = date('M', $timestamp);
		
		switch($month) {
			case 'Jan': {
				$month = 'янв';
				break;
			}
			
			case 'Feb': {
				$month = 'фев';
				break;
			}
			
			case 'Mar': {
				$month = 'мар';
				break;
			}
			
			case 'Apr': {
				$month = 'апр';
				break;
			}
			
			case 'May': {
				$month = 'май';
				break;
			}
			
			case 'Jun': {
				$month = 'июн';
				break;
			}
			
			case 'Jul': {
				$month = 'июл';
				break;
			}
			
			case 'Aug': {
				$month = 'авг';
				break;
			}
			
			case 'Sep': {
				$month = 'сен';
				break;
			}
			
			case 'Oct': {
				$month = 'окт';
				break;
			}
			
			case 'Nov': {
				$month = 'ноя';
				break;
			}
			
			case 'Dec': {
				$month = 'дек';
				break;
			}
		}
		
		return $month;
	}
	
	/*
		Объем файла (размер)
	*/
	function formatFileSize($sizeInBytes) {
		$units = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
		$unit = 0;
		
		while ($sizeInBytes >= 1024 && $unit < count($units) - 1) {
			$sizeInBytes /= 1024;
			$unit++;
		}

		return round($sizeInBytes, 2) . ' ' . $units[$unit];
	}
	
	/*
		Возвращаем байты файла
	*/
	function return_bytes($size_str) {
		switch (substr($size_str, -1)) {
			case 'M': case 'm': return (int)$size_str * 1048576;
			case 'K': case 'k': return (int)$size_str * 1024;
			case 'G': case 'g': return (int)$size_str * 1073741824;
			default: return (int)$size_str;
		}
	}
	
	/*
		Получение аватара пользователя
	*/
	function getAvatar($userid = null) {
		if(empty($userid)) {
			if(empty($_SESSION['id'])) {
				return 'no_image.jpg';
			}
			
			$userid = $_SESSION['id'];
		}
		
		try {
			$dataUser = users()->Get($userid);
			
			if(empty($dataUser)) {
				return 'no_image.jpg';
			}
			
			return $dataUser->image;
		}
		catch(Exception $e) {
			return 'no_image.jpg';
		}
	}
	
	/*
		Получение имени пользователя
	*/
	function getName($userid = null) {
		if(empty($userid)) {
			if(empty($_SESSION['id'])) {
				return getLang('alerts.errors.unknown');
			}
			
			$userid = $_SESSION['id'];
		}
		
		try {
			$dataUser = users()->Get($userid);
			
			if(empty($dataUser)) {
				return getLang('alerts.errors.unknown');
			}
			
			return $dataUser->login;
		}
		catch(Exception $e) {
			return getLang('alerts.errors.unknown');
		}
	}

	function getNameSmall($userid) {
		if(empty($userid)) {
			if(empty($_SESSION['id'])) {
				return 'Н';
			}

			$userid = $_SESSION['id'];
		}

		try {
			$dataUser = users()->Get($userid);

			if(empty($dataUser)) {
				return 'Н';
			}

			return mb_substr($dataUser->first_name, 0, 1) . mb_substr($dataUser->last_name, 0, 1);
		}
		catch(Exception $e) {
			return 'Н';
		}
	}
	
	/*
		Загрузка сторонней страницы
	*/
	function pageLoad($appearance = 'standart', $sample = 'sample', $content = '') {
		tpl()
		->SetAppearance($appearance)
		->Start($sample)
		->Content(tpl()->Get($content))
		->Show();
	}

	/* Загружаем Элемент */
	function getElement($name) {
		return tpl()->Execute(tpl()->Get($name));
	}
	
	/*
		Проверка прав пользователя
	*/
	function isRights($searchRights) {
		if(isset($_SESSION['id'])) {
			$userData = users()->Get($_SESSION['id']);
			
			if(strpos($userData->rights, $searchRights) !== false) {
				return true;
			}
		}
		
		return false;
	}
	
	/*
		Проверяем роль пользователя
	*/
	function isRole($roleId) {
		if(isset($_SESSION['id'])) {
			$dataUser = users()->Get($_SESSION['id']);
			
			if($dataUser->roleid == $roleId) {
				return true;
			}
		}
		
		return false;
	}
	
	/*
		Проверка прав пользователя
	*/
	function isRightsFromUser($searchRights, $userid) {
		$userData = users()->Get($userid);
		
		if(strpos($userData->rights, $searchRights) !== false) {
			return true;
		}
		
		return false;
	}
	
	/*
		Создание и отправка cURL
	*/
	function SendPost($site, $PostFields) {
		$ch = curl_init($site);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $PostFields);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_HEADER, false);
		$result = curl_exec($ch);
		curl_close($ch);

		return $result;
	}
	
	/*
		Получаем страницу
	*/
	function getSend($uri) {
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_URL, $uri);
		curl_setopt($ch, CURLOPT_REFERER, $uri);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
		$result = curl_exec($ch);
		curl_close($ch);
		
		return $result;
	}
	
	/*
		Получаем когда юзер был онлайн
	*/
	function userTimeAgo($userid) {
		$dataUser = users()->Get($userid);
		
		$timeNow = time();
		$diff = $timeNow - $dataUser->date_online;
		
		if($diff < 60) {
			return getLang('userTimeAgo.now');
		}
		elseif ($diff < 3600) {
			$minutes = floor($diff / 60);
			return getLang('userTimeAgo.formate', [$minutes, ($minutes == 1 ? getLang('userTimeAgo.times.minutes.0') : ($minutes < 5 ? getLang('userTimeAgo.times.minutes.1') : getLang('userTimeAgo.times.minutes.2')))]);
		}
		elseif ($diff < 86400) {
			$hours = floor($diff / 3600);
			return getLang('userTimeAgo.formate', [$hours, ($hours == 1 ? getLang('userTimeAgo.times.hours.0') : ($hours < 5 ? getLang('userTimeAgo.times.hours.1') : getLang('userTimeAgo.times.hours.2')))]);
		}
		elseif ($diff < 172800) {
			return getLang('userTimeAgo.yesterday');
		}
		elseif ($diff < 259200) {
			return getLang('userTimeAgo.two_days_ago');
		}
		else {
			$days = floor($diff / 86400);
			return getLang('userTimeAgo.formate', [$days, ($days == 1 ? getLang('userTimeAgo.times.days.0') : ($days < 5 ? getLang('userTimeAgo.times.days.1') : getLang('userTimeAgo.times.days.2')))]);
		}
	}

	/* Получаем сегодня/вчера и т.п. */
	function getTimeAgo($unix_time) {
		$timeNow = time();
		$diff = $timeNow - $unix_time;
		
		if($diff < 60) {
			return getLang('getTimeAgo.now');
		}
		elseif ($diff < 3600) {
			$minutes = floor($diff / 60);
			$result = getLang('getTimeAgo.formate', [
				$minutes,
				($minutes == 1 ? getLang('getTimeAgo.times.minutes.0') : ($minutes < 5 ? getLang('getTimeAgo.times.minutes.1') : getLang('getTimeAgo.times.minutes.2')))
			]);
		}
		elseif ($diff < 86400) {
			$hours = floor($diff / 3600);
			$result = getLang('getTimeAgo.formate', [
				$hours,
				($hours == 1 ? getLang('getTimeAgo.times.hours.0') : ($hours < 5 ? getLang('getTimeAgo.times.hours.1') : getLang('getTimeAgo.times.hours.2')))
			]);
		}
		elseif ($diff < 172800) {
			$result = getLang('getTimeAgo.yesterday', [date('H:i', $unix_time)]);
		}
		elseif ($diff < 259200) {
			$result = getLang('getTimeAgo.two_days_ago', [date('H:i', $unix_time)]);
		}
		else {
			$day = date('d.m.Y', $unix_time);
			$time = date('H:i', $unix_time);
			
			$result = getLang('getTimeAgo.date', [$day, $time]);
		}

		return $result;
	}
	
	/*
		Автосклонение
		$number = 1
		$forms = [ 'предложение', 'предложения', 'предложений' ]
		
		& Result предложение
	*/
	function getDeclension($number, $forms) {
		$number = abs($number) % 100;
		$n1 = $number % 10;
		
		if($number > 10 && $number < 20) {
			return $forms[2];
		}
		
		if($n1 > 1 && $n1 < 5) {
			return $forms[1];
		}
		
		if($n1 == 1) {
			return $forms[0];
		}
		
		return $forms[2];
	}
	
	/*
		Сколько юзер на сайте
		- функция показывает количество часов, дней, недель, месяцев и лет сколько пользователь уже находится на сайте.
	*/
	function timeOnSite($unixTime) {
		$currentTime = time();
		$elapsedTime = $currentTime - $unixTime;

		if($elapsedTime < 3600) {
			return getLang('timeOnSite.0');
		}
		elseif($elapsedTime < 3 * 3600) {
			return getLang('timeOnSite.1');
		}
		elseif($elapsedTime < 12 * 3600) {
			$hours = floor($elapsedTime / 3600);
			return getLang('timeOnSite.2', [$hours, ($hours == 1 ? getLang('getTimeAgo.times.hours.0') : ($hours <= 4 ? getLang('getTimeAgo.times.hours.1') : getLang('getTimeAgo.times.hours.2')))]);
		}
		elseif($elapsedTime < 24 * 3600) {
			$hours = floor($elapsedTime / 3600);
			return getLang('timeOnSite.2', [$hours, ($hours == 1 ? getLang('getTimeAgo.times.hours.0') : ($hours <= 4 ? getLang('getTimeAgo.times.hours.1') : getLang('getTimeAgo.times.hours.2')))]);
		}
		elseif($elapsedTime < 2 * 24 * 3600) {
			return getLang('timeOnSite.3');
		}
		elseif($elapsedTime < 7 * 24 * 3600) {
			$days = floor($elapsedTime / (24 * 3600));
			return getLang('timeOnSite.2', [$days, ($days === 1 ? getLang('getTimeAgo.times.days.0') : ($days <= 4 ? getLang('getTimeAgo.times.days.1') : getLang('getTimeAgo.times.days.2')))]);
		}
		elseif($elapsedTime < 4 * 7 * 24 * 3600) {
			$weeks = floor($elapsedTime / (7 * 24 * 3600));
			return getLang('timeOnSite.2', [$weeks, ($weeks === 1 ? getLang('getTimeAgo.times.weeks.0') : ($weeks <= 4 ? getLang('getTimeAgo.times.weeks.1') : getLang('getTimeAgo.times.weeks.2')))]);
		}
		elseif($elapsedTime < 12 * 30 * 24 * 3600) {
			$months = floor($elapsedTime / (30 * 24 * 3600));
			return getLang('timeOnSite.2', [$months, ($months === 1 ? getLang('getTimeAgo.times.months.0') : ($months <= 4 ? getLang('getTimeAgo.times.months.1') : getLang('getTimeAgo.times.months.2')))]);
		}
		else {
			$years = floor($elapsedTime / (365 * 24 * 3600));
			return getLang('timeOnSite.2', [$years, ($years == 1 ? getLang('getTimeAgo.times.year.0') : getLang('getTimeAgo.times.year.1'))]);
		}
	}

	
	/*
		Генерируем уникальный ID
	*/
	function generateUniqueId($numbers) {
		$hash = md5($numbers);
		$uniqueId = substr($hash, 0, 4) . '-' . substr($hash, 4, 4) . '-' . substr($hash, 8, 4);
		
		return $uniqueId;
	}
	
	/*
		Генерируем API ключи
	*/
	function generateApiKey($length = 32) {
		if($length <= 0 || $length % 2 != 0) {
			throw new InvalidArgumentException('Length must be a positive even number.');
		}
		
		return bin2hex(random_bytes($length / 2));
	}
	
	/*
		Генерируем код подтверждения
	*/
	function generateConfirmationCode($length = 6) {
		return substr(str_shuffle("0123456789"), 0, $length);
	}
	
	/*
		Платежные системы
	*/
	function Merchants() {
		global $Merchants;
		
		if(isset($Merchants)) {
			return $Merchants;
		}
		else {
			$path = __DIR__ . '/merchants.php';
			
			if(file_exists($path)) {
				return include_once($path);
			}
		}
	}
	
	/*
		CallBack
	*/
	function CallBack() {
		global $CallBack;
		
		if(isset($CallBack)) {
			return $CallBack;
		}
		
		$CallBack = new CallBack;
		return $CallBack;
	}
	
	/*
		Удаляем ненужные переменные
	*/
	function removeValuesFromArray(array &$array, array $valuesToRemove) {
		foreach($valuesToRemove as $value) {
			if(isset($array[$value])) {
				unset($array[$value]);
			}
		}
		
		return $array;
	}
	
	/*
		Проверяем на наличие переменных в массиве
	*/
	function validValuesFromArray(array &$array, array $valuesToValid) {
		foreach($valuesToValid as $value) {
			if(empty($array[$value])) {
				return false;
			}
		}
		
		return true;
	}
	
	/*
		Загрузка изображения
	*/
	function uploadImage($uploadFiles, $targetDir = 'uploads/', $watermarkText = false, $callback = null) {
		if(!isset($uploadFiles) || $uploadFiles['error'] !== UPLOAD_ERR_OK) {
			if(is_null($callback)) {
				throw new Exception(getLang('alerts.errors.uploads.image'));
			}
			else {
				call_user_func($callback, ['status' => 'error', 'message' => getLang('alerts.errors.uploads.image')]);
				return;
			}
		}
	
		$allowedTypes = ['image/jpeg', 'image/png', 'image/gif'];
		
		if(!in_array($uploadFiles['type'], $allowedTypes)) {
			if(is_null($callback)) {
				throw new Exception(getLang('alerts.errors.uploads.format'));
			}
			else {
				call_user_func($callback, ['status' => 'error', 'message' => getLang('alerts.errors.uploads.format')]);
				return;
			}
		}
	
		$fopen = finfo_open(FILEINFO_MIME_TYPE);
		$mimeType = finfo_file($fopen, $uploadFiles['tmp_name']);
		finfo_close($fopen);
	
		if(!in_array($mimeType, $allowedTypes)) {
			if(is_null($callback)) {
				throw new Exception(getLang('alerts.errors.uploads.mime'));
			}
			else {
				call_user_func($callback, ['status' => 'error', 'message' => getLang('alerts.errors.uploads.mime')]);
				return;
			}
		}
	
		$filename = uniqid() . '.' . pathinfo($uploadFiles['name'], PATHINFO_EXTENSION);
	
		if(!is_dir($targetDir)) {
			mkdir($targetDir, 0755, true);
		}
	
		$targetFile = $targetDir . basename($filename);
	
		if(move_uploaded_file($uploadFiles['tmp_name'], $targetFile)) {
			if($watermarkText) {
				addWatermark($targetFile, $watermarkText);
			}

			if(is_null($callback)) {
				return $filename;
			}
			else {
				call_user_func($callback, ['status' => 'success', 'filename' => $filename]);
				return;
			}
		}
		else {
			if(is_null($callback)) {
				throw new Exception(getLang('alerts.errors.uploads.moving'));
			}
			else {
				call_user_func($callback, ['status' => 'error', 'message' => getLang('alerts.errors.uploads.moving')]);
				return;
			}
		}
	}
	
	/*
		Загрузка файла
	*/
	function uploadFiles($uploadFiles, $targetDir = 'uploads/') {
		if(!isset($uploadFiles) || $uploadFiles['error'] !== UPLOAD_ERR_OK) {
			throw new Exception(getLang('alerts.errors.uploads.image'));
		}
	
		$maxFileSize = 10 * 1024 * 1024;
		if($uploadFiles['size'] > $maxFileSize) {
			throw new Exception(getLang('alerts.errors.uploads.size'));
		}

		$allowedTypes = [
			'application/pdf',
			'application/vnd.ms-excel',
			'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
			'application/zip',
			'application/x-rar-compressed',
			'application/x-7z-compressed',
			'text/plain',
			'image/jpeg',
			'image/png',
			'image/gif',
			'application/octet-stream'
		];		
	
		$fopen = finfo_open(FILEINFO_MIME_TYPE);
		$mimeType = finfo_file($fopen, $uploadFiles['tmp_name']);
		finfo_close($fopen);
	
		if(!in_array($mimeType, $allowedTypes)) {
			throw new Exception(getLang('alerts.errors.uploads.format'));
		}
	
		if(in_array($mimeType, ['image/jpeg', 'image/png', 'image/gif'])) {
			return uploadImage($uploadFiles, $targetDir);
		}
	
		$filename = uniqid() . '.' . pathinfo($uploadFiles['name'], PATHINFO_EXTENSION);
	
		if(!is_dir($targetDir)) {
			mkdir($targetDir, 0755, true);
		}
	
		$targetFile = $targetDir . basename($filename);
	
		if(move_uploaded_file($uploadFiles['tmp_name'], $targetFile)) {
			return $filename;
		}
		else {
			throw new Exception(getLang('alerts.errors.uploads.moving'));
		}
	}	

	function addWatermark($imagePath, $text) {
		$image = imagecreatefromstring(file_get_contents($imagePath));
		$textColor = imagecolorallocatealpha($image, 255, 255, 255, 63);
		
		$fontPath = $_SERVER['DOCUMENT_ROOT'] . '/assets/system/fonts/Roboto-Bold.ttf';
		$fontSize = 16;
		
		$imageWidth = imagesx($image);
		$imageHeight = imagesy($image);
		
		$bbox = imagettfbbox($fontSize, 0, $fontPath, $text);
		$textWidth = abs($bbox[2] - $bbox[0]);
		$textHeight = abs($bbox[1] - $bbox[7]);
		
		$x = 20;
		$y = $textHeight + 20;
		
		imagettftext($image, $fontSize, 0, $x - 1, $y, $textColor, $fontPath, $text);
		imagettftext($image, $fontSize, 0, $x + 1, $y, $textColor, $fontPath, $text);
		imagettftext($image, $fontSize, 0, $x, $y - 1, $textColor, $fontPath, $text);
		imagettftext($image, $fontSize, 0, $x, $y + 1, $textColor, $fontPath, $text);
		imagettftext($image, $fontSize, 0, $x, $y, $textColor, $fontPath, $text);
		
		imagejpeg($image, $imagePath);
		imagedestroy($image);
	}	

	/*
		Маркет
	*/
	function Market() {
		global $classMarket;
		
		if(empty($classMarket)) {
			$classMarket = new Market;
		}
		
		return $classMarket;
	}

	/*
		Маркет премиум
	*/
	function MarketPremium() {
		global $MarketPremium;

		if(empty($MarketPremium)) {
			$MarketPremium = new MarketPremium;
		}

		return $MarketPremium;
	}
	
	/*
		Создаем функцию маркета подтверджений
	*/
	function MarketVerify() {
		global $MarketVerify;
		
		if(empty($MarketVerify)) {
			$MarketVerify = new MarketVerify;
			return $MarketVerify;
		}
		
		return $MarketVerify;
	}
	
	/* Делим предложение на MetaTags */
	function splitSentenceIntoWords($sentence) {
		$words = preg_split('/[\s,]+/', mb_strtolower($sentence));

		$words = array_filter($words, function($word) {
			return !empty($word);
		});
		
		$result = implode(', ', $words);
		return $result;
	}
	
	/* Get BreadCrumb */
	function getBreadCrumb($arr) {
		return tpl()->getBreadCrumb($arr);
	}
	
	/* Get not ready messages */
	function getRowNotReadyMessages($user_dialog_id = null) {
		$classUsersMessages = new UsersMessages;
		return $classUsersMessages->rowNotReady($user_dialog_id);
	}
	
	/* Connect Telegram bot */
	function connectTelegramBot() {
		$pathBot = $_SERVER['DOCUMENT_ROOT'] . '/../configuration/bots.json';
		
		if(!file_exists($pathBot)) {
			return null;
		}
		
		$dataBot = json_decode(file_get_contents($pathBot), true);
		
		if(json_last_error() !== JSON_ERROR_NONE) {
			return null;
		}
		
		return new Telegram($dataBot['telegram']);
	}
	
	/* Рекурсивное удаление директорий и файлов */
	function deleteDirectory($dir) {
		if(!is_dir($dir)) {
			return false;
		}
		
		$items = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($dir, RecursiveDirectoryIterator::SKIP_DOTS), RecursiveIteratorIterator::CHILD_FIRST);
		
		foreach($items as $item) {
			if($item->getFilename() === '.htaccess') {
				continue;
			}
			
			if($item->isDir()) {
				rmdir($item->getRealPath());
			}
			else {
				unlink($item->getRealPath());
			}
		}
		
		return true;
	}
	
	/* Update cache */
	function updateCache() {
		/* Обновляем кэш в стилях */
		$conf = conf();
		
		$classAdminConfigs = new AdminConfigs;
		$classAdminConfigs->Edit($conf, [
			'cache' => $conf->cache + 1
		]);
		
		$oldPath = $_SERVER['DOCUMENT_ROOT'] . '/../configuration/system.json';
		$newPath = $_SERVER['DOCUMENT_ROOT'] . '/../application/backups/configuration/system_' . date('Y-m-d') . '.json';
		
		if(copy($oldPath, $newPath)) {
			file_put_contents($oldPath, json_encode($conf));
		}
		
		$dirs = [
			$_SERVER['DOCUMENT_ROOT'] . '/assets/admin/css/start.css',
			$_SERVER['DOCUMENT_ROOT'] . '/assets/standart/css/start.css'
		];
		
		for($i = 0; $i < sizeof($dirs); $i++) {
			updateCSSVersion($dirs[$i], $conf->cache + 1);
		}
		
		/* Удаляем директорию кэша шаблонов */
		deleteDirectory($_SERVER['DOCUMENT_ROOT'] . '/../application/cache/appearance/');
	}
	
	/* Обновляем версию CSS */
	function updateCSSVersion($filePath, $newVersion) {
		$cssContent = file_get_contents($filePath);
		
		if($cssContent === false) {
			throw new Exception("Не удалось прочитать файл: $filePath");
		}
		
		$pattern = "/url\('([^']+\.css)\?v=\d+'\)/";
		$replacement = "url('$1?v=$newVersion')";
		$updatedContent = preg_replace($pattern, $replacement, $cssContent);
		
		if($updatedContent === null) {
			throw new Exception(getLang('alerts.errors.update.css.0', [$filePath]));
		}
		
		$result = file_put_contents($filePath, $updatedContent);
		
		if($result === false) {
			throw new Exception(getLang('alerts.errors.update.css.1', [$filePath]));
		}
		
		return true;
	}

	/* Рекурсивное перемещение файлов */
	function moveDirectory($source, $destination) {
		if(!is_dir($source)) {
			throw new Exception(getLang('alerts.errors.moveDirectory.0', [$source]));
		}

		if(!is_dir($destination)) {
			mkdir($destination, 0755, true);
		}

		$dir = opendir($source);
	
		if($dir === false) {
			throw new Exception(getLang('alerts.errors.moveDirectory.1', [$source]));
		}

		while(($file = readdir($dir)) !== false) {
			if($file == '.' || $file == '..') {
				continue;
			}
	
			$srcPath = $source . DIRECTORY_SEPARATOR . $file;
			$dstPath = $destination . DIRECTORY_SEPARATOR . $file;
	
			if(is_dir($srcPath)) {
				moveDirectory($srcPath, $dstPath);
			}
			else {
				if (!rename($srcPath, $dstPath)) {
					throw new Exception(getLang('alerts.errors.moveDirectory.2', [$srcPath]));
				}
			}
		}
	
		closedir($dir);
		rmdir($source);
	}

	/* getNotData */
	function getNotData($text, $table = false, $colspan = 2) {
		if($table) {
			return tpl()->Set(['{text}' => $text, '{colspan}' => $colspan], tpl()->Get('elements/no_data_table'));
		}

		return tpl()->Set(['{text}' => $text], tpl()->Get('elements/no_data'));
	}

	/* Возвращаемся назад */
	function historyBack() {
		die('<script>window.history.back();</script>');
	}

	/* Изменяем формат для Meta Tags */
	function formatMetaTags($text) {
		$temp = htmlspecialchars_decode($text);
		$temp = strip_tags($temp);
		$temp = str_replace('"', "", $temp);
		$temp = str_replace("'", "", $temp);

		return $temp;
	}

	/* Формирование цены */
	function formatPrice($productId) {
		$default = [
			'total' => 0,
			'discount' => 0,
			'oldprice' => 0
		];
	
		try {
			$productData = Product()->Get($productId)['product'];
	
			if (isset($productData['discount']) && $productData['discount'] > 0) {
				$discountAmount = $productData['price'] / 100 * $productData['discount'];
				$total = $productData['price'] - $discountAmount;
	
				$default = [
					'total' => formatPriceValue(isset($total) ? $total : $productData['price']),
					'discount' => $productData['discount'],
					'oldprice' => formatPriceValue($productData['price'])
				];
			}
			else {
				$default['total'] = formatPriceValue($productData['price']);
			}
		}
		finally {
			return (object)$default;
		}
	}
	
	function formatPriceValue($value) {
		if(floor($value) == $value) {
			return number_format($value, 0);
		}

		return number_format($value, 2);
	}

	/* Реформирование стоимости */
	function reFormatPrice($price) {
		$price = str_replace(' ', '', $price);
		$price = str_replace(',', '', $price);
		$price = str_replace('.', '.', $price);
		$price = floatval($price);
		
		return $price;
	}
	

	/* Получение данных категории */
	function getDataCategory($id) {
		$get = Product()->Get($id);

		foreach($get['category'] as $id => $data) {
			return $data;
		}
	}

	/* Получение данных подкатегории */
	function getDataSubcategory($id) {
		$get = Product()->Get($id);

		foreach($get['subcategory'] as $id => $data) {
			return $data;
		}
	}

	/* Noty */
	function Noty() {
		global $Notifications;

		if(empty($Notifications)) {
			$Notifications = new UsersNoty;
		}

		return $Notifications;
	}

	/* Help */
	function Help() {
		global $Help;

		if(empty($Help)) {
			$Help = new Help;
		}

		return $Help;
	}

	/* Получаем языковую модель */
	function getLang($key, $placeholders = []) {
		global $Languages;

		if(empty($Languages)) {
			$Languages = Languages();
		}

		try {
			return $Languages->getText($key, $placeholders);
		}
		catch(Exception $e) {
			return $e->getMessage();
		}
	}

	/* Языковая система */
	function Languages() {
		global $Languages;

		if(empty($Languages)) {
			$Languages = new Languages;
			$Languages->loadFromDir($_SERVER['DOCUMENT_ROOT'] . '/../application/system/languages/');
			$Languages->Set(conf()->language);
		}

		return $Languages;
	}

	/* Формат даты и времени */
	function uDate($unix_time) {
		$day = date('d.m.Y', $unix_time);
		$time = date('H:i', $unix_time);

		return getLang('getTimeAgo.date', [$day, $time]);
	}

	/* Споры и возвраты */
	function Refunds() {
		global $Refunds;

		if(empty($Refunds)) {
			$Refunds = new Refunds;
		}

		return $Refunds;
	}

	/* Получаем TEMP директорию */
	function getTempDir($payback = '') {
		$primaryPath = $_SERVER['DOCUMENT_ROOT'] . '/../application/system/temp';
		$fallbackPath = $_SERVER['DOCUMENT_ROOT'] . '/uploads';
		
		if(!file_exists($primaryPath)) {
			if(!mkdir($primaryPath, 0777, true)) {
				error_log("Failed to create directory: $primaryPath");
			}
		}
	
		if(file_exists($primaryPath) && is_writable($primaryPath)) {
			return $primaryPath . $payback;
		}
	
		if(file_exists($fallbackPath) && is_writable($fallbackPath)) {
			return $fallbackPath . $payback;
		}
	
		throw new Exception("Couldn't find or create a writable temporary directory. Checked paths: $primaryPath, $fallbackPath");
	}
	
	/* Сервер */
	function Server() {
		global $Server;

		if(empty($Server)) {
			$Server = new Server;
		}

		return $Server;
	}

	/* Рекурсивное выполнение PHP скриптов */
	function include_php_files($dir) {
		if(!is_dir($dir)) {
			return;
		}

		$files = scandir($dir);

		foreach($files as $file) {
			if($file === '.' || $file === '..') {
				continue;
			}
	
			$path = $dir . DIRECTORY_SEPARATOR . $file;

			if(is_dir($path)) {
				include_php_files($path);
			}
			elseif (pathinfo($file, PATHINFO_EXTENSION) === 'php') {
				require_once $path;
			}
		}
	}

	/* PHPMailer */
	function PHPMailer() {
		global $PHPMailer;

		if(empty($PHPMailer)) {
			$PHPMailer = new Mailer(conf()->mailer);
		}

		return $PHPMailer;
	}

	/* Удаляем сведения об системе */
	function removeSystemData() {
		$path_data = $_SERVER['DOCUMENT_ROOT'] . '/../configuration/system/data.json';

		if(file_exists($path_data)) {
			return unlink($path_data);
		}

		return true;
	}

	/* Создаем рекурсивно путь */
	function createPath($path, $permissions = 0755) {
		if(is_dir($path)) {
			return true;
		}
		
		if(createPath(dirname($path), $permissions)) {
			return mkdir($path, $permissions);
		}
		
		return false;
	}

	/* Создаем рекурсивно путь для файла */
	function createPathForFile($path, $permissions = 0755) {
		$directory = pathinfo($path, PATHINFO_DIRNAME);
	
		if(is_dir($directory)) {
			return true;
		}
	
		return mkdir($directory, $permissions, true);
	}

	/* Получаем патч до configuration */
	function getPath($path) {
		return $_SERVER['DOCUMENT_ROOT'] . '/../' . $path;
	}

	/* Кастомные языки */
	function get_custom_lang($name) {
		$lang = isset($_SESSION['language']) ? $_SESSION['language'] : conf()->language;
		$data = Languages()->getKeys($lang);

		if(isset($data['custom']['additionally'][$name])) {
			$name = $data['custom']['additionally'][$name];
		}
		elseif(isset($data['custom']['data_auto'][$name])) {
			$name = $data['custom']['data_auto'][$name];
		}
		elseif(isset($data['custom']['subcategory'][$name])) {
			$name = $data['custom']['subcategory'][$name];
		}
		elseif(isset($data['custom']['category'][$name])) {
			$name = $data['custom']['category'][$name];
		}
		elseif(isset($data['filters'][$name])) {
			$name = $data['filters'][$name];
		}
		
		return htmlspecialchars_decode($name);
	}

	/* Reviews */
	function Reviews() {
		global $Reviews;

		if(empty($Reviews)) {
			$Reviews = new Reviews;
		}

		return $Reviews;
	}

	/* Получаем изображение товара */
	function getProductImage($product_id) {
		$productData = Product()->Get($product_id)['product'] ?? null;
		
		if(!$productData || empty($productData['image'])) {
			return 'no_image.jpg';
		}
		
		$path = $_SERVER['DOCUMENT_ROOT'] . '/uploads/images/product/' . $productData['image'];
		return file_exists($path) ? $productData['image'] : 'no_image.jpg';
	}

	// Патч до ассинхрона
	function pathAsynchronous($file = '') {
		$path = __DIR__ . '/../asynchronous/' . $file;

		if(file_exists($path)) {
			return $path;
		}

		return false;
	}