<?PHP
	class Category {
		/* Получаем категорию */
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__category WHERE id=:id OR search_link=:id LIMIT 1');
				$sth->execute([':id' => $id]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/* Формирование списка Подкатегорий */
		public function formateSubcategory($dataCategory) {
			tpl()->ClearCell('formateSubcategory');
			
			$dataThisCategory = $this->Get($dataCategory[0]['category_id']);
			
			tpl()->AddCell('formateSubcategory', tpl()->Set([
				'{id}' => '',
				'{name}' => get_custom_lang($dataThisCategory->name)
			], tpl()->Get('elements/product/category')));
			
			foreach($dataCategory as $data) {
				tpl()->AddCell('formateSubcategory', tpl()->Set([
					'{id}' => $data['id'],
					'{name}' => $data['name'],
					'{category_id}' => $data['category_id'],
					'{search_link}' => isset($data['search_link']) ? $data['search_link'] : null
				], tpl()->Get('elements/product/category')));
			}
			
			return tpl()->Execute(tpl()->GetCell('formateSubcategory'));
		}
		
		/* Получаем все разделы и подразделы */
		public function getAll() {
			try {
				$sth = pdo()->query('SELECT * FROM product__category ORDER BY position ASC');
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('category.no'));
				}
				
				tpl()->ClearCell('listCategory');
				
				while($dataCategory = $sth->fetch(PDO::FETCH_OBJ)) {
					$ath = pdo()->prepare('SELECT * FROM product__subcategory WHERE category_id=:category_id ORDER BY position ASC');
					$ath->execute([':category_id' => $dataCategory->id]);
					
					if(!$ath->rowCount()) {
						continue;
					}
					
					tpl()->ClearCell('listSubcategory');
					
					while($dataSubcategory = $ath->fetch(PDO::FETCH_OBJ)) {
						tpl()->AddCell('listSubcategory', tpl()->Set([
							'{category_id}' => $dataCategory->id,
							'{item_id}' => $dataSubcategory->id,
							'{item_name}' => get_custom_lang($dataSubcategory->name),
							'{item_rows}' => Product()->rowFromSubcategory($dataCategory->id),
							'{search_link}' => isset($dataCategory->search_link) ? $dataCategory->search_link : $dataCategory->id
						], tpl()->Get('elements/market/subcategory/items')));
					}
					
					tpl()->AddCell('listCategory', tpl()->Set([
						'{item_id}' => $dataCategory->id,
						'{item_image}' => $dataCategory->icon,
						'{item_name}' => get_custom_lang($dataCategory->name),
						'{item_subcategories}' => tpl()->GetCell('listSubcategory'),
						'{search_link}' => isset($dataCategory->search_link) ? $dataCategory->search_link : $dataCategory->id
					], tpl()->Get('elements/market/category/items')));
				}
				
				if(!tpl()->IsCell('listCategory')) {
					return getNotData(getLang('category.no'));
				}
				
				return tpl()->Execute(tpl()->GetCell('listCategory'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}

		/* Проверяем валидность Категории */
		public function IsValid($id) {
			if(empty($id)) {
				return false;
			}

			try {
				$sth = pdo()->prepare('SELECT * FROM product__category WHERE id=:id OR search_link=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);

				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}

		/* Получаем все товары */
		public function getProductGUI($id) {
			try {
				if(!$this->IsValid($id)) {
					return getNotData(getLang('category.no'));
				}

				$sth=pdo()->prepare('SELECT * FROM product__subcategory WHERE category_id=:id ORDER BY position ASC');
				$sth->execute([':id'=>$id]);

				if($sth->rowCount() == 0) {
					return getNotData(getLang('category.no'));
				}

				$subcategories = [];
				while($row=$sth->fetch(PDO::FETCH_OBJ)) {
					$subcategories[]=$row->id;
				}

				if(!empty($subcategories)) {
					$productListHtml = (new SubCategory)->getProducts($subcategories);

					tpl()->ClearCell('listData');
					tpl()->AddCell('listData',$productListHtml);

					return tpl()->Execute(tpl()->GetCell('listData'));
				}

				return getNotData(getLang('category.no'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}

		/* Получаем товары по ID подкатегории */
		public function getProductFromIdGUI($id) {
			try {
				$SubCategory = new SubCategory;
				return $SubCategory->getProducts($id);
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}

		/* Получаем подкатегории в видео графики */
		public function getSubcategoryGUI($category_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE category_id=:category_id ORDER BY position ASC');
				$sth->execute([':category_id' => $category_id]);

				if(!$sth->rowCount()) {
					return getNotData(getLang('category.no_subcategory'));
				}

				$dataCategory = $this->Get($category_id);

				tpl()->ClearCell('listSubcategoryGUI');
				tpl()->AddCell('listSubcategoryGUI', tpl()->Set([
					'{category_id}' => $category_id,
					'{subcategory_id}' => null,
					'{subcategory_name}' => getLang('market.all_product'),
					'{search_link}' => isset($dataCategory->search_link) ? $dataCategory->search_link : $category_id
				], tpl()->Get('elements/market/subcategory/s-1')));
				
				while($dataSubcategory = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('listSubcategoryGUI', tpl()->Set([
						'{category_id}' => $category_id,
						'{subcategory_id}' => $dataSubcategory->id,
						'{subcategory_name}' => get_custom_lang($dataSubcategory->name),
						'{search_link}' => isset($dataCategory->search_link) ? $dataCategory->search_link : $category_id
					], tpl()->Get('elements/market/subcategory/s-1')));
				}

				return tpl()->Execute(tpl()->GetCell('listSubcategoryGUI'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}

		/* Форматируем название категории и подкатегорий */
		public function formateNameCategory($category_id, $subcategory_id = null) {
			try {
				$dataCategory = $this->Get($category_id);

				if(empty($dataCategory)) {
					return null;
				}

				tpl()->ClearCell('formateNameCategory');
				tpl()->AddCell('formateNameCategory', tpl()->Set(['{name}' => get_custom_lang($dataCategory->name)], '<li>{name}</li>'));

				if(!is_null($subcategory_id)) {
					$classSubcategory = new SubCategory;
					$dataSubcategory = $classSubcategory->Get($subcategory_id);

					if(empty($dataSubcategory)) {
						return null;
					}

					tpl()->AddCell('formateNameCategory', tpl()->Set(['{name}' => get_custom_lang($dataSubcategory->name)], '<li>{name}</li>'));
				}

				return tpl()->Execute(tpl()->GetCell('formateNameCategory'));
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}

		/* Получаем категорию по её Дополнительных данным */
		public function getAllFromData($data) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__category WHERE data=:data ORDER BY position ASC');
				$sth->execute([':data' => $data]);
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('category.no'));
				}
				
				tpl()->ClearCell('listCategory');
				
				while($dataCategory = $sth->fetch(PDO::FETCH_OBJ)) {
					$ath = pdo()->prepare('SELECT * FROM product__subcategory WHERE category_id=:category_id ORDER BY position ASC');
					$ath->execute([':category_id' => $dataCategory->id]);
					
					if(!$ath->rowCount()) {
						continue;
					}
					
					tpl()->ClearCell('listSubcategory');
					
					while($dataSubcategory = $ath->fetch(PDO::FETCH_OBJ)) {
						tpl()->AddCell('listSubcategory', tpl()->Set([
							'{category_id}' => $dataCategory->id,
							'{item_id}' => $dataSubcategory->id,
							'{item_name}' => get_custom_lang($dataSubcategory->name),
							'{item_rows}' => Product()->rowFromSubcategory($dataCategory->id)
						], tpl()->Get('elements/market/subcategory/items')));
					}
					
					tpl()->AddCell('listCategory', tpl()->Set([
						'{item_id}' => $dataCategory->id,
						'{item_image}' => $dataCategory->icon,
						'{item_name}' => get_custom_lang($dataCategory->name),
						'{item_subcategories}' => tpl()->GetCell('listSubcategory')
					], tpl()->Get('elements/market/category/items')));
				}
				
				if(!tpl()->IsCell('listCategory')) {
					return getNotData(getLang('category.no'));
				}
				
				return tpl()->Execute(tpl()->GetCell('listCategory'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}

		/* Получаем название категории согласно данным */
		public function getFromData($data) {
			try {
				foreach($data as $val) {
					return $val;
				}
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return false;
			}
		}
	}