<?PHP
	class MarketPremium {
		/* Добавляем запись премиум товаров */
		public function Add($productid) {
			try {
				if($this->IsValid($productid)) {
					$this->removeProduct($productid);
				}

				$date_start = time();
				$date_end = $date_start + (conf()->premium->days * 24 * 60 * 60);

				$sth = pdo()->prepare('INSERT INTO product__premium(productid, userid, date_start, date_end) VALUES (:productid, :userid, :date_start, :date_end)');
				$sth->execute([
					':productid' => $productid,
					':userid' => $_SESSION['id'],
					':date_start' => $date_start,
					':date_end' => $date_end
				]);

				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Проверяем наличие рекламы товара */
		public function IsValid($product_id) {
			try {
				if(empty($product_id)) {
					return false;
				}

				$sth = pdo()->prepare('SELECT * FROM `product__premium` WHERE `productid`=:product_id LIMIT 1');
				$sth->execute([':product_id' => $product_id]);

				return (bool)$sth->fetch();
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Удаляем старые записи о бусте */
		public function removeProduct($product_id) {
			try {
				$sth = pdo()->prepare('DELETE FROM `product__premium` WHERE `productid`=:product_id');
				$sth->execute([':product_id' => $product_id]);

				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Получаем запись премиум товара */
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__premium WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);

				if(!$sth->rowCount()) {
					return null;
				}

				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Получаем список товаров для главной страницы */
		public function getList() {
			try {
				$sth = pdo()->query('SELECT * FROM product__premium ORDER BY id DESC');

				if(!$sth->rowCount()) {
					return getNotData(getLang('labels.no_actuals'));
				}

				tpl()->ClearCell('getList');
				$now = time();

				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$data = Product()->Get($row->productid);
					$product = $data['product'];
					
					if($product['enable'] != 1) {
						continue;
					}

					if($row->date_end <= $now) {
						continue;
					}

					$category = getDataCategory($product['id']);
					$subcategory = getDataSubcategory($product['id']);
					$price = formatPrice($product['id']);

					tpl()->AddCell('getList', tpl()->Set([
						'{id}' => $row->id,
						'{name}' => htmlspecialchars_decode($product['name']),
						'{product_id}' => $product['id'],
						'{image}' => $product['image'],
						'{category_name}' => get_custom_lang($category['name']),
						'{category_image}' => $category['icon'],
						'{subcategory_name}' => get_custom_lang($subcategory['name']),
						'{price}' => $price->total,
						'{discount}' => $price->discount,
						'{oldprice}' => $price->oldprice,
						'{seller_id}' => $product['author_id'],
						'{seller_image}' => getAvatar($product['author_id']),
						'{seller_name}' => getName($product['author_id']),
						'{stars_count}' => Reviews()->getStarsProduct($product['id']),
						'{stars_reviews}' => Reviews()->rowProduct($product['id'])
					], tpl()->Get('elements/product/premium')));
				}

				$content = tpl()->GetCell('getList');
				if(!mb_strlen($content)) {
					return getNotData(getLang('labels.no_actuals'));
				}

				return tpl()->Execute($content);
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}

		/* Количество рекламируемых товаров пользователем */
		public function rowCount($userid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__premium WHERE userid=:userid AND date_end > UNIX_TIMESTAMP()');
				$sth->execute([':userid' => $userid]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}

		/* Проверяем бустится ли товар или нет */
		public function GetForProduct($productId) {
			try {
				$sth = pdo()->prepare('SELECT date_end, id FROM product__premium WHERE productid=:productId');
				$sth->execute([':productId' => $productId]);
		
				foreach($sth->fetchAll(PDO::FETCH_OBJ) as $row) {
					if($row->date_end > time()) {
						return [
							'position' => $row->id,
							'date_end' => $row->date_end
						];
					}
				}
		
				return null;
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Получаем кнопку буста товара */
		public function getButton($productId, $type = 1) {
			return match(true) {
				$type == 2 => tpl()->Set([
					'{productId}' => $productId,
					'{text}' => getLang('market.my_product.table.boost.button.up')
				], '<a href="javascript:void(0);" data-premium="{productId}">{text}</a>'),
				default => tpl()->Set([
					'{productId}' => $productId,
					'{text}' => getLang('market.my_product.table.boost.button.label')
				], '<a href="javascript:void(0);" data-premium="{productId}">{text}</a>')
			};
		}
	}