<?PHP
	class Product {
		/* Проверка наличия товара */
		public function IsValid($id) {
			if(!is_numeric($id)) {
				return false;
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM product WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/* Получаем сведения товара */
		public function Get($id) {
			if(is_null($id)) {
				return null;
			}
			
			try {
				/* Получаем продукт */
				$sth = pdo()->prepare('SELECT * FROM product WHERE id=:id LIMIT 1');
				$sth->execute([ ':id' => $id ]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				$data['product'] = $sth->fetch(PDO::FETCH_ASSOC);
				
				/* Получаем подкатегории */
				$dataSubcategories = json_decode($data['product']['subcategories'], true);
				$placeholders = implode(',', array_fill(0, count($dataSubcategories), '?'));
				$sth = pdo()->prepare("SELECT * FROM product__subcategory WHERE id IN ($placeholders)");
				
				foreach($dataSubcategories as $key => $id) {
					$sth->bindValue($key + 1, $id, PDO::PARAM_INT);
				}
				
				$sth->execute();
				$data['subcategory'] = $sth->fetchAll(PDO::FETCH_ASSOC);
				
				/* Получаем категории */
				$categoryIds = array_unique(array_column($data['subcategory'], 'category_id'));
				$sth = pdo()->prepare('SELECT * FROM product__category WHERE id=:id');
				$categoryDetails = [];
				
				foreach($categoryIds as $categoryId) {
					$sth->bindValue(':id', $categoryId, PDO::PARAM_INT);
					$sth->execute();
					$categoryDetails[$categoryId] = $sth->fetch(PDO::FETCH_ASSOC);
				}
				
				$data['category'] = $categoryDetails;
				
				/* Выводим результат */
				return $data;
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/* Проверяем доступность новых товаров */
		public function isNext($start = null) {
			try {
				if(empty($start)) {
					$start = 0;
				}
				
				$limit = (int)conf()->index_count_products;
				$start = $start * $limit;
				
				$sql = 'SELECT * FROM product WHERE enable=1 ORDER BY id DESC LIMIT :limit OFFSET :start';
				$sth = pdo()->prepare($sql);
				$sth->bindValue(':limit', $limit, PDO::PARAM_INT);
				$sth->bindValue(':start', $start, PDO::PARAM_INT);
				$sth->execute();
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/* Получаем список товаров */
		public function getAllGUI($start = null) {
			try {
				if(empty($start)) {
					$start = 0;
				}
				
				$limit = (int)conf()->index_count_products;
				$start = $start * $limit;
				
				$sql = 'SELECT * FROM product WHERE enable=1 ORDER BY id DESC LIMIT :limit OFFSET :start';
				$sth = pdo()->prepare($sql);
				$sth->bindValue(':limit', $limit, PDO::PARAM_INT);
				$sth->bindValue(':start', $start, PDO::PARAM_INT);
				$sth->execute();
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('labels.no_actuals'));
				}
				
				$tpl = new Template;
				$SellerProduct = new SellerProduct;
				$Filters = new Filters;
				
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$data = Product()->Get($row->id);
					$price = formatPrice($row->id);
					$tempFilters = $Filters->FormateData($row->id);

					$tpl->AddCell('listProducts', $tpl->Set([
						'{id}' => $row->id,
						'{image}' => $row->image,
						'{price}' => $price->total,
						'{discount}' => $price->discount,
						'{oldprice}' => $price->oldprice,
						'{name}' => htmlspecialchars_decode($row->name),
						'{author_id}' => $row->author_id,
						'{currency}' => conf()->currency->html,
						'{row_sells}' => $SellerProduct->rowSells($data['product']['author_id']),
						'{date}' => date('d.m.Y', $row->date),
						'{seller_id}' => $data['product']['author_id'],
						'{seller_image}' => getAvatar($data['product']['author_id']),
						'{seller_name}' => getName($data['product']['author_id']),
						'{stars_count}' => Reviews()->getStarsProduct($data['product']['id']),
						'{stars_reviews}' => Reviews()->rowProduct($data['product']['id']),
						'{filters}' => is_null($tempFilters) ? '' : $tempFilters
					], $tpl->Get('elements/product/item')));
				}
				
				return $tpl->Execute($tpl->GetCell('listProducts'));
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}
		
		/* Число доступных товаров */
		public function rowCount() {
			try {
				$sth = pdo()->query('SELECT COUNT(*) AS Total FROM product WHERE enable=1');
				
				if(!$sth->rowCount()) {
					return 0;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ)->Total;
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
		/* Получаем товар по его подкатегории */
		public function rowFromSubcategory($subcategories) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product WHERE JSON_CONTAINS(subcategories, :subcategories) AND enable=1');
				$sth->execute([
					':subcategories' => $subcategories
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
		/* Получаем товар по Автору */
		public function getUserAllGUI($user_id) {
			if(!is_numeric($user_id)) {
				return getNotData(getLang('labels.no_actuals'));
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM product WHERE enable=1 AND author_id=:author_id ORDER BY id DESC');
				$sth->execute([':author_id' => $user_id]);
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('labels.no_actuals'));
				}
				
				$tpl = new Template;
				$SellerProduct = new SellerProduct;
				$Filters = new Filters;

				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$data = Product()->Get($row->id);
					$price = formatPrice($row->id);
					$tempFilters = $Filters->FormateData($row->id);

					$tpl->AddCell('listProducts', $tpl->Set([
						'{id}' => $row->id,
						'{image}' => $row->image,
						'{price}' => $price->total,
						'{discount}' => $price->discount,
						'{oldprice}' => $price->oldprice,
						'{name}' => htmlspecialchars_decode($row->name),
						'{author_id}' => $row->author_id,
						'{currency}' => conf()->currency->html,
						'{row_sells}' => $SellerProduct->rowSells($data['product']['author_id']),
						'{date}' => date('d.m.Y', $row->date),
						'{seller_id}' => $data['product']['author_id'],
						'{seller_image}' => getAvatar($data['product']['author_id']),
						'{seller_name}' => getName($data['product']['author_id']),
						'{stars_count}' => Reviews()->getStarsProduct($data['product']['id']),
						'{stars_reviews}' => Reviews()->rowProduct($data['product']['id']),
						'{filters}' => is_null($tempFilters) ? '' : $tempFilters
					], $tpl->Get('elements/product/item')));
				}
				
				return $tpl->Execute($tpl->GetCell('listProducts'));
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}

		/* Добавляем просмотр */
		public function addViews($id) {
			if(isset($_SESSION['views'][$id])) {
				return true;
			}
			
			$_SESSION['views'][$id] = true;

			try {
				$dataProduct = $this->Get($id);

				if(is_null($dataProduct)) {
					return false;
				}

				$dataProduct['product']['views']++;

				$sth = pdo()->prepare('UPDATE `product` SET views=:views WHERE id=:id LIMIT 1');
				$sth->execute([':views' => $dataProduct['product']['views'], ':id' => $id]);

				return true;
			}
			catch(Exception $e) {
				return false;
			}
		}

		/* Добавляем скриншоты */
		public function addImages($product_id, $files) {
			foreach($files['name'] as $key => $name) {
				$file = [
					'name' => $files['name'][$key],
					'type' => $files['type'][$key],
					'tmp_name' => $files['tmp_name'][$key],
					'error' => $files['error'][$key],
					'size' => $files['size'][$key]
				];
		
				try {
					$uploadedFileName = uploadImage($file, 'uploads/images/product/screenshots/', conf()->site_name);

					$sth = pdo()->prepare('INSERT INTO product__images(productid, userid, filename, date) VALUES (:productid, :userid, :filename, :date)');
					$sth->execute([
						':productid' => $product_id,
						':userid' => $_SESSION['id'],
						':filename' => $uploadedFileName,
						':date' => time()
					]);
				}
				catch (Exception $e) {
					continue;
				}
			}
		}
		
		/* Получаем скриншоты */
		public function getImages($product_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__images WHERE productid=:product_id');
				$sth->execute([':product_id' => $product_id]);

				if(!$sth->rowCount()) {
					return tpl()->Set(['{image}' => getProductImage($product_id)], tpl()->Get('elements/swiper/screenshots'));
				}

				tpl()->ClearCell('getImages');
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$path = $_SERVER['DOCUMENT_ROOT'] . '/uploads/images/product/screenshots/' . $row->filename;

					if(!file_exists($path)) {
						continue;
					}

					tpl()->AddCell('getImages', tpl()->Set([
						'{image}' => 'screenshots/' . $row->filename
					], tpl()->Get('elements/swiper/screenshots')));
				}

				if(!mb_strlen(tpl()->GetCell('getImages'))) {
					return tpl()->Set(['{image}' => getProductImage($product_id)], tpl()->Get('elements/swiper/screenshots'));
				}

				return tpl()->GetCell('getImages');
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}

		/* Удаляем скриншоты */
		public function removeImages($product_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__images WHERE productid=:product_id');
				$sth->execute([':product_id' => $product_id]);

				if(!$sth->rowCount()) {
					return true;
				}

				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$path = $_SERVER['DOCUMENT_ROOT'] . '/uploads/images/product/screenshots/' . $row->filename;

					if(file_exists($path)) {
						unlink($path);
						
						$ath = pdo()->prepare('DELETE FROM product__images WHERE id=:id LIMIT 1');
						$ath->execute([':id' => $row->id]);
					}
				}

				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Количество товаров у пользователя */
		public function rowUserCount($userid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product WHERE author_id=:author_id');
				$sth->execute([':author_id' => $userid]);

				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}

		/* Количество товаров на модерации */
		public function rowUserCountModeration($userid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product WHERE author_id=:author_id AND enable=3');
				$sth->execute([':author_id' => $userid]);

				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}

		/* Количество доступных товаров */
		public function rowUserCountEnabled($userid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product WHERE author_id=:author_id AND enable=1');
				$sth->execute([':author_id' => $userid]);

				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
        /* Формирование количества */
        public function formateQuantity($count) {
            $text = match($count) {
                null => getLang('market.my_product.table.quantity.0'),
                0 => getLang('market.my_product.table.quantity.1'),
                default => getLang('market.my_product.table.quantity.2', $count),
            };
            
            return $text;
        }

		/* Получаем количество продажи товара */
		public function rowSells($product_id) {
			try {
				$sth = pdo()->prepare("SELECT COUNT(*) FROM money__transactions WHERE JSON_UNQUOTE(JSON_EXTRACT(data, '$.product_id')) IN (:product_id) AND status=1");
				$sth->execute([':product_id' => $product_id]);

				return (int)$sth->fetchColumn();
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return 0;
			}
		}

		/*
			Обновляем данные товара
		*/
		public function Update($product_id, $data) {
			try {
				$setPart = [];
				$params = [];
				
				foreach($data as $key => $value) {
					if($value == null) {
						$setPart[] = "`$key` = NULL";
					}
					else {
						$setPart[] = "`$key` = :$key";
						$params[":$key"] = $value;
					}
				}
		
				$sql = "UPDATE `product` SET " . implode(", ", $setPart) . " WHERE `id` = :product_id";
				$params[':product_id'] = $product_id;
				
				$stmt = pdo()->prepare($sql);
				$stmt->execute($params);
		
				return true;
			}
			catch (Exception $e) {
				error_log("Ошибка обновления товара с ID $product_id: " . $e->getMessage());
				return false;
			}
		}

		/* Покупал ли пользователь товар или нет */
		public function isBuyed($user_id, $product_id) {
			try {
				$sth = pdo()->prepare("SELECT * FROM money__transactions WHERE JSON_UNQUOTE(JSON_EXTRACT(data, '$.product_id')) = :product_id AND type = 3 AND user_id=:user_id LIMIT 1");
				$sth->execute([
					':user_id' => $user_id,
					':product_id' => $product_id
				]);

				return ($sth->rowCount() == 0) ? false : true;
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return false;
			}
		}

		/* Получаем товар по транзакции */
		public function getTransFromId($trans_id) {
			try {
				$sth = pdo()->prepare("
					SELECT 
						JSON_EXTRACT(data, '$.product_id') AS product_id,
						user_id
					FROM `money__transactions`
					WHERE `id` = :trans_id
					LIMIT 1
				");
				$sth->execute([ ':trans_id' => $trans_id ]);
				
				$result = $sth->fetch(PDO::FETCH_ASSOC);
				return $result ? $result : null;
			}
			catch(Exception $e) {
				error_log('Error in getTransFromId: ' . $e->getMessage());
				return null;
			}
		}

		/* Получаем количество товара */
		public function getQuantity($product_id) {
			try {
				$sth = pdo()->prepare('SELECT `data_auto`, `quantity` FROM `product` WHERE `id` = :product_id LIMIT 1');
				$sth->execute([':product_id' => $product_id]);
				$row = $sth->fetch(PDO::FETCH_OBJ);

				if(!$row) {
					return 0;
				}

				$dataAuto = is_null($row->data_auto) ? null : json_decode($row->data_auto, true);
				if(json_last_error() != JSON_ERROR_NONE) {
					$dataAuto = null;
				}

				return is_null($dataAuto) 
					? (!is_null($row->quantity) ? $row->quantity : 0) 
					: (is_array($dataAuto) ? count($dataAuto) : 0);
			}
			catch (Exception $e) {
				error_log('Error in getQuantity: ' . $e->getMessage());
				return 0;
			}
		}

		/* Получаем тип товара (авто-выдача или ручная) */
		public function getTypeDelivery($product_id) {
			try {
				$sth = pdo()->prepare('SELECT `data_auto`, `quantity` FROM `product` WHERE `id` = :product_id LIMIT 1');
				$sth->execute([':product_id' => $product_id]);
				$row = $sth->fetch(PDO::FETCH_OBJ);

				if(!$row) {
					return null;
				}

				return is_null($row->data_auto) ? 'manual' : 'auto';
			}
			catch (Exception $e) {
				error_log('Error in getQuantity: ' . $e->getMessage());
				return null;
			}
		}
	}