<?PHP
	/*
		Доступ к Refunds имеет флаг: 
	*/

	class Refunds {
		var $status = [];

		public function __construct() {
			$this->status = [
				0 => ['label' => getLang('refunds.status.0')],
				1 => ['label' => getLang('refunds.status.1')],
				2 => ['label' => getLang('refunds.status.2')],
				3 => ['label' => getLang('refunds.status.3')]
			];
		}

		/* Получаем количество возвратов по товару */
		public function rowCount($product_id) {
			try {
				$sql = "SELECT COUNT(*) AS total_refunds
					FROM refunds
					JOIN money__transactions ON refunds.transaction_id = money__transactions.id
					WHERE JSON_EXTRACT(money__transactions.data, '$.product_id') = :product_id;";
				$sth = pdo()->prepare($sql);
				$sth->execute([
					':product_id' => $product_id
				]);

				return (int)$sth->fetchColumn();
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return 0;
			}
		}

		/* Установка статуса */
		public function SetStatus($id, $status) {
			try {
				$sth = pdo()->prepare('UPDATE refunds SET status=:status WHERE id=:id LIMIT 1');
				$sth->execute([':status' => $status, ':id' => $id]);

				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Проверяем доступен ли спор */
		public function IsAvailable($id) {
			try {
				$data = $this->Get($id);

				switch($data->status) {
					case 2: {
						return true;
						break;
					}
				}

				return false;
			}
			catch(Exception $e) {
				return false;
			}
		}

		/* Проверяем наличие спора */
		public function IsValid($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);

				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}

		/* Проверяем права доступа, для топика */
		public function IsAccess($refundId, $userId) {
			try {
				$data = $this->Get($refundId);

				if($data->user_id == $userId || $data->defendant_id == $userId) {
					return true;
				}

				return isRights('d');
			}
			catch(Exception $e) {
				return false;
			}
		}		

		/*
			Создаем спор или возврат средств
		*/
		public function Create($_ARRAY = []) {
			if(empty($_ARRAY['defendant_id'])) {
				throw new Exception(getLang('library.refunds.create.select_defendant_id'));
			}
			
			if(!users()->isValid($_ARRAY['defendant_id'])) {
				throw new Exception(getLang('library.refunds.create.select_not_register'));
			}
			
			if(empty($_ARRAY['transaction_id']) AND empty($_ARRAY['price'])) {
				throw new Exception(getLang('alerts.errors.inputs'));
			}
			
			try {
				if(isset($_ARRAY['transaction_id'])) {
					$classTransactions = new UsersTransactions;
					$dataTransaction = $classTransactions->Get($_ARRAY['transaction_id']);
					
					if(empty($dataTransaction) OR $dataTransaction->user_id != $_SESSION['id']) {
						throw new Exception(getLang('library.refunds.create.trans_not_exists'));
					}
				}
				
				$sth = pdo()->prepare('INSERT INTO refunds(user_id, defendant_id, transaction_id, price, date) VALUES (:user_id, :defendant_id, :transaction_id, :price, :date)');
				$sth->execute([
					':user_id' => $_SESSION['id'],
					':defendant_id' => $_ARRAY['defendant_id'],
					':transaction_id' => isset($_ARRAY['transaction_id']) ? $_ARRAY['transaction_id'] : null,
					':price' => isset($dataTransaction) ? $dataTransaction->amount : $_ARRAY['price'],
					':date' => time()
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Получение визуальных споров и возвратов
		*/
		public function getListGUI($user_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds WHERE user_id=:user_id OR defendant_id=:user_id ORDER BY id DESC');
				$sth->execute([':user_id' => $user_id]);
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('elements.refunds.no_data'));
				}
				
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('listRefunds', tpl()->Set([
						'{id}' => $row->id,
						'{user_id}' => $row->user_id,
						'{user_name}' => getName($row->user_id),
						'{defendant_id}' => $row->defendant_id,
						'{defendant_name}' => getName($row->defendant_id),
						'{price}' => $row->price,
						'{status}' => $row->status,
						'{date}' => date('d.m.Y', $row->date),
					], tpl()->Get('elements/refunds/item')));
				}
				
				return tpl()->Execute(tpl()->GetCell('listRefunds'));
			}
			catch(Exception $e) {
				return tpl()->Execute(tpl()->Get('elements/refunds/no_data'));
			}
		}

		/*
			Количество споров у продавца
		*/
		public function rowCountForSeller($userid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds WHERE defendant_id=:userid');
				$sth->execute(['userid' => $userid]);

				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}

		/*
			Количество возвратов
		*/
		public function rowCountForSellerSuccess($userid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds WHERE defendant_id=:userid AND status=1');
				$sth->execute(['userid' => $userid]);

				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}

		/* Получаем список дел */
		public function getAll() {
			try {
				$sth = pdo()->query('SELECT * FROM `refunds` ORDER BY id DESC');
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('refunds.no_data'), true, 6);
				}

				tpl()->ClearCell('getAll');
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('getAll', tpl()->Set([
						'{id}' => $row->id,
						'{date}' => getTimeAgo($row->date),
						'{plaintiff_id}' => $row->user_id,
						'{plaintiff_name}' => getName($row->user_id),
						'{defendant_id}' => $row->defendant_id,
						'{defendant_name}' => getName($row->defendant_id),
						'{price}' => $row->price,
						'{decision}' => $this->status[$row->status]['label'],
						'{status_id}' => $row->status
					], tpl()->Get('elements/refunds/index/item')));
				}

				return tpl()->Execute(tpl()->GetCell('getAll'));
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}

		/* Получаем дело */
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);

				if(!$sth->rowCount()) {
					return null;
				}

				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Получаем количество активных записей */
		public function getCount() {
			try {
				$sth = pdo()->query('SELECT COUNT(CASE WHEN status = 0 THEN 1 END) AS `0`, COUNT(CASE WHEN status = 1 THEN 1 END) AS `1`, COUNT(CASE WHEN status = 2 THEN 1 END) AS `2`, COUNT(CASE WHEN status = 3 THEN 1 END) AS `3`, COUNT(*) AS total FROM `refunds`');
				
				if(!$sth->rowCount()) {
					return [0, 0, 0, 0, 'total' => 0];
				}

				return $sth->fetch(PDO::FETCH_ASSOC);
			}
			catch(Exception $e) {
				return [0, 0, 0, 0, 'total' => 0];
			}
		}

		/* Получаем количество возвращенных средств */
		public function getCountPrice() {
			$count = 0;

			try {
				$sth = pdo()->query('SELECT * FROM refunds WHERE status=1');

				if(!$sth->rowCount()) {
					return $count;
				}

				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$count += $row->price;
				}

				return $count;
			}
			catch(Exception $e) {
				return $count;
			}
		}

		/* Проверяем ведутся ли споры по данной покупке */
		public function IsAction($trans_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds WHERE transaction_id=:transaction_id ORDER BY id DESC LIMIT 1');
				$sth->execute([':transaction_id' => $trans_id]);

				if(!$sth->rowCount()) {
					return false;
				}

				$row = $sth->fetch(PDO::FETCH_OBJ);

				switch($row->status) {
					case 0: case 1: case 3: {
						return false;
						break;
					}
				}

				return true;
			}
			catch(Exception $e) {
				return false;
			}
		}

		/* Добавляем блокировку на баланс */
		public function BlockBalance($sellerId, $transId) {
			$User = users()->Get($sellerId);

			if(!is_null($User->balance_frozen)) {
				$data = json_decode($User->balance_frozen, true);
		
				foreach($data as $key => $value) {
					if($value['trans_id'] == $transId) {
						$data[$key]['dispute'] = true;
						break;
					}
				}

				$sth = pdo()->prepare('UPDATE users SET balance_frozen=:balance_frozen WHERE id=:id LIMIT 1');
				$sth->execute([
					':balance_frozen' => json_encode($data),
					':id' => $User->id
				]);
			}
		}

		/* Снимаем блокировку с баланса */
		public function UnblockBalance($sellerId, $transId) {
			$User = users()->Get($sellerId);

			if(!is_null($User->balance_frozen)) {
				$data = json_decode($User->balance_frozen, true);
		
				foreach($data as $key => $value) {
					if($value['trans_id'] == $transId) {
						unset($data[$key]['dispute']);
						break;
					}
				}

				$sth = pdo()->prepare('UPDATE users SET balance_frozen=:balance_frozen WHERE id=:id LIMIT 1');
				$sth->execute([
					':balance_frozen' => json_encode($data),
					':id' => $User->id
				]);
			}
		}

		/* Узнаем, был ли спор положительным по транзакции */
		public function IsPositively($trans_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds WHERE transaction_id=:transaction_id ORDER BY id DESC LIMIT 1');
				$sth->execute([':transaction_id' => $trans_id]);

				if(!$sth->rowCount()) {
					return false;
				}

				$row = $sth->fetch(PDO::FETCH_OBJ);

				switch($row->status) {
					case 1: {
						return true;
					}
				}

				return false;
			}
			catch(Exception $e) {
				return false;
			}
		}
	}