<?PHP
	class RefundsMessages {
		/* Отправление сообщения */
		public function Send($id, $message, $files = null) {
			try {
				$sth = pdo()->prepare('INSERT INTO `refunds__messages`(`refund_id`, `user_id`, `message`, `filename`, `date`) VALUES (:refund_id, :user_id, :message, :filename, :date)');
				$sth->execute([
					':refund_id' => $id,
					':user_id' => $_SESSION['id'],
					':message' => (is_null($message) || (mb_strlen($message) <= 0)) ? null : htmlspecialchars($message),
					':filename' => null,
					':date' => time()
				]);
				
				$lastInsertId = pdo()->lastInsertId();

				if(isset($files)) {
					try {
						$path = 'uploads/refunds/' . $id . '/';
						$filename = uploadFiles($files, $path);

						$sth = pdo()->prepare('UPDATE refunds__messages SET filename=:filename WHERE id=:id LIMIT 1');
						$sth->execute([':filename' => $filename, ':id' => $lastInsertId]);
					}
					catch(Exception $e) {
						//Игнорируем ошибку
					}
				}

				return $lastInsertId;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Отправка системного сообщения */
		public function SendSystem($id, $message) {
			try {
				$sth = pdo()->prepare('INSERT INTO `refunds__messages`(`refund_id`, `user_id`, `message`, `filename`, `date`) VALUES (:refund_id, :user_id, :message, :filename, :date)');
				$sth->execute([
					':refund_id' => $id,
					':user_id' => 0,
					':message' => htmlspecialchars($message),
					':filename' => null,
					':date' => time()
				]);
				
				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Получаем сообщение */
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds__messages WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);

				if(!$sth->rowCount()) {
					return null;
				}

				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Получаем историю чата */
		public function getList($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM refunds__messages WHERE refund_id=:refund_id ORDER BY id DESC');
				$sth->execute([':refund_id' => $id]);

				if(!$sth->rowCount()) {
					return getNotData(getLang('refunds.no_data'));
				}

				$Refunds = new Refunds;
				
				tpl()->ClearCell('getList');
				$count_messages = 0;
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					if($row->user_id == 0) {
						$data = $Refunds->Get($row->refund_id);
						tpl()->AddCell('getList', tpl()->Set([
							'{id}' => $row->id,
							'{message}' => ($data->status == 1) ? getLang('refunds.library.messages.system.confirm', htmlspecialchars_decode($row->message)) : getLang('refunds.library.messages.system.rejection', htmlspecialchars_decode($row->message)),
							'{date}' => uDate($row->date)
						], tpl()->Get('elements/refunds/home/message_system')));
					}
					else {
						if(empty($_SESSION['id']) || !Refunds()->IsAccess($row->refund_id, $_SESSION['id'])) {
							continue;
						}						
						
						$count_messages++;

						if(isset($row->message) && mb_strlen($row->message) >= 1) {
							$message = htmlspecialchars_decode($row->message);
						}
						else {
							$message = null;
						}
	
						tpl()->AddCell('getList', tpl()->Set([
							'{id}' => $row->id,
							'{profile_id}' => $row->user_id,
							'{profile_name}' => getName($row->user_id),
							'{profile_image}' => getAvatar($row->user_id),
							'{message}' => $message,
							'{date}' => uDate($row->date),
							'{is_message}' => (isset($row->message) && (mb_strlen(htmlspecialchars_decode($row->message) >= 1))) ? 'yes' : 'no',
							'{is_file}' => isset($row->filename) ? 'yes' : 'no',
							'{filename}' => isset($row->filename) ? $row->filename : null,
							'{refund_id}' => $row->refund_id,
							'{profile_category}' => $this->getTypeUser($row->refund_id, $row->user_id)
						], tpl()->Get('elements/refunds/home/message')));
					}
				}

				if($count_messages == 0) {
					tpl()->AddCell('getList', getLang('refunds.message_no_access'));
				}

				return tpl()->Execute(tpl()->GetCell('getList'));
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}

		/* Получаем тип пользователя (категория: истец, ответчик) */
		public function getTypeUser($refund_id, $user_id) {
			try {
				$Refunds = new Refunds;
				$Refunds = $Refunds->Get($refund_id);
				
				if(empty($Refunds)) {
					return getLang('system.unknown');
				}

				if($Refunds->defendant_id == $user_id) {
					return getLang('refunds.defendant');
				}
				elseif($Refunds->user_id == $user_id) {
					return getLang('refunds.plaintiff');
				}
				
				$User = users()->Get($user_id);
				$UsersRole = new UsersRole;

				return $UsersRole->getName($User->roleid);
			}
			catch(Exception $e) {

			}
		}
	}