<?PHP
	class Reviews {
		/* Получаем рейтинг пользователя */
		public function getStars($user_id) {
			try {
				$sth = pdo()->prepare('SELECT `stars` FROM `users__reviews` WHERE `whom_id`=:whom_id');
				$sth->execute([
					':whom_id' => $user_id
				]);

				if($sth->rowCount() == 0) {
					return number_format(5, 1, '.', '');
				}

				$stars = $sth->fetchAll(PDO::FETCH_COLUMN, 0);

				$totalStars = array_sum($stars);
				$averageStars = $totalStars / count($stars);

				return number_format($averageStars, 1, '.', '');
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return number_format(5, 1, '.', '');
			}
		}

		/* Получаем рейтинг товара */
		public function getStarsProduct($product_id) {
			try {
				$sth = pdo()->prepare('SELECT `stars` FROM `users__reviews` WHERE `product_id`=:product_id');
				$sth->execute([
					':product_id' => $product_id
				]);

				if($sth->rowCount() == 0) {
					return number_format(5, 1, '.', '');
				}

				$stars = $sth->fetchAll(PDO::FETCH_COLUMN, 0);

				$totalStars = array_sum($stars);
				$averageStars = $totalStars / count($stars);

				return number_format($averageStars, 1, '.', '');
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return number_format(5, 1, '.', '');
			}
		}

		/* Рендерим звезды */
		public function renderStars($rating) {
			$fullStars = floor($rating);
			$halfStar = ($rating - $fullStars) >= 0.5;
			$emptyStars = 5 - $fullStars - ($halfStar ? 1 : 0);
			
			$html = '';
			for($i = 0; $i < $fullStars; $i++) {
				$html .= "<i class='bx bxs-star'></i>";
			}

			if($halfStar) {
				$html .= "<i class='bx bxs-star-half'></i>";
			}

			for($i = 0; $i < $emptyStars; $i++) {
				$html .= "<i class='bx bx-star'></i>";
			}

			return $html;
		}

		/* Добавляем отзыв о товаре */
		public function addProduct($product_id, $stars = 5, $message = null) {
			try {
				$dataProduct = Product()->Get($product_id);

				if(empty($dataProduct['product'])) {
					throw new Exception('Товар не найден');
				}

				$Product = $dataProduct['product'];
				unset($dataProduct);

				$sth = pdo()->prepare('INSERT INTO `users__reviews`(`user_id`, `whom_id`, `product_id`, `message`, `stars`, `date`) VALUES (:user_id, :whom_id, :product_id, :message, :stars, :date)');
				$sth->execute([
					':user_id' => $_SESSION['id'],
					':whom_id' => $Product['author_id'],
					':product_id' => $product_id,
					':message' => isset($message) ? htmlspecialchars($message) : $message,
					':stars' => $stars,
					':date' => time()
				]);

				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return false;
			}
		}
		
		/* Проверяем оставлил ли пользователь отзыв данному товару */
		public function isProduct($product_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM `users__reviews` WHERE `user_id`=:user_id AND `product_id`=:product_id LIMIT 1');
				$sth->execute([
					':user_id' => $_SESSION['id'],
					':product_id' => $product_id
				]);

				if($sth->rowCount() == 0) {
					return false;
				}

				return true;
			}
			catch(Exception $e) {
				return true;
			}
		}

		/* Получаем количество отзывов продавца */
		public function rowUser($whom_id) {
			try {
				$sth = pdo()->prepare('SELECT COUNT(*) FROM `users__reviews` WHERE `whom_id`=:whom_id');
				$sth->execute([
					':whom_id' => $whom_id
				]);

				return (int)$sth->fetchColumn();
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return 0;
			}
		}

		/* Получаем количество отзывов на продукте */
		public function rowProduct($product_id) {
			try {
				$sth = pdo()->prepare('SELECT COUNT(*) FROM `users__reviews` WHERE `product_id`=:product_id');
				$sth->execute([
					':product_id' => $product_id
				]);

				return (int)$sth->fetchColumn();
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return 0;
			}
		}

		/* Получаем отзывы за товар */
		public function getListProduct($product_id, int $limit = 3) {
			try {
				$sth = pdo()->prepare('SELECT * FROM `users__reviews` WHERE `product_id`=:product_id ORDER BY `id` DESC LIMIT :limit');
				$sth->bindValue(':product_id', $product_id, PDO::PARAM_INT);
				$sth->bindValue(':limit', $limit, PDO::PARAM_INT);
				$sth->execute();				

				if($sth->rowCount() == 0) {
					return null;
				}

				$Category = new Category;

				tpl()->ClearCell('getListProduct');
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$Product = Product()->Get($product_id);
					$data = $Category->getFromData($Product['category']);

					tpl()->AddCell('getListProduct', tpl()->Set([
						'{review_id}' => $row->id,
						'{review_content}' => htmlspecialchars_decode($row->message),
						'{review_date}' => date('d.m.Y H:i', $row->date),
						'{review_stars}' => $this->renderStars((float)$row->stars),
						'{review_stars_count}' => number_format($row->stars, 1),
						'{user_id}' => $row->user_id,
						'{user_name}' => getName($row->user_id),
						'{user_avatar}' => getAvatar($row->user_id),
						'{review_category}' => isset($data['name']) ? get_custom_lang($data['name']) : '',
						'{search_link}' => isset($data['search_link']) ? $data['search_link'] : $data['id']
					], tpl()->Get('elements/reviews/product')));
				}
				
				return tpl()->Execute(
					tpl()->GetCell('getListProduct')
				);
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return null;
			}
		}

		/* Узнаем, можем мы ли оставить комментарий под товаром */
		public function hasUserReviewedProduct($user_id, $product_id) {
			try {
				if (!Product()->isBuyed($user_id, $product_id)) {
					return false;
				}
				
				$sth = pdo()->prepare('SELECT COUNT(*) FROM `users__reviews` WHERE `user_id`=:user_id AND `product_id`=:product_id LIMIT 1');
				$sth->execute([
					':user_id' => $user_id,
					':product_id' => $product_id
				]);
				
				return $sth->fetchColumn() == 0;
			} catch (Exception $e) {
				error_log('Error in hasUserReviewedProduct: ' . $e->getMessage());
				return false;
			}
		}		
	}