<?PHP
	class SubCategory {
		/* Проверяем существует каталог или нет */
		public function IsValid($id) {
			if(!is_numeric($id)) {
				return false;
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Получаем товары по ID каталога
		*/
		public function getProducts($subcategories) {
			try {
				if (!is_array($subcategories)) {
					$subcategories = [$subcategories];
				}
		
				$subcategories = array_map('intval', array_unique($subcategories));
				if (empty($subcategories)) {
					return getNotData(getLang('labels.no_actuals'));
				}
				
				$queryParts = [];
				foreach ($subcategories as $subcategory) {
					$queryParts[] = 'JSON_CONTAINS(subcategories,"' . $subcategory . '")';
				}
				
				$query = 'SELECT * FROM product WHERE enable=1 AND (' . implode(' OR ', $queryParts) . ') ORDER BY id DESC';
				$sth = pdo()->prepare($query);
				$sth->execute();
				
				if ($sth->rowCount() == 0) {
					return getNotData(getLang('labels.no_actuals'));
				}
				
				$products = [];
				while ($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$products[] = [
						'info' => $row,
						'premium' => MarketPremium()->GetForProduct($row->id)
					];
				}
		
				usort($products, function ($a, $b) {
					$aPremium = $a['premium'];
					$bPremium = $b['premium'];
				
					// Сортировка по наличию премиум
					$comparePremium = ($bPremium != null) <=> ($aPremium != null);
					if ($comparePremium != 0) {
						return $comparePremium;
					}
		
					// Сортировка премиум по position
					if ($aPremium != null && $bPremium != null) {
						return $bPremium['position'] <=> $aPremium['position'];
					}
				
					// Сортировка остальных по id (по убыванию)
					return $b['info']->id <=> $a['info']->id;
				});
		
				$SellerProduct = new SellerProduct;
				$Filters = new Filters;

				tpl()->ClearCell('listProducts');
				foreach ($products as $data) {
					$productData = Product()->Get($data['info']->id);
					$productPrice = formatPrice($data['info']->id);
					$tempFilters = $Filters->FormateData($data['info']->id);
		
					tpl()->AddCell('listProducts', tpl()->Set([
						'{id}' => $data['info']->id,
						'{image}' => $data['info']->image,
						'{price}' => $productPrice->total,
						'{discount}' => $productPrice->discount,
						'{oldprice}' => $productPrice->oldprice,
						'{name}' => htmlspecialchars_decode($data['info']->name),
						'{author_id}' => $data['info']->author_id,
						'{row_sells}' => $SellerProduct->rowSells($data['info']->author_id),
						'{date}' => date('d.m.Y', $data['info']->date),
						'{seller_id}' => $productData['product']['author_id'],
						'{seller_image}' => getAvatar($productData['product']['author_id']),
						'{seller_name}' => getName($productData['product']['author_id']),
						'{stars_count}' => Reviews()->getStarsProduct($productData['product']['id']),
						'{stars_reviews}' => Reviews()->rowProduct($productData['product']['id']),
						'{filters}' => is_null($tempFilters) ? '' : $tempFilters
					], tpl()->Get('elements/product/item')));
				}
		
				return tpl()->Execute(tpl()->GetCell('listProducts'));
			} catch (Exception $e) {
				return $e->getMessage();
			}
		}
		
		
		/*
			Получаем подкатегорию
		*/
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
	}