<?PHP
    class SellerIncome {
        private $SellerProduct;
        private $UsersTransactions;

        public function __construct() {
            $this->SellerProduct = new SellerProduct;
            $this->UsersTransactions = new UsersTransactions;
        }

        /* Добавляем прибыль */
        public function Add($user_id, $trans_id, $price) {
            try {
                $sth = pdo()->prepare('INSERT INTO users__income(trans_id, user_id, price, date) VALUES (:trans_id, :user_id, :price, :date)');
                $sth->execute([
                    ':trans_id' => $trans_id,
                    ':user_id' => $user_id,
                    ':price' => $price,
                    ':date' => time()
                ]);
            }
            catch(Exception $e) {
                throw $e;
            }
        }

        /* Проверка на заморозку */
        public function isFrozen($user_id, $trans_id) {
            try {
                $user = Users()->Get($user_id);
                $frozen_balance = $user->balance_frozen;

                if(is_null($frozen_balance)) {
                    return false;
                }

                $frozen_balance = json_decode($frozen_balance, true);

                foreach($frozen_balance as $key => $value) {
                    if($value['trans_id'] == $trans_id) {
                        return true;
                    }
                }

                return false;
            }
            catch(Exception $e) {
                return true;
            }
        }

        /* Получаем список продаж */
        public function getList($userid) {
            try {
                $productList = $this->SellerProduct->formatProducts($userid);

                if(!$productList) {
                    return getNotData(getLang('seller.product.no'), true, 7);
                }

                $productIds = array_column($productList, 'id');
                $sth = pdo()->prepare('SELECT * FROM money__transactions WHERE type=:type ORDER BY id DESC');
                $sth->execute([':type' => 3]);

                if(!$sth->rowCount()) {
                    return getNotData(getLang('seller.product.no_exists'), true, 7);
                }

                $transactions = $sth->fetchAll(PDO::FETCH_ASSOC);
                $UsersBalance = new UsersBalance;

                tpl()->ClearCell('getList');
                
                foreach($transactions as $transaction) {
                    $data = json_decode($transaction['data'], true);

                    if(isset($data['product_id']) && in_array($data['product_id'], $productIds)) {
                        $product = Product()->Get($data['product_id'])['product'];
                        $getFrozenFromTransId = $UsersBalance->getFrozenFromTransId($userid, $transaction['id']);

                        if(isset($getFrozenFromTransId['dispute']) && $getFrozenFromTransId['dispute'] == true) {
                            $statusLabel = $this->UsersTransactions->dataStatus[5]['name'];
                            $statusId = 'arbitration';
                        }elseif(isset($getFrozenFromTransId)) {
                            $statusLabel = getLang('seller.income.wait_confirm');
                            $statusId = 'waiting-confirm';
                        }
                        elseif(Refunds()->IsPositively($transaction['id'])) {
                            $statusLabel = $this->UsersTransactions->dataStatus[4]['name'];
                            $statusId = 'arbitration';
                        }
                        else {
                            $statusLabel = $this->UsersTransactions->dataStatus[$transaction['status']]['name'];
                            $statusId = $transaction['status'];
                        }

                        $auto_closed = $UsersBalance->getFrozenEndTime($getFrozenFromTransId);

                        tpl()->AddCell('getList', tpl()->Set([
                            '{id}' => $transaction['id'],
                            '{productId}' => $product['id'],
                            '{productName}' => htmlspecialchars_decode($product['name']),
                            '{price}' => $transaction['amount'],
                            '{buyer}' => getName($transaction['user_id']),
                            '{buyerId}' => $transaction['user_id'],
                            '{statusId}' => $statusId,
                            '{statusLabel}' => $statusLabel,
                            '{date}' => getTimeAgo($transaction['date']),
                            '{auto_closed}' => $auto_closed['label'],
                            '{auto_closed_class}' => $auto_closed['class']
                        ], tpl()->Get('elements/table/income/item')));
                    }
                }

                return tpl()->Execute(tpl()->GetCell('getList'));
            }
            catch(Exception $e) {
                throw $e;
            }
        }

        /* Проверяем наличие сделки между пользователями */
        public function isDeal($user_id) {
            try {
                $user = users()->Get($user_id);

                if(is_null($user->balance_frozen)) {
                    return false;
                }
                
                $frozen_balance = json_decode($user->balance_frozen, true);

                foreach($frozen_balance as $key => $value) {
                    $sth = pdo()->prepare('SELECT * FROM money__transactions WHERE id=:id AND user_id=:user_id AND type=3 LIMIT 1');
                    $sth->execute([
                        ':id' => $value['trans_id'],
                        ':user_id' => $_SESSION['id']
                    ]);

                    if($sth->rowCount()) {
                        return true;
                    }
                }

                return false;
            }
            catch(Exception $e) {
                return false;
            }
        }

        /* Получаем сделки между покупателем и продавцом */
        public function getDeals($user_id) {
            try {
                $user = users()->Get($user_id);

                if(is_null($user->balance_frozen)) {
                    return false;
                }

                $data = json_decode($user->balance_frozen, true);

                foreach($data as $key => $value) {
                    $sth = pdo()->prepare('SELECT * FROM money__transactions WHERE id=:id AND user_id=:user_id AND type=3 LIMIT 1');
                    $sth->execute([
                        ':id' => $value['trans_id'],
                        ':user_id' => $_SESSION['id']
                    ]);

                    if(!$sth->rowCount()) {
                        continue;
                    }

                    $row = $sth->fetch(PDO::FETCH_OBJ);
                    
                    if(is_null($row->data)) {
                        continue;
                    }

                    $trans_data = json_decode($row->data, true);
                    $product_data = Product()->Get($trans_data['product_id']);
                    $product = $product_data['product'];

                    tpl()->AddCell('getDeals', tpl()->Set([
                        '{trans_id}' => $value['trans_id'],
                        '{user_id}' => $row->user_id,
                        '{seller_id}' => $user->id,
                        '{product_name}' => htmlspecialchars_decode($product['name']),
                        '{is_action}' => Refunds()->IsAction($value['trans_id']) ? 'yes' : 'no'
                    ], tpl()->Get('elements/messenger/confirm/item')));
                }

                return tpl()->Execute(tpl()->GetCell('getDeals'));
            }
            catch(Exception $e) {
                return false;
            }
        }

        /* Получаем сумму заработанною пользователем */
        public function rowPrice($user_id) {
            try {
                $sth = pdo()->prepare('SELECT * FROM users__income WHERE user_id=:user_id');
                $sth->execute([':user_id' => $user_id]);

                $count = 0;

                if(!$sth->rowCount()) {
                    return $count;
                }

                while($row = $sth->fetch(PDO::FETCH_OBJ)) {
                    $count += $row->price;
                }

                return $count;
            }
            catch(Exception $e) {
                return 0;
            }
        }
    }