<?PHP
    class SellerOutput {
        private $merchants;
        var $Status = [];

        /* Подгружаем нужные конфиги */
        public function __construct() {
            $path = $_SERVER['DOCUMENT_ROOT'] . '/../configuration/output/merchants.json';

            if(!file_exists($path)) {
                die(getLang('alerts.output_not_merchants'));
            }

            $this->Status[0]['label'] = getLang('output.noty.status.0');
            $this->Status[1]['label'] = getLang('output.noty.status.1');
            $this->Status[2]['label'] = getLang('output.noty.status.2');
            $this->Status[3]['label'] = getLang('output.noty.status.3');

            $this->merchants = json_decode(file_get_contents($path), true);
        }

        /* Проверяем наличие мерчанта */
        public function isMerchant($name) {
            foreach($this->merchants as $key => $data) {
                if($key == $name) {
                    return true;
                }
            }

            return false;
        }

        /* Формируем мерчанты для вывода в Select */
        public function getListSelect() {
            tpl()->ClearCell('getListSelect');

            foreach($this->merchants as $key => $data) {
                tpl()->AddCell('getListSelect', tpl()->Set([
                    '{name}' => $key,
                    '{value}' => $key,
                    '{percent}' => $data['percent']
                ], '<option value="{value}">{name} {percent}%</option>'));
            }

            return tpl()->GetCell('getListSelect');
        }

        /* Получаем сведения */
        public function Get($id) {
            try {
                $sth = pdo()->prepare('SELECT * FROM money__output WHERE id=:id LIMIT 1');
                $sth->execute([':id' => $id]);

                if(!$sth->rowCount()) {
                    return null;
                }

                return $sth->fetch(PDO::FETCH_OBJ);
            }
            catch(Exception $e) {
                return null;
            }
        }

        /* Запрос на вывод средств */
        public function Send($user_id, $merchant, $requisites, $price) {
            try {
                $sth = pdo()->prepare('INSERT INTO money__output(user_id, merchant, requisites, price, date) VALUES (:user_id, :merchant, :requisites, :price, :date)');
                $sth->execute([
                    ':user_id' => $user_id,
                    ':merchant' => $merchant,
                    ':requisites' => $requisites,
                    ':price' => $price,
                    ':date' => time()
                ]);

                return pdo()->lastInsertId();
            }
            catch(Exception $e) {
                throw $e;
            }
        }

        /* Количество запросов */
        public function rowCount($user_id) {
            try {
                $sth = pdo()->prepare('SELECT * FROM money__output WHERE user_id=:user_id AND status=2');
                $sth->execute([':user_id' => $user_id]);

                return $sth->rowCount();
            }
            catch(Exception $e) {
                return 0;
            }
        }

        /* Получаем историю вывода */
        public function getList($user_id) {
            try {
                $sth = pdo()->prepare('SELECT * FROM money__output WHERE user_id=:user_id ORDER BY id DESC');
                $sth->execute([':user_id' => $user_id]);

                if(!$sth->rowCount()) {
                    return getNotData(getLang('library.admin.output.errors.0'), true, 3);
                }

                tpl()->ClearCell('getList');
                while($row = $sth->fetch(PDO::FETCH_OBJ)) {
                    tpl()->AddCell('getList', tpl()->Set([
                        '{id}' => $row->id,
                        '{date}' => getLang('getTimeAgo.date', [date('d.m.Y', $row->date), date('H:i', $row->date)]),
                        '{merchant}' => $row->merchant,
                        '{requisites}' => $row->requisites,
                        '{price}' => $row->price,
                        '{statusId}' => $row->status,
                        '{statusLabel}' => $this->Status[$row->status]['label'],
                        '{message}' => isset($row->message) ? htmlspecialchars_decode($row->message) : null
                    ], tpl()->Get('elements/table/output/item')));
                }

                return tpl()->Execute(tpl()->GetCell('getList'));
            }
            catch(Exception $e) {
                return getNotData($e->getMessage(), true, 3);
            }
        }

        /* Сумма вывода */
        public function rowPrice($user_id) {
            try {
                $sth = pdo()->prepare('SELECT * FROM money__output WHERE user_id=:user_id AND status=1');
                $sth->execute([':user_id' => $user_id]);
                $count = 0;

                if(!$sth->rowCount()) {
                    return $count;
                }

                while($row = $sth->fetch(PDO::FETCH_OBJ)) {
                    $count += $row->price;
                }

                return $count;
            }
            catch(Exception $e) {
                return 0;
            }
        }
    }