<?PHP
    class SellerProduct {
        private $status = [
            0 => ['class' => 'text-info'],
            1 => ['class' => 'text-success'],
            2 => ['class' => 'text-warning'],
            3 => ['class' => 'text-danger'],
            4 => ['class' => 'text-dark']
        ];

        public function __construct() {
            $this->status[0]['label'] = getLang('library.sellerproduct.status.0');
            $this->status[1]['label'] = getLang('library.sellerproduct.status.1');
            $this->status[2]['label'] = getLang('library.sellerproduct.status.2');
            $this->status[3]['label'] = getLang('library.sellerproduct.status.3');
            $this->status[4]['label'] = getLang('library.sellerproduct.status.4');
        }

        /* Получаем список товаров продавца */
        public function getList() {
            try {
                $sth = pdo()->prepare('SELECT * FROM product WHERE author_id=:author_id ORDER BY id DESC');
                $sth->execute([':author_id' => $_SESSION['id']]);

                if(!$sth->rowCount()) {
                    return getNotData(getLang('seller.product.no'), true, 10);
                }

                tpl()->ClearCell('getList');

                while($row = $sth->fetch(PDO::FETCH_OBJ)) {
                    $price = formatPrice($row->id);

                    if($row->enable == 3) {
                        $status_message = $row->reason;
                    }

                    $boostProduct = MarketPremium()->GetForProduct($row->id);
                    
                    $boost = match (true) {
                        $row->enable != 1 => getLang('market.my_product.table.boost.labels.0'),
                        is_null($boostProduct) || ($row->enable == 1 && $boostProduct['date_end'] < time()) => MarketPremium()->getButton($row->id),
                        default => getLang('market.my_product.table.boost.labels.1', [
                            date('d.m', $boostProduct['date_end']),
                            MarketPremium()->getButton($row->id, 2)
                        ]),
                    };                    

                    tpl()->AddCell('getList', tpl()->Set([
                        '{id}' => $row->id,
                        '{name}' => htmlspecialchars_decode($row->name),
                        '{image}' => $row->image,
                        '{price}' => $price->total,
                        '{date}' => uDate($row->date),
                        '{date_update}' => is_null($row->date_update) ? getLang('market.my_product.table.status.date_update') : uDate($row->date_update),
                        '{status_label}' => $this->status[$row->enable]['label'],
                        '{status_class}' => $this->status[$row->enable]['class'],
                        '{status_message}' => isset($status_message) ? $status_message : '',
                        '{quantity}' => Product()->formateQuantity(Product()->getQuantity($row->id)),
                        '{boost}' => $boost,
                        '{toggle}' => $row->enable
                    ], tpl()->Get('elements/table/my_product/item')));

                    unset($status_message);
                }

                return tpl()->Execute(tpl()->GetCell('getList'));
            }
            catch(Exception $e) {
                return getNotData($e->getMessage(), true, 10);
            }
        }

        /* Форматируем товары продавца */
        public function formatProducts($userid) {
            try {
                $sth = pdo()->prepare('SELECT id FROM product WHERE author_id=:author_id');
                $sth->execute([':author_id' => $userid]);

                if(!$sth->rowCount()) {
                    return 0;
                }
                
                return $sth->fetchAll(PDO::FETCH_ASSOC);
            }
            catch(Exception $e) {
                throw $e;
            }
        }

        /* Количество продаж за сегодня */
        public function rowSellsToDay($userid) {
            try {
                $start_date = strtotime('today midnight');
                $end_date = strtotime('tomorrow midnight') - 1;
                $products = $this->formatProducts($userid);

                if(!$products) {
                    return 0;
                }
                
                $productIds = array_column($products, 'id');
                $sth = pdo()->prepare('SELECT * FROM money__transactions WHERE type=:type AND date BETWEEN :start_date AND :end_date');
                $sth->execute([':type' => 3, ':start_date' => $start_date, ':end_date' => $end_date]);

                $transactions = $sth->fetchAll(PDO::FETCH_ASSOC);
                $salesCount = 0;

                foreach($transactions as $transaction) {
                    $data = json_decode($transaction['data'], true);

                    if(isset($data['product_id']) && in_array($data['product_id'], $productIds)) {
                        $salesCount++;
                    }
                }

                return $salesCount;
            }
            catch(Exception $e) {
                return 0;
            }
        }


        /* Получаем товары продавца (в массиве) */
        public function getProducts($userid) {
            try {
                $sth = pdo()->prepare('SELECT * FROM product WHERE author_id=:author_id ORDER BY id DESC');
                $sth->execute([
                    ':author_id' => $userid
                ]);

                if(!$sth->rowCount()) {
                    return null;
                }

                $arr = [];

                while($row = $sth->fetch(PDO::FETCH_ASSOC)) {
                    $arr[] = $row;
                }

                return $arr;
            }
            catch(Exception $e) {
                return $e->getMessage();
            }
        }

        /* Получаем список продаж (в массиве) */
        public function querySells($userid) {
            try {
                $dataProducts = $this->getProducts($userid);

                if(empty($dataProducts)) {
                    return null;
                }

                foreach($dataProducts as $value) {
                    $formats[] = $value['id'];
                }

                if (empty($formats)) {
                    return null;
                }

                $placeholders = implode(',', array_fill(0, count($formats), '?'));

                $sql = "SELECT * FROM money__transactions WHERE JSON_UNQUOTE(JSON_EXTRACT(data, '$.product_id')) IN ($placeholders) AND status=1 ORDER BY id DESC";
                $sth = pdo()->prepare($sql);
                $sth->execute($formats);

                return $sth->fetchAll(PDO::FETCH_ASSOC);
            }
            catch(Exception $e) {
                return $e->getMessage();
            }
        }

        /* Получаем табличный список продаж */
        public function listSells($userid) {
            try {
                $dataSells = $this->querySells($userid);

                if(is_null($dataSells)) {
                    return 'Нет продаж';
                }
                
                tpl()->ClearCell('listSells');
                foreach($dataSells as $key => $value) {
                    $data = json_decode($value['data']);
                    $dataProduct = Product()->Get($data->product_id);

                    tpl()->AddCell('listSells', tpl()->Set([
                        '{id}' => $value['id'],
                        '{product_name}' => $dataProduct['product']['name'],
                        '{product_id}' => $dataProduct['product']['id'],
                        '{product_count}' => $data->count,
                        '{buyer_name}' => getName($value['user_id']),
                        '{buyer_id}' => $value['user_id'],
                        '{price}' => number_format($value['amount'], 2, ',', '.'),
                        '{price_total}' => number_format($value['amount'], 2, ',', '.'),
                        '{date}' => date('d.m в H:i', $value['date'])
                    ], tpl()->Get('elements/market/seller/history/sells')));
                }

                return tpl()->Execute(tpl()->GetCell('listSells'));
            }
            catch(Exception $e) {
                return $e->getMessage();
            }
        }

        /* Получаем количество товаров */
        public function rowAll($userid) {
           $dataProducts = $this->getProducts($userid);

            if(is_null($dataProducts)) {
                return 0;
            }

           return count($dataProducts);
        }

        /* Получаем количество проданного товара */
        public function rowSells($userid) {
            $data = $this->querySells($userid);

            if(is_null($data)) {
                return 0;
            }

            return count($data);
        }
    }