<?PHP
	class AdminCategory {
		/* Получаем категорию */
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__category WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);

				if(!$sth->rowCount()) {
					return null;
				}

				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Получаем категории */
		public function getAll() {
			try {
				$sth = pdo()->query('SELECT * FROM `product__category` ORDER BY position ASC');
				
				if(!$sth->rowCount()) {
					return tpl()->Set(['{text}' => getLang('library.admin.category.errors.5')], tpl()->Get('elements/no_data'));
				}
				
				tpl()->ClearCell('listCategory');
				while($category = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('listCategory', tpl()->Set([
						'{item_id}' => $category->id,
						'{item_name}' => get_custom_lang($category->name),
						'{item_icon}' => $category->icon,
						'{item_position}' => $category->position
					], tpl()->Get('elements/items/category')));
				}
				
				return tpl()->Execute(tpl()->GetCell('listCategory'));
			}
			catch(Exception $e) {
				return tpl()->Set(['{text}' => $e->getMessage()], tpl()->Get('elements/no_data'));
			}
		}
		
		/* Добавляем раздел */
		public function Add($_ARRAY = []) {
			try {
				$sth = pdo()->prepare('INSERT INTO product__category(name, search_link, icon, cover, data, position) VALUES (:name, :search_link, :icon, :cover, :data, :position)');
				$sth->execute([
					':name' => $_ARRAY['name'],
					':search_link' => $_ARRAY['search_link'],
					':icon' => $_ARRAY['icon'],
					':cover' => $_ARRAY['cover'],
					':data' => isset($_ARRAY['data']) ? $_ARRAY['data'] : null,
					':position' => $this->getLastPositionId() + 1
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}

		/* Изменяем раздел */
		public function onEdit($_ARRAY = []) {
			try {
				$sth = pdo()->prepare('UPDATE `product__category` SET name=:name, search_link=:search_link, icon=:icon, cover=:cover, data=:data WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $_ARRAY['id'],
					':name' => $_ARRAY['name'],
					':search_link' => $_ARRAY['search_link'],
					':icon' => $_ARRAY['icon'],
					':cover' => $_ARRAY['cover'],
					':data' => $_ARRAY['data']
				]);
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Получаем последний ID позиции */
		public function getLastPositionId() {
			try {
				$sth = pdo()->query('SELECT * FROM product__category ORDER BY position DESC LIMIT 1');
				
				if(!$sth->rowCount()) {
					return 0;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ)->position;
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
		/* Получение подраздел */
		public function getSubcategories($category_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE category_id=:category_id ORDER BY position ASC');
				$sth->execute([
					':category_id' => $category_id
				]);
				
				if(!$sth->rowCount()) {
					return tpl()->Set(['{text}' => 'Нет разделов'], tpl()->Get('elements/no_data'));
				}
				
				tpl()->ClearCell('listSubcategory');
				
				while($subcategory = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('listSubcategory', tpl()->Set([
						'{item_id}' => $subcategory->id,
						'{item_name}' => $subcategory->name,
						'{item_position}' => $subcategory->position
					], tpl()->Get('elements/items/subcategory')));
				}
				
				return tpl()->Execute(tpl()->GetCell('listSubcategory'));
			}
			catch(Exception $e) {
				return tpl()->Set(['{text}' => $e->getMessage()], tpl()->Get('elements/no_data'));
			}
		}
		
		/* Добавляем подраздел */
		public function addSubcategory($category_id, $name) {
			try {
				$lastId = $this->getLastPositionSubcategoryId($category_id);
				
				$sth = pdo()->prepare('INSERT INTO product__subcategory(name, category_id, position) VALUES (:name, :category_id, :position)');
				$sth->execute([
					':name' => $name,
					':category_id' => $category_id,
					':position' => $lastId + 1
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Получаем последний ID позиции подраздела */
		public function getLastPositionSubcategoryId($category_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE category_id=:category_id ORDER BY position DESC LIMIT 1');
				$sth->execute([
					':category_id' => $category_id
				]);
				
				if(!$sth->rowCount()) {
					return 0;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ)->position;
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
		/* Получаем раздел по айди */
		public function getCategory($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__category WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/* Получаем подраздел по айди */
		public function getSubcategory($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/* Меняем позиции разедлов */
		public function changeCategoryPosition($selfId, $triggerId) {
			try {
				$selfData = $this->getCategory($selfId);
				
				if(empty($selfData)) {
					throw new Exception(getLang('library.admin.category.errors.0'));
				}
				
				$triggerData = $this->getCategory($triggerId);
				
				if(empty($selfData)) {
					throw new Exception(getLang('library.admin.category.errors.1'));
				}
				
				$sth = pdo()->prepare('UPDATE product__category SET position=:position WHERE id=:id LIMIT 1');
				$sth->execute([':position' => $triggerData->position, ':id' => $selfData->id]);
				
				$sth = pdo()->prepare('UPDATE product__category SET position=:position WHERE id=:id LIMIT 1');
				$sth->execute([':position' => $selfData->position, ':id' => $triggerData->id]);
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Меняем позиции подразделов */
		public function changeSubcategoryPosition($selfId, $triggerId) {
			try {
				$selfData = $this->getSubcategory($selfId);
				
				if(empty($selfData)) {
					throw new Exception(getLang('library.admin.category.errors.2'));
				}
				
				$triggerData = $this->getSubcategory($triggerId);
				
				if(empty($selfData)) {
					throw new Exception(getLang('library.admin.category.errors.3'));
				}
				
				$sth = pdo()->prepare('UPDATE product__subcategory SET position=:position WHERE id=:id LIMIT 1');
				$sth->execute([':position' => $triggerData->position, ':id' => $selfData->id]);
				
				$sth = pdo()->prepare('UPDATE product__subcategory SET position=:position WHERE id=:id LIMIT 1');
				$sth->execute([':position' => $selfData->position, ':id' => $triggerData->id]);
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Удаление раздела */
		public function removeCategory($triggerId) {
			try {
				$dataCategory = $this->getCategory($triggerId);
				
				if(empty($dataCategory)) {
					throw new Exception(getLang('library.admin.category.errors.4'));
				}
				
				/* Удаляем картинку */
				$pathIcon = $_SERVER['DOCUMENT_ROOT'] . '/uploads/images/icons/' . $dataCategory->icon;
				if(file_exists($pathIcon)) {
					unlink($pathIcon);
				}
				
				/* Удаляем раздел */
				$sth = pdo()->prepare('DELETE FROM product__category WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $triggerId]);
				
				/* Получаем подразделы */
				$sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE category_id=:category_id');
				$sth->execute([':category_id' => $triggerId]);
				
				if(!$sth->rowCount()) {
					return true;
				}
				
				while($triggerData = $sth->fetch(PDO::FETCH_OBJ)) {
					$this->removeSubcategory($triggerData->id);
				}
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Удаление подраздела */
		public function removeSubcategory($triggerId) {
			try {
				$sth = pdo()->prepare('DELETE FROM product__subcategory WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $triggerId]);
				
				/* Удаляем весь товар из данного подраздела */
				$sth = pdo()->prepare('DELETE FROM product WHERE JSON_CONTAINS(subcategories, :subcategory)');
				$sth->execute([':subcategory' => $triggerId]);
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
	}