<?php
class AdminIncome {
    var $category = [];

	public function __construct() {
		$this->category = [
			3 => getLang('admin.income.library.category.0'),
			5 => getLang('admin.income.library.category.1')
		];
	}

	// Получаем месяца
    public function getMonths() {
        $months = Languages()->getList()['months'];
        return json_encode($months['short']);
    }

    // Формируем прибыль по месяцам
    public function getProfitJson($date, $type = 3) {
        if (preg_match('/^\d{4}$/', $date)) {
            // Обработка года (YYYY)
            $startOfYear = strtotime("{$date}-01-01 00:00:00");
            $endOfYear = strtotime("{$date}-12-31 23:59:59");

            $sth = pdo()->prepare("
                WITH RankedStats AS (
                    SELECT 
                        CONCAT('month_', MONTH(FROM_UNIXTIME(s.date)) - 1) AS month,
                        SUM(s.price) OVER (PARTITION BY MONTH(FROM_UNIXTIME(s.date)) - 1) AS total_price,
                        ROW_NUMBER() OVER (PARTITION BY MONTH(FROM_UNIXTIME(s.date)) - 1 ORDER BY s.date) AS row_num,
                        mt.id AS transaction_id,
                        mt.type AS transaction_type
                    FROM 
                        stats s
                    JOIN 
                        money__transactions mt ON s.trans_id = mt.id
                    WHERE 
                        s.date BETWEEN :startOfYear AND :endOfYear
                        AND mt.type = :type
                )
                SELECT 
                    month,
                    total_price,
                    transaction_id,
                    transaction_type
                FROM 
                    RankedStats
                WHERE 
                    row_num = 1
                ORDER BY 
                    month
            ");

            $sth->execute([':startOfYear' => $startOfYear, ':endOfYear' => $endOfYear, ':type' => $type]);
        }
        elseif (preg_match('/^\d{4}-\d{2}$/', $date)) {
            // Обработка года и месяца (YYYY-MM)
            list($year, $month) = explode('-', $date);
            $startOfMonth = strtotime("{$year}-{$month}-01 00:00:00");
            $endOfMonth = strtotime("{$year}-{$month}-" . date('t', $startOfMonth) . " 23:59:59");

            $sth = pdo()->prepare("
                WITH RankedStats AS (
                    SELECT 
                        CONCAT('month_', MONTH(FROM_UNIXTIME(s.date)) - 1) AS month,
                        SUM(s.price) OVER (PARTITION BY MONTH(FROM_UNIXTIME(s.date)) - 1) AS total_price,
                        ROW_NUMBER() OVER (PARTITION BY MONTH(FROM_UNIXTIME(s.date)) - 1 ORDER BY s.date) AS row_num,
                        mt.id AS transaction_id,
                        mt.type AS transaction_type
                    FROM 
                        stats s
                    JOIN 
                        money__transactions mt ON s.trans_id = mt.id
                    WHERE 
                        s.date BETWEEN :startOfMonth AND :endOfMonth
                        AND mt.type = :type
                )
                SELECT 
                    month,
                    total_price,
                    transaction_id,
                    transaction_type
                FROM 
                    RankedStats
                WHERE 
                    row_num = 1
                ORDER BY 
                    month
            ");

            $sth->execute([':startOfMonth' => $startOfMonth, ':endOfMonth' => $endOfMonth, ':type' => $type]);
        }
        elseif (preg_match('/^\d{4}-\d{2}-\d{2}$/', $date)) {
            // Обработка полной даты (YYYY-MM-DD)
            $startOfDay = strtotime("{$date} 00:00:00");
            $endOfDay = strtotime("{$date} 23:59:59");

            $sth = pdo()->prepare("
                WITH RankedStats AS (
                    SELECT 
                        CONCAT('month_', MONTH(FROM_UNIXTIME(s.date)) - 1) AS month,
                        SUM(s.price) OVER (PARTITION BY MONTH(FROM_UNIXTIME(s.date)) - 1) AS total_price,
                        ROW_NUMBER() OVER (PARTITION BY MONTH(FROM_UNIXTIME(s.date)) - 1 ORDER BY s.date) AS row_num,
                        mt.id AS transaction_id,
                        mt.type AS transaction_type
                    FROM 
                        stats s
                    JOIN 
                        money__transactions mt ON s.trans_id = mt.id
                    WHERE 
                        s.date BETWEEN :startOfDay AND :endOfDay
                        AND mt.type = :type
                )
                SELECT 
                    month,
                    total_price,
                    transaction_id,
                    transaction_type
                FROM 
                    RankedStats
                WHERE 
                    row_num = 1
                ORDER BY 
                    month
            ");

            $sth->execute([':startOfDay' => $startOfDay, ':endOfDay' => $endOfDay, ':type' => $type]);
        }
        else {
            throw new Exception(getLang('admin.income.library.labels.0'));
        }

        if ($sth->rowCount() == 0) {
            return json_encode([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]);
        }

        $result = [];
        while ($row = $sth->fetch(PDO::FETCH_ASSOC)) {
            $result[$row['month']] = $row['total_price'];
        }

        $months = ['month_0', 'month_1', 'month_2', 'month_3', 'month_4', 'month_5', 'month_6', 'month_7', 'month_8', 'month_9', 'month_10', 'month_11'];

        foreach ($months as $month) {
            if (!isset($result[$month])) {
                $result[$month] = 0;
            }
        }

        ksort($result);
        return json_encode(array_values($result));
    }

    // Получаем табличную статистику прибыли
    public function getTable($date) {
        try {
            if (preg_match('/^\d{4}$/', $date)) {
                // Обработка года (YYYY)
                $startOfYear = strtotime("{$date}-01-01 00:00:00");
                $endOfYear = strtotime("{$date}-12-31 23:59:59");

                $sth = pdo()->prepare('
                    SELECT 
                        stats.*, 
                        money__transactions.user_id, 
                        money__transactions.type, 
                        money__transactions.data,
                        SUM(stats.price) OVER (ORDER BY stats.id) AS cumulative_profit
                    FROM 
                        stats
                    JOIN 
                        money__transactions ON stats.trans_id = money__transactions.id
                    WHERE 
                        stats.date BETWEEN :startOfYear AND :endOfYear
                    ORDER BY 
                        stats.id DESC
                ');

                $sth->execute([
                    ':startOfYear' => $startOfYear,
                    ':endOfYear' => $endOfYear
                ]);
            }
            elseif (preg_match('/^\d{4}-\d{2}$/', $date)) {
                // Обработка года и месяца (YYYY-MM)
                list($year, $month) = explode('-', $date);
                $startOfMonth = strtotime("{$year}-{$month}-01 00:00:00");
                $endOfMonth = strtotime("{$year}-{$month}-" . date('t', $startOfMonth) . " 23:59:59");

                $sth = pdo()->prepare('
                    SELECT 
                        stats.*, 
                        money__transactions.user_id, 
                        money__transactions.type, 
                        money__transactions.data,
                        SUM(stats.price) OVER (ORDER BY stats.id) AS cumulative_profit
                    FROM 
                        stats
                    JOIN 
                        money__transactions ON stats.trans_id = money__transactions.id
                    WHERE 
                        stats.date BETWEEN :startOfMonth AND :endOfMonth
                    ORDER BY 
                        stats.id DESC
                ');

                $sth->execute([
                    ':startOfMonth' => $startOfMonth,
                    ':endOfMonth' => $endOfMonth
                ]);
            }
            elseif (preg_match('/^\d{4}-\d{2}-\d{2}$/', $date)) {
                // Обработка полной даты (YYYY-MM-DD)
                $startOfDay = strtotime("{$date} 00:00:00");
                $endOfDay = strtotime("{$date} 23:59:59");

                $sth = pdo()->prepare('
                    SELECT 
                        stats.*, 
                        money__transactions.user_id, 
                        money__transactions.type, 
                        money__transactions.data,
                        SUM(stats.price) OVER (ORDER BY stats.id) AS cumulative_profit
                    FROM 
                        stats
                    JOIN 
                        money__transactions ON stats.trans_id = money__transactions.id
                    WHERE 
                        stats.date BETWEEN :startOfDay AND :endOfDay
                    ORDER BY 
                        stats.id DESC
                ');

                $sth->execute([
                    ':startOfDay' => $startOfDay,
                    ':endOfDay' => $endOfDay
                ]);
            }
            else {
                throw new Exception(getLang('admin.income.library.labels.0'));
            }

            if($sth->rowCount() == 0) {
                return getNotData(getLang('admin.income.library.labels.1'), true, 4);
            }

            tpl()->ClearCell('admin_income');
            while ($row = $sth->fetch(PDO::FETCH_OBJ)) {
                tpl()->AddCell('admin_income', tpl()->Set([
                    '{date}' => getTimeAgo($row->date),
                    '{category}' => $this->category[$row->type],
                    '{profit}' => $row->price,
                    '{cumulative_profit}' => $row->cumulative_profit
                ], tpl()->Get('elements/chart/income')));
            }

            return tpl()->Execute(tpl()->GetCell('admin_income'));
        }
        catch (Exception $e) {
            error_log('AdminIncome: ' . $e->getMessage());
            return getNotData($e->getMessage(), true, 4);
        }
    }

    // Получаем сведения по прибыли
    public function getProfit($date, $type = 3) {
        if (preg_match('/^\d{4}$/', $date)) {
            // Обработка года (YYYY)
            $startOfYear = strtotime("{$date}-01-01 00:00:00");
            $endOfYear = strtotime("{$date}-12-31 23:59:59");

            $sth = pdo()->prepare("
                SELECT 
                    SUM(s.price) AS total_price
                FROM 
                    stats s
                JOIN 
                    money__transactions mt ON s.trans_id = mt.id
                WHERE 
                    s.date BETWEEN :startOfYear AND :endOfYear
                    AND mt.type = :type
                GROUP BY 
                    mt.type, DATE(FROM_UNIXTIME(s.date))
            ");

            $sth->execute([
                ':startOfYear' => $startOfYear,
                ':endOfYear' => $endOfYear,
                ':type' => $type
            ]);
        }
        elseif (preg_match('/^\d{4}-\d{2}$/', $date)) {
            // Обработка года и месяца (YYYY-MM)
            list($year, $month) = explode('-', $date);
            $startOfMonth = strtotime("{$year}-{$month}-01 00:00:00");
            $endOfMonth = strtotime("{$year}-{$month}-" . date('t', $startOfMonth) . " 23:59:59");

            $sth = pdo()->prepare("
                SELECT 
                    SUM(s.price) AS total_price
                FROM 
                    stats s
                JOIN 
                    money__transactions mt ON s.trans_id = mt.id
                WHERE 
                    s.date BETWEEN :startOfMonth AND :endOfMonth
                    AND mt.type = :type
                GROUP BY 
                    mt.type, DATE(FROM_UNIXTIME(s.date))
            ");

            $sth->execute([
                ':startOfMonth' => $startOfMonth,
                ':endOfMonth' => $endOfMonth,
                ':type' => $type
            ]);
        }
        elseif (preg_match('/^\d{4}-\d{2}-\d{2}$/', $date)) {
            // Обработка полной даты (YYYY-MM-DD)
            $startOfDay = strtotime("{$date} 00:00:00");
            $endOfDay = strtotime("{$date} 23:59:59");

            $sth = pdo()->prepare("
                SELECT 
                    SUM(s.price) AS total_price
                FROM 
                    stats s
                JOIN 
                    money__transactions mt ON s.trans_id = mt.id
                WHERE 
                    s.date BETWEEN :startOfDay AND :endOfDay
                    AND mt.type = :type
                GROUP BY 
                    mt.type, DATE(FROM_UNIXTIME(s.date))
            ");

            $sth->execute([
                ':startOfDay' => $startOfDay,
                ':endOfDay' => $endOfDay,
                ':type' => $type
            ]);
        }
        else {
            throw new Exception(getLang('admin.income.library.labels.0'));
        }

        $results = $sth->fetchAll(PDO::FETCH_ASSOC);
        $total_price = 0;

        foreach ($results as $row) {
            $total_price += $row['total_price'];
        }

        return $total_price;
    }

    // Разница в прибыли
    public function getProfitPercentageChange($date, $type = 3) {
        $currentProfit = $this->getProfit($date, $type);
        $previousDate = $this->getPreviousPeriod($date);
        $previousProfit = $this->getProfit($previousDate, $type);

        if ($previousProfit == 0) {
            return 0;
        }

        $percentageChange = (($currentProfit - $previousProfit) / $previousProfit) * 100;
        return round($percentageChange, 1);
    }

    // Получаем предыдущий период
    private function getPreviousPeriod($date) {
        if (preg_match('/^\d{4}$/', $date)) {
            return (intval($date) - 1);
        }
        elseif (preg_match('/^\d{4}-\d{2}$/', $date)) {
            // Для года и месяца (YYYY-MM)
            list($year, $month) = explode('-', $date);
            $previousMonth = intval($month) - 1;

            if ($previousMonth < 1) {
                $previousMonth = 12;
                $year--;
            }

            return sprintf('%04d-%02d', $year, $previousMonth);
        }
        elseif(preg_match('/^\d{4}-\d{2}-\d{2}$/', $date)) {
            return date('Y-m-d', strtotime($date . ' -1 day'));
        }

        throw new Exception(getLang('admin.income.library.labels.0'));
    }

    // Вычисляем общий процент от всей прибыли
    public function getTotalProfitPercentageChange($date) {
        $currentTotalProfit = $this->getProfit($date, 3) + $this->getProfit($date, 5);
        $previousDate = $this->getPreviousPeriod($date);
        $previousTotalProfit = $this->getProfit($previousDate, 3) + $this->getProfit($previousDate, 5);

        if ($previousTotalProfit == 0) {
            return 0;
        }

        $percentageChange = (($currentTotalProfit - $previousTotalProfit) / $previousTotalProfit) * 100;
        return round($percentageChange, 1);
    }
}