<?PHP
	class AdminOutput {
        private $merchants;
        var $Status = [];

        /* Подгружаем нужные конфиги */
        public function __construct() {
            $path = $_SERVER['DOCUMENT_ROOT'] . '/../configuration/output/merchants.json';

            if(!file_exists($path)) {
                die(getLang('library.admin.merchants.errors.0'));
            }

			$this->Status = [
				0 => ['label' => getLang('library.admin.merchants.status.0')],
				1 => ['label' => getLang('library.admin.merchants.status.1')],
				2 => ['label' => getLang('library.admin.merchants.status.2')],
				3 => ['label' => getLang('library.admin.merchants.status.3')]
			];

            $this->merchants = json_decode(file_get_contents($path), true);
        }

		/* Получаем историю выводов */
		public function getList() {
			try {
				$sth = pdo()->query('SELECT * FROM money__output ORDER BY id DESC');
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('library.admin.merchants.errors.1'), true, 7);
				}

				tpl()->ClearCell('getList');
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('getList', tpl()->Set([
						'{id}' => $row->id,
						'{user_id}' => $row->user_id,
						'{user_name}' => getName($row->user_id),
						'{merchant}' => $row->merchant,
						'{requisites}' => $row->requisites,
						'{price}' => $row->price,
						'{statusId}' => $row->status,
						'{statusMessage}' => $this->Status[$row->status]['label'],
						'{message}' => isset($row->message) ? htmlspecialchars_decode($row->message) : '',
						'{date}' => getLang('getTimeAgo.date', [date('d.m.Y', $row->date), date('H:i', $row->date)])
					], tpl()->Get('elements/table/item_output')));
				}

				return tpl()->Execute(tpl()->GetCell('getList'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage(), true, 7);
			}
		}

		/* Получаем данные */
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM money__output WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);

				if(!$sth->rowCount()) {
					return null;
				}

				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Отмена вывода средств */
		public function onRejection($id, $message) {
			try {
				$data = $this->Get($id);

				if(empty($data)) {
					throw new Exception(getLang('library.admin.merchants.errors.2'));
				}

				$sth = pdo()->prepare('UPDATE money__output SET status=:status, message=:message WHERE id=:id LIMIT 1');
				$sth->execute([':status' => 3, ':message' => htmlspecialchars($message), ':id' => $id]);

				$UsersBalance = new UsersBalance;
				$UsersBalance->addBalance($data->user_id, $data->price);

				CallBack()->Call('onRejectionOutput', [
					'id' => $data->id,
					'user_id' => $data->user_id,
					'price' => $data->price,
					'merchant' => $data->merchant,
					'requisites' => $data->requisites,
					'message' => $message,
					'date' => time()
				]);

				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Подтверждение вывода средств */
		public function onSuccess($id) {
			try {
				$data = $this->Get($id);

				if(empty($data)) {
					throw new Exception(getLang('library.admin.merchants.errors.2'));
				}
				
				$sth = pdo()->prepare('UPDATE money__output SET status=:status, message=:message WHERE id=:id LIMIT 1');
				$sth->execute([':status' => 1, ':id' => $id, ':message' => null]);

				CallBack()->Call('onSuccessOutput', [
					'id' => $data->id,
					'user_id' => $data->user_id,
					'price' => $data->price,
					'merchant' => $data->merchant,
					'requisites' => $data->requisites,
					'date' => time()
				]);

				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}
	}