<?PHP
	class AdminPages {
		/* Получение данных */
		public function Get($id) {
			if(empty($id)) {
				return null;
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM pages WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/* Обновление страницы */
		public function Update($_ARRAY = []) {
			try {
				$sth = pdo()->prepare('UPDATE pages SET link=:link, title=:title, description=:description, keywords=:keywords, content=:content, enable=:enable WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $_ARRAY['id'],
					':link' => $_ARRAY['link'],
					':title' => htmlspecialchars(strip_tags($_ARRAY['title'])),
					':description' => htmlspecialchars(strip_tags($_ARRAY['description'])),
					':keywords' => htmlspecialchars(strip_tags($_ARRAY['keywords'])),
					':content' => htmlspecialchars($_ARRAY['content']),
					':enable' => $_ARRAY['enable']
				]);
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Добавление страницы */
		public function Add($_ARRAY = []) {
			try {
				$sth = pdo()->prepare('INSERT INTO pages(link, title, description, keywords, content, enable, date) VALUES (:link, :title, :description, :keywords, :content, :enable, :date)');
				$sth->execute([
					':link' => $_ARRAY['link'],
					':title' => htmlspecialchars(strip_tags($_ARRAY['title'])),
					':description' => htmlspecialchars(strip_tags($_ARRAY['description'])),
					':keywords' => htmlspecialchars(strip_tags($_ARRAY['keywords'])),
					':content' => htmlspecialchars($_ARRAY['content']),
					':enable' => $_ARRAY['enable'],
					':date' => time()
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Список всех страниц */
		public function getAll() {
			try {
				$sth = pdo()->query('SELECT * FROM pages ORDER BY id DESC');
				
				if(!$sth->rowCount()) {
					return tpl()->Execute(tpl()->Set(['{text}' => 'Нет данных'], tpl()->Get('elements/no_data')));
				}
				
				tpl()->ClearCell('AdminPages');
				
				while($page = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('AdminPages', tpl()->Set([
						'{page_id}' => $page->id,
						'{page_name}' => $page->title,
						'{page_link}' => $page->link
					], tpl()->Get('elements/items/pages')));
				}
				
				return tpl()->Execute(tpl()->GetCell('AdminPages'));
			}
			catch(Exception $e) {
				return tpl()->Execute(tpl()->Set(['{text}' => getLang('library.admin.pages.errors.0')], tpl()->Get('elements/no_data')));
			}
		}
		
		/* Удаление страницы */
		public function onDelete($id) {
			try {
				$sth = pdo()->prepare('DELETE FROM pages WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
	}