<?PHP
    class AdminSubcategoryData {
        var $types = [];

        public function __construct() {
            $this->types = [
                'text' => [
                    'name' => getLang('library.admin.subcategory_data.types.text'),
                    'sample' => '<input type="text" id="{id}" name="{name}" class="{class}" minlength="{minlength}" maxlength="{maxlength}" value="{value}" placeholder="{placeholder}" autocomplete="off" {required}>'
                ],
                'number' => [
                    'name' => getLang('library.admin.subcategory_data.types.number'),
                    'sample' => '<input type="number" id="{id}" name="{name}" class="{class}" min="{min}" max="{max}" value="{value}" placeholder="{placeholder}" autocomplete="off" {required}>'
                ],
                'select' => [
                    'name' => getLang('library.admin.subcategory_data.types.select'),
                    'sample' => '<select name="{name}" id="{id}" class="{class}" {required}>{options}</select>'
                ]
            ];
        }

        /* Проверяем наличие подраздела */
        public function Is($id) {
            try {
                $sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE id=:id LIMIT 1');
                $sth->execute([':id' => $id]);

                return $sth->rowCount();
            }
            catch(Exception $e) {
                return false;
            }
        }

        /* Получаем сведения о подкатегории */
        public function Get($id) {
            try {
                $sth = pdo()->prepare('SELECT * FROM product__subcategory WHERE id=:id LIMIT 1');
                $sth->execute([':id' => $id]);

                if(!$sth->rowCount()) {
                    return null;
                }

                return $sth->fetch(PDO::FETCH_OBJ);
            }
            catch(Exception $e) {
                return null;
            }
        }

        /* Добавляем данные в подраздел */
        public function Add($data) {
            try {
                $allowedMethods = ['data', 'data_auto'];
                $method = $data['method'];
            
                if(!in_array($method, $allowedMethods, true)) {
                    throw new Exception('Invalid method provided');
                }
            
                unset($data['method']);
            
                $oldData = $this->Get($data['id']);
                unset($data['id']);
            
                if(is_null($oldData)) {
                    throw new Exception(getLang('library.admin.subcategory_data.errors.0'));
                }
                
                switch($method) {
                    case 'data': {
                        $endData = is_null($oldData->data) ? [uniqid() => $data] : array_merge(json_decode($oldData->data, true), [uniqid() => $data]);
                        break;
                    }

                    default: {
                        $endData = is_null($oldData->data_auto) ? [uniqid() => $data] : array_merge(json_decode($oldData->data_auto, true), [uniqid() => $data]);
                    }
                }

                $query = 'UPDATE product__subcategory SET ' . $method . ' = :data WHERE id = :id LIMIT 1';
                $sth = pdo()->prepare($query);
                $sth->execute([
                    ':id' => $oldData->id,
                    ':data' => json_encode($endData)
                ]);
            
                return true;
            }
            catch(Exception $e) {
                throw new Exception($e->getMessage());
            }
        }

        /* Изменение данных */
        public function Edit($data) {
            try {
                $allowedMethods = ['data', 'data_auto'];
                $method = $data['method'];
                
                if(!in_array($method, $allowedMethods, true)) {
                    throw new Exception('Invalid method provided');
                }
                
                unset($data['method']);
                
                $oldData = $this->Get($data['id']);
                unset($data['id']);
                
                switch($method) {
                    case 'data': {
                        if(is_null($oldData) || is_null($oldData->data)) {
                            throw new Exception(getLang('library.admin.subcategory_data.errors.0'));
                        }
                        
                        $endData = json_decode($oldData->data, true);
                        break;
                    }

                    default: {
                        if(is_null($oldData) || is_null($oldData->data_auto)) {
                            throw new Exception(getLang('library.admin.subcategory_data.errors.0'));
                        }
                        
                        $endData = json_decode($oldData->data_auto, true);
                    }
                }

                $uniqid = $data['uniqid'];
                unset($data['uniqid']);
                
                if(!array_key_exists($uniqid, $endData)) {
                    throw new Exception(getLang('library.admin.subcategory_data.errors.1'));
                }
                
                $endData[$uniqid] = ['type' => $endData[$uniqid]['type']] + $data;
                
                $query = 'UPDATE product__subcategory SET ' . $method . ' = :data WHERE id = :id LIMIT 1';
                $sth = pdo()->prepare($query);
                $sth->execute([
                    ':id' => $oldData->id,
                    ':data' => json_encode($endData)
                ]);
                
                return true;                
            }
            catch(Exception $e) {
                throw new Exception($e->getMessage());
            }
        }

        /* Получаем список */
        public function getList($id) {
            try {
                $data = $this->Get($id);
                if(is_null($data) || is_null($data->data)) {
                    return getNotData(getLang('library.admin.subcategory_data.errors.2'), true, 3);
                }

                $jsonData = json_decode($data->data, true);

                tpl()->ClearCell('getList');
                foreach($jsonData as $uniqid => $value) {
                    tpl()->AddCell('getList', tpl()->Set([
                        '{id}' => $data->id,
                        '{uniqid}' => $uniqid,
                        '{label}' => $value['label'],
                        '{type}' => $this->types[$value['type']]['name'],
                        '{method}' => 'data'
                    ], tpl()->Get('elements/items/subitem')));
                }
                
                return tpl()->Execute(tpl()->GetCell('getList'));
            }
            catch(Exception $e) {
                return getNotData($e->getMessage());
            }
        }

        /* Получаем список */
        public function getListAuto($id) {
            try {
                $data = $this->Get($id);
                if(is_null($data) || is_null($data->data_auto)) {
                    return getNotData(getLang('library.admin.subcategory_data.errors.2'), true, 3);
                }

                $jsonData = json_decode($data->data_auto, true);

                tpl()->ClearCell('getList');
                foreach($jsonData as $uniqid => $value) {
                    tpl()->AddCell('getList', tpl()->Set([
                        '{id}' => $data->id,
                        '{uniqid}' => $uniqid,
                        '{label}' => $value['label'],
                        '{type}' => $this->types[$value['type']]['name'],
                        '{method}' => 'data_auto'
                    ], tpl()->Get('elements/items/subitem')));
                }
                
                return tpl()->Execute(tpl()->GetCell('getList'));
            }
            catch(Exception $e) {
                return getNotData($e->getMessage());
            }
        }

        /* Удаление формы */
        public function Delete($arr) {
            try {
                $allowedMethods = ['data', 'data_auto'];
                $method = $arr['method'];
                
                if(!in_array($method, $allowedMethods, true)) {
                    throw new Exception('Invalid method provided');
                }
                
                unset($arr['method']);

                $id = $arr['id'];
                $data = $this->Get($id);
        
                if(is_null($data)) {
                    throw new Exception(getLang('library.admin.subcategory_data.errors.0'));
                }
                
                switch($method) {
                    case 'data': {
                        if(is_null($data->data)) {
                            throw new Exception(getLang('library.admin.subcategory_data.errors.3'));
                        }
                
                        $jsonData = json_decode($data->data, true);
                        if (json_last_error() != JSON_ERROR_NONE) {
                            throw new Exception(getLang('library.admin.subcategory_data.errors.4'));
                        }

                        break;
                    }

                    default: {
                        if(is_null($data->data_auto)) {
                            throw new Exception(getLang('library.admin.subcategory_data.errors.3'));
                        }
                
                        $jsonData = json_decode($data->data_auto, true);
                        if (json_last_error() != JSON_ERROR_NONE) {
                            throw new Exception(getLang('library.admin.subcategory_data.errors.4'));
                        }

                        break;
                    }
                }

                $uniqid = $arr['uniqid'];
        
                if (empty($jsonData[$uniqid])) {
                    throw new Exception(getLang('library.admin.subcategory_data.errors.5'));
                }
        
                unset($jsonData[$uniqid]);
                
                if(count($jsonData) <= 0) {
                    $jsonData = null;
                }

                $sth = pdo()->prepare('UPDATE product__subcategory SET ' . $method . ' = :data WHERE id = :id LIMIT 1');
                $sth->execute([':id' => $id, ':data' => is_null($jsonData) ? $jsonData : json_encode($jsonData, JSON_UNESCAPED_UNICODE)]);
        
                return true;
            }
            catch (Exception $e) {
                throw $e;
            }
        }
    }