<?PHP
	class Telegram {
		private $apiLink = 'https://api.telegram.org/bot';
		private $conf;
		
		public function __construct($conf) {
			$this->conf = $conf;
		}
		
		/* Проверяем валидность секретного ключа */
		public function isValid($secretKey) {
			if($secretKey == $this->conf['secretKey']) {
				return true;
			}
			
			return false;
		}
		
		/* Проверяем доступность телеграм бота */
		public function isEnable() {
			return $this->conf['enable'];
		}
		
		/* Удаляем веб хук */
		public function deleteWebhook() {
			$queryLink = $this->onFormate('/deleteWebhook');
			$data = [
				'drop_pending_updates' => 'True'
			];
			
			$Result = json_decode(getSend($queryLink . '?' . http_build_query($data)), true);
			
			if($Result['ok']) {
				return true;
			}
			
			return false;
		}
		
		/* Создаем веб хук */
		public function setWebhook() {
			$queryLink = $this->onFormate('/setWebhook');
			$postFields = [
				'url' => 'https://' . $_SERVER['SERVER_NAME'] . '/api/v3/system/onWebhook',
				'secret_token' => $this->conf['secretKey']
			];
			
			$Result = json_decode(SendPost($queryLink, $postFields), true);
			
			if($Result['ok']) {
				return true;
			}
			
			return false;
		}
		
		/* Форматирование ссылки */
		public function onFormate($query) {
			$queryLink = $this->apiLink . $this->conf['apiToken'] . $query;
			return $queryLink;
		}
		
		/* Отправка боту */
		public function onSend($chatId, $message, $reply_markup = null) {
			$data = [
				'chat_id' => $chatId,
				'text' => $message
			];
			
			if(isset($reply_markup)) {
				$data['reply_markup'] = json_encode($reply_markup);
			}
			
			$queryLink = $this->onFormate('/sendMessage?' . http_build_query($data));
			getSend($queryLink);
		}
		
		/* Форматируем сообщение */
		public function onFormateMessage($query, $data) {
			$query = str_replace('{profile_name}', $data['message']['from']['first_name'], $query);
			$query = str_replace('{site_name}', conf()->site_name, $query);
			$query = str_replace('{site_host}', $_SERVER['SERVER_NAME'], $query);
			
			return $query;
		}
		
		/* Проверяем есть ли данные в базе */
		public function isData($chatId) {
			try {
				$sth = pdo()->prepare('SELECT * FROM users__telegram WHERE chat_id=:chat_id LIMIT 1');
				$sth->execute([
					':chat_id' => $chatId
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/* Создаем данные в базе */
		public function onCreate($chatId) {
			try {
				$sth = pdo()->prepare('INSERT INTO users__telegram(chat_id) VALUES (:chat_id)');
				$sth->execute([
					':chat_id' => $chatId
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				die($e->getMessage());
			}
		}
		
		/* Получаем сведения с базы */
		public function getData($chatId) {
			try {
				if(!$this->isData($chatId)) {
					return null;
				}
				
				$sth = pdo()->prepare('SELECT * FROM users__telegram WHERE chat_id=:chat_id LIMIT 1');
				$sth->execute([
					':chat_id' => $chatId
				]);
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/* Задаем данные в базе */
		public function setLastCommand($chatId, $lastCommand) {
			try {
				$sth = pdo()->prepare('UPDATE users__telegram SET last_command=:last_command WHERE chat_id=:chat_id LIMIT 1');
				$sth->execute([
					':last_command' => $lastCommand,
					':chat_id' => $chatId
				]);
				
				return true;
			}
			catch(Exception $e) {
				die($e->getMessage());
			}
		}
		
		/* Удаляем профиль привязки */
		public function onDelete($chatId) {
			try {
				$dataTelegram = $this->getData($chatId);
				
				$sth = pdo()->prepare('DELETE FROM users__telegram WHERE chat_id=:chat_id LIMIT 1');
				$sth->execute([
					':chat_id' => $chatId
				]);
				
				$sth = pdo()->prepare('UPDATE `users` SET telegram_id=:telegram_id WHERE id=:id LIMIT 1');
				$sth->execute([
					':telegram_id' => null,
					':id' => $dataTelegram->user_id
				]);
				
				return true;
			}
			catch(Exception $e) {
				die($e->getMessage());
			}
		}
		
		/* Обновляем профиль */
		public function setUserId($chatId, $userId) {
			try {
				$sth = pdo()->prepare('UPDATE users__telegram SET user_id=:user_id WHERE chat_id=:chat_id LIMIT 1');
				$sth->execute([
					':user_id' => $userId,
					':chat_id' => $chatId
				]);
				
				return true;
			}
			catch(Exception $e) {
				die($e->getMessage());
			}
		}
		
		/* Удаляем сообщение */
		public function removeMessage($chatId, $messageId) {
			$data = [
				'chat_id' => $chatId,
				'message_id' => $messageId
			];
			
			$queryLink = $this->onFormate('/deleteMessage?' . http_build_query($data));
			getSend($queryLink);
		}
		
		/* inline_keyboard menu */
		public function inlineKeyboardMenu($chatId) {
			$dataTelegram = $this->getData($chatId);
			
			if(is_null($dataTelegram)) {
				$inline_keyboard = [
					[
						[
							'text' => '🔐 Привязать профиль',
							'callback_data' => json_encode([
								'action' => 'onBind'
							])
						],
						[
							'text' => '⚙️ К настройкам',
							'url' => 'https://' . $_SERVER['SERVER_NAME'] . '/account/settings'
						]
					]
				];
			}
			else {
				$dataUser = users()->Get($dataTelegram->user_id);
				$inline_keyboard = $this->createInlineKeyboard([
					[
						'text' => '🛍 Мои товары',
						'callback_data' => json_encode([
							'action' => 'onMyProduct'
						])
					],
					[
						'text' => '🕓 История продаж',
						'callback_data' => json_encode([
							'action' => 'onHistorySells'
						])
					],
					[
						'text' => '⚙️ Настройки',
						'callback_data' => json_encode([
							'action' => 'onSettings'
						])
					]
				]);
			}
			
			return ['inline_keyboard' => $inline_keyboard];
		}
		
		/* Получаем список товаров пользователя */
		public function getMyProduct($chatId) {
			if(!$this->isData($chatId)) {
				$this->onSend($chatId, 'Выберите действите', $this->inlineKeyboardMenu($chatId));
				return;
			}
			
			$dataTelegram = $this->getData($chatId);
			
			if(empty($dataTelegram->user_id)) {
				$this->onSend($chatId, 'Выберите действите', $this->inlineKeyboardMenu($chatId));
				return;
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM product WHERE author_id=:author_id ORDER BY id DESC');
				$sth->execute([
					':author_id' => $dataTelegram->user_id
				]);
				
				if(!$sth->rowCount()) {
					$this->onSend($chatId, 'У вас нет товаров', ['inline_keyboard' => [[['text' => '🔙 Назад', 'callback_data' => json_encode(['action' => 'onMenu'])]]]]);
					return;
				}
				
				$inline_keyboard = [];
				$current_row = [];
				
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$current_row[] = [
						'text' => $row->name,
						'callback_data' => json_encode([
							'action' => 'onSelectProduct',
							'productId' => $row->id
						])
					];

					if(count($current_row) == 2) {
						$inline_keyboard[] = $current_row;
						$current_row = [];
					}
				}
				
				$current_row[] = [
					'text' => '🔙 Назад',
					'callback_data' => json_encode([
						'action' => 'onMenu'
					])
				];
				
				if(!empty($current_row)) {
					$inline_keyboard[] = $current_row;
				}
				
				$this->onSend($chatId, 'Выберите товар', ['inline_keyboard' => $inline_keyboard]);
			}
			catch(Exception $e) {
				die($e->getMessage());
			}
		}
		
		/* Получаем список продаж пользователя */
		public function getMySells($chatId) {
			if(!$this->isData($chatId)) {
				$this->onSend($chatId, 'Выберите действите', $this->inlineKeyboardMenu($chatId));
				return;
			}
			
			$dataTelegram = $this->getData($chatId);
			
			if(empty($dataTelegram->user_id)) {
				$this->onSend($chatId, 'Выберите действите', $this->inlineKeyboardMenu($chatId));
				return;
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM money__transactions WHERE type=3 AND JSON_EXTRACT(data, \'$.product_id\') IN (SELECT id FROM product WHERE author_id=:author_id) ORDER BY id DESC');
				$sth->execute([
					':author_id' => $dataTelegram->user_id
				]);
				
				if(!$sth->rowCount()) {
					$this->onSend($chatId, 'Продажи отсутствуют', ['inline_keyboard' => [[['text' => '🔙 Назад', 'callback_data' => json_encode(['action' => 'onMenu'])]]]]);
					return;
				}
				
				$inline_keyboard = [];
				$current_row = [];
				
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$current_row[] = [
						'text' => '#' . $row->id . date(' (d.m.Y в H:i)', $row->date),
						'callback_data' => json_encode([
							'action' => 'onSelectTransaction',
							'id' => $row->id
						])
					];

					if(count($current_row) == 2) {
						$inline_keyboard[] = $current_row;
						$current_row = [];
					}
				}
				
				if(!empty($current_row)) {
					$inline_keyboard[] = $current_row;
				}
				
				$inline_keyboard[] = [[
					'text' => '🔙 Назад',
					'callback_data' => json_encode([
						'action' => 'onMenu'
					])
				]];
				
				$this->onSend($chatId, 'Выберите транзакцию', ['inline_keyboard' => $inline_keyboard]);
			}
			catch(Exception $e) {
				die($e->getMessage());
			}
		}
		
		/* Формируем меню */
		public function createInlineKeyboard($menuItems) {
			$inline_keyboard = [];
			
			foreach($menuItems as $item) {
				if(isset($item['text'])) {
					$inline_keyboard[] = [$item];
				}
				else {
					$inline_keyboard[] = $item;
				}
			}
			
			return $inline_keyboard;
		} 
	}