<?PHP
	class Backup {
		private $backupDir;
		private $siteDir;
		private $dbHost;
		private $dbUser;
		private $dbPass;
		private $dbName;

		public function __construct($backupDir, $siteDir, $dbHost, $dbUser, $dbPass, $dbName) {
			$this->backupDir = $backupDir;
			$this->siteDir = $siteDir;
			$this->dbHost = $dbHost;
			$this->dbUser = $dbUser;
			$this->dbPass = $dbPass;
			$this->dbName = $dbName;
			
			if(!file_exists($this->backupDir)) {
				mkdir($this->backupDir, 0755, true);
			}
		}

		public function createBackup() {
			$backupFileName = 'backup_' . date('Y-m-d_H-i-s') . '.zip';
			$backupFilePath = $this->backupDir . $backupFileName;
			
			if($this->zipSite($this->siteDir, $backupFilePath)) {
				echo "Файлы сайта успешно архивированы в $backupFilePath\n";
			} else {
				echo "Ошибка архивирования файлов сайта\n";
				return false;
			}
			
			$dbBackupFile = $this->backupDir . 'db_' . date('Y-m-d_H-i-s') . '.sql';
			if($this->backupDatabase($dbBackupFile)) {
				echo "База данных успешно сохранена в $dbBackupFile\n";
			} else {
				echo "Ошибка резервного копирования базы данных\n";
				return false;
			}
			
			$zip = new ZipArchive();
			if($zip->open($backupFilePath) == TRUE) {
				$zip->addFile($dbBackupFile, basename($dbBackupFile));
				$zip->close();
				unlink($dbBackupFile);
				echo "Дамп базы данных добавлен в архив\n";
			} else {
				echo "Ошибка открытия архива для добавления дампа базы данных\n";
				return false;
			}

			return true;
		}

		private function zipSite($source, $destination) {
			if(!extension_loaded('zip') || !file_exists($source)) {
				return false;
			}
			
			$zip = new ZipArchive();
			if(!$zip->open($destination, ZipArchive::CREATE)) {
				return false;
			}
			
			$source = realpath($source);
			if(is_dir($source)) {
				$files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($source), RecursiveIteratorIterator::SELF_FIRST);
				foreach ($files as $file) {
					$file = realpath($file);
					if(is_dir($file)) {
						$zip->addEmptyDir(str_replace($source . '/', '', $file . '/'));
					} elseif(is_file($file)) {
						$zip->addFile($file, str_replace($source . '/', '', $file));
					}
				}
			}
			elseif(is_file($source)) {
				$zip->addFile($source, basename($source));
			}
			
			return $zip->close();
		}

		private function backupDatabase($outputFile) {
			$command = sprintf('mysqldump --opt --host=%s --user=%s --password=%s %s > %s', escapeshellarg($this->dbHost), escapeshellarg($this->dbUser), escapeshellarg($this->dbPass), escapeshellarg($this->dbName), escapeshellarg($outputFile));
			
			system($command, $returnVar);
			return $returnVar == 0;
		}
	}