<?PHP
	class Bootstrap {
		const CORE_NAMESPACE = 'UnionCMS';
		private $namespaces = [];

		// Регистрируем автозагрузчик
		public function Register() {
			spl_autoload_register([$this, 'Loader'], true, true);
		}

		// Основной метод автозагрузки
		public function Loader($className) {
			// Разбиваем имя класса на части по символу '\'
			$classNameParts = explode('\\', $className);

			// Проверяем, есть ли разделение пространства имен
			if(stripos($className, '\\') != false) {
				$className = array_pop($classNameParts); // Извлекаем последний элемент (сам класс)
				$namespace = implode('\\', $classNameParts); // Остальная часть — пространство имен
			}
			else {
				$namespace = self::CORE_NAMESPACE;
			}

			// Проверка, существует ли пространство имен в нашем массиве
			if(array_key_exists($namespace, $this->namespaces)) {
				$directories = $this->namespaces[$namespace];
				$classFileName = 'class.' . strtolower($className) . '.php';

				// Перебираем директории для поиска файла класса
				foreach($directories as $directory) {
					$filePath = $directory . DIRECTORY_SEPARATOR . $classFileName;

					if(file_exists($filePath)) {
						require_once $filePath;
						return true;
					}
				}

				// Если класс не найден
				throw new Exception("Class $className not found in the specified namespaces!");
			}

			// Если пространство имен не зарегистрировано
			throw new Exception("Namespace $namespace not registered!");
		}

		// Добавляем пространство имен с соответствующими директориями
		public function addNamespace($namespace, $directories) {
			if(is_array($directories)) {
				$this->namespaces[$namespace] = $directories;
			}
			else {
				$this->namespaces[$namespace] = [$directories];
			}
		}
	}
