<?PHP
	class Captcha {
		private $code;
		private $width;
		private $height;

		public function __construct($width = 150, $height = 50) {
			$this->width = $width;
			$this->height = $height;
		}

		// Генерация случайного кода
		public function generateCode($length = 6) {
			$chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
			$this->code = substr(str_shuffle($chars), 0, $length);
			return $this->code;
		}

		// Получение сгенерированного кода
		public function getCode() {
			return $this->code;
		}

		// Генерация изображения CAPTCHA
		public function renderImage($fontPath = null) {
			if (!$this->code) {
				throw new Exception('Сначала вызовите метод generateCode.');
			}

			$image = imagecreatetruecolor($this->width, $this->height);

			// Цвета
			$bgColor = imagecolorallocate($image, 255, 255, 255);
			$textColor = imagecolorallocate($image, 0, 0, 0);
			$lineColor = imagecolorallocate($image, 220, 220, 220);

			// Заливка фона
			imagefilledrectangle($image, 0, 0, $this->width, $this->height, $bgColor);

			// Случайные линии для усложнения
			for ($i = 0; $i < 5; $i++) {
				imageline($image, rand(0, $this->width), rand(0, $this->height), rand(0, $this->width), rand(0, $this->height), $lineColor);
			}

			// Добавление текста
			if ($fontPath && file_exists($fontPath)) {
				$fontSize = $this->height * 0.5;
				$x = rand(10, $this->width - 10 - $fontSize * strlen($this->code));
				$y = $this->height * 0.7;
				imagettftext($image, $fontSize, rand(-10, 10), $x, $y, $textColor, $fontPath, $this->code);
			} else {
				$fontSize = 5;
				$x = ($this->width - imagefontwidth($fontSize) * strlen($this->code)) / 2;
				$y = ($this->height - imagefontheight($fontSize)) / 2;
				imagestring($image, $fontSize, $x, $y, $this->code, $textColor);
			}

			// Вывод изображения
			header('Content-Type: image/png');
			imagepng($image);
			imagedestroy($image);
		}
	}