<?PHP
	class Currency {
		private $path;
		private $currency;
		private $course = [
			'usd' => [
				'rub' => 101.68
			]
		];

		public function __construct() {
			$this->currency = conf()->currency->system;
			$this->path = $_SERVER['DOCUMENT_ROOT'] . '/../configuration/system/currency';
		}

		public function onCron() {
			if($this->currency == 'rub' && $this->getExchangeRateCBR()) {
				$this->getCourseRub();
			}
			
			$cryptoRates = $this->getCryptoRates();
			if($cryptoRates != null) {
				$path = $this->path . '/crypto.json';

				if(createPathForFile($path)) {
					file_put_contents($path, json_encode($cryptoRates, JSON_PRETTY_PRINT));
				}
			}
		}

		public function getCourseRub() {
			$path = $this->path . '/cbr.json';

			if(createPathForFile($path) && file_exists($path)) {
				return json_decode(file_get_contents($path), true);
			}

			return null;
		}

		public function convertRUBToUSD($rubAmount, $exchangeRate) {
			return $rubAmount / $exchangeRate;
		}

		public function getDollars() {
			if($this->currency == 'rub') {
				$path = $this->path . '/cbr.json';

				if(file_exists($path)) {
					$data = json_decode(file_get_contents($path), true);
					if(isset($data['USD']['Value'])) {
						$this->course['usd']['rub'] = $data['USD']['Value'];
						return $this->course['usd']['rub'];
					}
				}
			}

			return null;
		}

		public function getCrypto() {
			if($this->currency == 'rub') {
				$path = $this->path . '/crypto.json';

				if(file_exists($path)) {
					return json_decode(file_get_contents($path), true);
				}
			}
		}

		public function getExchangeRateCBR() {
			$url = 'https://www.cbr.ru/scripts/XML_daily.asp';
			$xml = simplexml_load_file($url);
			if ($xml == false) {
				return false;
			}

			$currencies = [];
			foreach ($xml->Valute as $valute) {
				$currencies[(string)$valute->CharCode] = [
					'Name' => (string)$valute->Name,
					'Value' => (float)str_replace(',', '.', (string)$valute->Value),
				];
			}

			$path = $this->path . '/cbr.json';
			if (createPathForFile($path)) {
				file_put_contents($path, json_encode($currencies, JSON_PRETTY_PRINT));
				return true;
			}
			return false;
		}

		public function getCryptoRates() {
			$apiUrl = "https://api.coingecko.com/api/v3/simple/price?ids=bitcoin,ethereum,tether&vs_currencies=usd&include_market_cap=false&include_24hr_vol=false&include_24hr_change=false&include_last_updated_at=false";
			$response = $this->curlRequest($apiUrl);
			
			if ($response == false) {
				return null;
			}

			$data = json_decode($response, true);
			if ($data == null) {
				return null;
			}

			return [
				"BTC" => $data['bitcoin']['usd'],
				"ETH" => $data['ethereum']['usd'],
				"USDT" => $data['tether']['usd']
			];
		}

		public function cryptoConvert($amount, $crypto) {
			$data = $this->getCrypto();
			
			if(empty($data[$crypto])) {
				return false;
			}

			$cryptoRate = $data[$crypto];

			if($this->currency == 'rub') {
				$usdAmount = $this->convertRUBToUSD($amount, $this->getDollars());
			}
			
			return $usdAmount * $cryptoRate;
		}
		

		private function curlRequest($url) {
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_URL, $url);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			$response = curl_exec($ch);
			curl_close($ch);
			return $response;
		}
	}
