<?PHP
	class Help {
		private $path;
		var $rightString = 'b';
		var $status = [];
		var $theme = [];

		public function __construct() {
			$this->status = [
				["name" => getLang('library.help.status.0')],
				["name" => getLang('library.help.status.1')],
				["name" => getLang('library.help.status.2')],
				["name" => getLang('library.help.status.2')]
			];

			$this->theme = [
				["name" => getLang('library.help.theme.0')],
				["name" => getLang('library.help.theme.1')],
				["name" => getLang('library.help.theme.2')],
				["name" => getLang('library.help.theme.3')],
				["name" => getLang('library.help.theme.4')]
			];
		}
		
		/*
			Получаем данные тикета
		*/
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM help WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/*
			Создаем запрос
		*/
		public function Create($_ARRAY = []) {
			try {
				if(empty($_ARRAY['themeid'])) {
					throw new Exception(getLang('library.help.errors.0'));
				}
				
				if(empty($_ARRAY['message']) || !mb_strlen($_ARRAY['message']) || mb_strlen($_ARRAY['message']) < 4 || mb_strlen($_ARRAY['message']) > 1024) {
					throw new Exception(getLang('library.help.errors.1'));
				}
				
				$sth = pdo()->prepare('INSERT INTO `help`(`userid`, `themeid`, `data`, `date`) VALUES (:userid, :themeid, :data, :date)');
				$sth->execute([':userid' => $_SESSION['id'], ':themeid' => $_ARRAY['themeid'], ':data' => $_ARRAY['data'], ':date' => time()]);
				
				$id = pdo()->lastInsertId();
				$this->SendMesage($id, $_ARRAY['message']);
				
				Result([
					'Alert' => 'Success',
					'TicketId' => $id
				]);
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Отправляем сообщение
		*/
		public function SendMesage($id, $message) {
			try {
				if(empty($message) || !mb_strlen($message) || mb_strlen($message) < 4 || mb_strlen($message) > 1024) {
					throw new Exception(getLang('library.help.errors.2'));
				}
				
				$sth = pdo()->prepare('INSERT INTO `help__messages`(`helpid`, `userid`, `message`, `date`) VALUES (:helpid, :userid, :message, :date)');
				$sth->execute([
					':helpid' => $id,
					':userid' => $_SESSION['id'],
					':message' => strip_tags($message),
					':date' => time()
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Получаем последнее сообщение
		*/
		public function getMessageFromId($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM help__messages WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Проверяем доступ к тикету
		*/
		public function isAccess($ticket_id) {
			$dataTicket = $this->Get($ticket_id);
			
			if(empty($dataTicket)) {
				return false;
			}
			
			if(empty($_SESSION['id'])) {
				return false;
			}
			
			$dataUser = users()->Get($_SESSION['id']);
			
			switch($dataUser->roleid) {
				case 1: {
					return true;
				}
				
				case 3: {
					return true;
				}
				
				default: {
					/* Если пользователь не имеет группу 1 и 3, тогда проверяем на права */
					/* Права b дают пользователю зайти и работать с тикетами (удалять и т.п.) */
					if(isRights($this->rightString)) {
						return true;
					}
				}
			}
			
			if($dataTicket->userid == $dataUser->id) {
				return true;
			}
			
			return false;
		}
		
		/*
			Права на тикет
		*/
		public function isRights($userid) {
			if(empty($_SESSION['id'])) {
				return false;
			}
			
			$dataUser = users()->Get($userid);
			
			switch($dataUser->roleid) {
				case 1: {
					return true;
				}
				
				case 3: {
					return true;
				}
				
				default: {
					if(isRightsFromUser($this->rightString, $userid)) {
						return true;
					}
				}
			}
			
			return false;
		}
		
		/*
			Получаем статус
		*/
		public function getStatus($id = 0) {
			return $this->status[$id];
		}
		
		/*
			Проверяем существует ли индекс темы
		*/
		public function isTheme($id) {
			return isset($this->theme[$id]);
		}
		
		/*
			Получаем тему
		*/
		public function getTheme($id = 0) {
			return $this->theme[$id];
		}
		
		/*
			Получаем список тем
		*/
		public function getThemes($id = null) {
			$dataResult = '';
			
			foreach($this->theme as $key => $value) {
				$dataResult .= sprintf('<option value="%s" %s>%s</option>', $key, (isset($id) ? (($id == $key) ? 'selected' : '') : ''), $value['name']);
			}
			
			return $dataResult;
		}
		
		/*
			Получаем список покупок пользователя
		*/
		public function getTransactions() {
			try {
				$sth = pdo()->prepare('SELECT * FROM money__transactions WHERE type=3 AND user_id=:user_id ORDER BY id DESC');
				$sth->execute([
					':user_id' => $_SESSION['id']
				]);
				
				if(!$sth->rowCount()) {
					return '<option selected disabled>' . getLang('library.help.errors.3') . '</option>';
				}
				
				$dataTransactions = '<option selected disabled>' . getLang('library.help.errors.4') . '</option>';
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$dataBuy = json_decode($row->data, true);
					$dataProduct = Product()->Get($dataBuy['product_id']);
					
					$dataTransactions .= sprintf('<option value="%s">%s</option>', $dataProduct['product']['id'], $dataProduct['product']['name']);
				}
				
				return $dataTransactions;
			}
			catch(Exception $e) {
				
			}
		}
		
		/*
			Получаем список своих тикетов
		*/
		public function getMyTickets() {
			try {
				$sth = pdo()->prepare('SELECT * FROM help WHERE userid=:userid ORDER BY id DESC');
				$sth->execute([
					':userid' => $_SESSION['id']
				]);
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('library.help.errors.5'));
				}
				
				tpl()->ClearCell('Tickets');
				
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('Tickets', tpl()->Set([
						'{ticket_id}' => $row->id,
						'{ticket_name}' => $this->getTheme($row->themeid)['name'],
						'{status_id}' => $row->statusid,
						'{status_text}' => $this->getStatus($row->statusid)['name']
					], tpl()->Get('elements/help/ticket')));
				}
				
				return tpl()->Execute(tpl()->GetCell('Tickets'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}
		
		/*
			Получаем список всех тикетов
		*/
		public function getAllTickets() {
			try {
				$sth = pdo()->query('SELECT * FROM help ORDER BY statusid DESC');
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('library.help.errors.5'));
				}
				
				tpl()->ClearCell('Tickets');
				
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('Tickets', tpl()->Set([
						'{ticket_id}' => $row->id,
						'{ticket_name}' => $this->getTheme($row->themeid)['name'],
						'{status_id}' => $row->statusid,
						'{status_text}' => $this->getStatus($row->statusid)['name']
					], tpl()->Get('elements/help/ticket')));
				}
				
				return tpl()->Execute(tpl()->GetCell('Tickets'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}
		
		/*
			Получаем сообщения
		*/
		public function getMessages($ticketid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM help__messages WHERE helpid=:helpid ORDER BY id ASC');
				$sth->execute([
					':helpid' => $ticketid
				]);
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('library.help.errors.6'));
				}
				
				tpl()->ClearCell('Messages');
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					tpl()->AddCell('Messages', tpl()->Set([
						'{is_support}' => $this->isRights($row->userid) ? '1' : '0',
						'{author_image}' => getAvatar($row->userid),
						'{author_id}' => $row->userid,
						'{author_name}' => getName($row->userid),
						'{message}' => htmlspecialchars_decode($row->message),
						'{date}' => getLang('getTimeAgo.date', [date('d.m.Y', $row->date), date('H:i', $row->date)])
					], tpl()->Get('elements/help/message')));
				}
				
				return tpl()->Execute(tpl()->GetCell('Messages'));
			}
			catch(Exception $e) {
				return getNotData($e->getMessage());
			}
		}
		
		/*
			Формируем сведения по Theme ID
		*/
		public function getDataTheme($ticket_id) {
			try {
				$dataTicket = $this->Get($ticket_id);
				$dataString = '';
				
				if(empty($dataTicket)) {
					return $dataString;
				}
				
				switch($dataTicket->themeid) {
					case 2: {
						$dataString = sprintf('<li>%s: <a href="%s">%s</a></li>', getLang('system.groups.seller'), $dataTicket->data, $dataTicket->data);
						break;
					}
					
					case 3: {
						$dataProduct = Product()->Get($dataTicket->data);
						
						if(empty($dataProduct)) {
							$dataString = sprintf('<li>%s: %s</li>', getLang('system.purchase'), getLang('system.errors.undefined'));
						}
						else {
							$dataString =  sprintf('<li>%s: <a href="%s">%s</a></li>', getLang('system.purchase'), $dataProduct['product']['id'], $dataProduct['product']['name']);
						}
						
						break;
					}
				}
				
				return $dataString;
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}
		
		/*
			Установка статуса тикета
		*/
		public function setStatus($ticket_id, $status_id) {
			try {
				$sth = pdo()->prepare('UPDATE help SET statusid=:status_id WHERE id=:ticket_id LIMIT 1');
				$sth->execute([
					':status_id' => $status_id,
					':ticket_id' => $ticket_id
				]);
				
				return true;
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}
		
		/*
			Проверка на закрытость тикета
		*/
		public function isClose($ticket_id) {
			if(empty($ticket_id)) {
				return true;
			}
			
			try {
				$dataTicket = $this->Get($ticket_id);
				
				if(empty($dataTicket)) {
					return true;
				}
				
				if($dataTicket->statusid != 1) {
					return false;
				}
				
				return true;
			}
			catch(Exception $e) {
				return true;
			}
		}

		/*
			Получаем кол-во непрочитанных сообщений
		*/
		public function rowCount() {
			try {
				$sth = pdo()->query('SELECT * FROM help WHERE statusid=2');
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}
	}