<?PHP
	require_once(__DIR__ . '/PHPMailer/src/Exception.php');
	require_once(__DIR__ . '/PHPMailer/src/PHPMailer.php');
	require_once(__DIR__ . '/PHPMailer/src/SMTP.php');
	
	class Mailer {
		var $mail;
		var $opts;

		public function __construct($opts, $debug = false) {
			$mail = new PHPMailer\PHPMailer\PHPMailer();
			
			if($debug) {
				$mail->Debugoutput = 'html';
				$mail->SMTPDebug = 2;
			}

			$mail->SMTPOptions = array(
				'ssl' => array(
					'verify_peer' => false,
					'verify_peer_name' => false,
					'allow_self_signed' => true
				)
			);

			$mail->isSMTP();
			$mail->SMTPSecure = PHPMailer\PHPMailer\PHPMailer::ENCRYPTION_SMTPS;
			$mail->SMTPAuth = true;
			$this->setConfigs($mail, $opts);
			$this->mail = $mail;
			$this->setFrom();

			return $mail;
		}

		public function setConfigs($mail, $opts) {
			$this->opts = $opts;

			$mail->Host = $opts->host;
			$mail->Hostname = $opts->hostname;
			$mail->Username = $opts->username;
			$mail->Password = $opts->password;
			$mail->Port = $opts->port;
		}

		public function setFrom() {
			$this->mail->setFrom($this->opts->username, $this->opts->body_name);
		}

		public function setContent($subject, $body, $altbody = null) {
			$this->mail->isHTML(true);
			$this->mail->CharSet = $this->opts->charset;
			$this->mail->Subject = $subject;
			$this->mail->Body = $body;

			if(is_null($altbody)) {
				$this->mail->AltBody = strip_tags($body);
			}
			else {
				$this->mail->AltBody = strip_tags($altbody);
			}
		}

		public function addAddress($email) {
			$this->mail->addAddress($email, $email);
		}

		public function Send() {
			$this->mail->send();
		}

		public function getMail() {
			return $this->mail;
		}

		/*
			Отправка сообщения по старой системе
		*/
		public function oldSend($to, $subject, $body) {
			$message = $body;
			$headers = "MIME-Version: 1.0\r\n";
			$headers .= "Content-type:text/html;charset=UTF-8\r\n";
			$headers .= "From: webmaster@" . $_SERVER['SERVER_NAME'] . "\r\n";
			$headers .= "Reply-To: webmaster@" . $_SERVER['SERVER_NAME'] . "\r\n";
			$headers .= "X-Mailer: PHP/" . phpversion();

			if(mail($to, $subject, $message, $headers)) {
				return true;
			}

			return false;
		}

		/*
			Проверяем работу SMTP
		*/
		public function checkSMTP($hostname, $port, $username, $password) {
			try {
				set_time_limit(5);

				$mail = new PHPMailer\PHPMailer\PHPMailer(true);
				$mail->isSMTP();
				$mail->SMTPOptions = array(
					'ssl' => array(
						'verify_peer' => false,
						'verify_peer_name' => false,
						'allow_self_signed' => true
					)
				);
				$mail->Host = $hostname;
				$mail->Port = $port;
				$mail->SMTPSecure = PHPMailer\PHPMailer\PHPMailer::ENCRYPTION_SMTPS;
				$mail->SMTPAuth = true;
				$mail->Username = $username;
				$mail->Password = $password;
				
				if($mail->smtpConnect()) {
					$mail->smtpClose();
					return true;
				}
				else {
					return false;
				}
			}
			catch (Exception $e) {
				error_log('Check SMTP: ' . $e->getMessage());
				return false;
			}
		}
	}
