<?PHP
	class Pages {
		private $Routes = [];
		protected $replace;
		
		public function __construct() {
			$this->replace = ['%appearance%' => '../appearance/' . conf()->appearance];
		}

		/*
			Запускаем Роутинг
		*/
		public function Start() {
			$arrPages = [
				$_SERVER['DOCUMENT_ROOT'] . '/../application/system/pages',
				$_SERVER['DOCUMENT_ROOT'] . '/../application/appearance/' . conf()->appearance . '/scripts/pages'
			];

			$this->loadRoutesFromDirectory($arrPages)->Execute(
				$this->removeGetParams($_SERVER['REQUEST_URI'])
			);
		}
		
		/*
			Загружаем страницы из директории или массива директорий
		*/
		public function loadRoutesFromDirectory($directories) {
			if(!is_array($directories)) {
				$directories = [$directories];
			}

			foreach($directories as $directory) {
				if(is_dir($directory)) {
					$files = new RecursiveIteratorIterator(new RecursiveDirectoryIterator($directory));
				
					foreach($files as $file) {
						if($file->isFile() && pathinfo($file->getFilename(), PATHINFO_EXTENSION) == 'json') {
							$this->loadRoutesFromFile($file->getRealPath());
						}
					}
				}
			}
			
			return $this;
		}
		
		/*
			Загружаем файл роутинга
		*/
		private function loadRoutesFromFile($filePath) {
			$jsonData = file_get_contents($filePath);
			$routes = json_decode($jsonData, true);
			
			if (json_last_error() == JSON_ERROR_NONE) {
				$this->Routes = array_merge($this->Routes, $routes);
			} else {
				die('Ошибка в файле маршрутов: ' . $filePath);
			}
		}
		
		/*
			Выполняем
		*/
		public function Execute($Uri) {
			foreach ($this->Routes as $pattern => $page) {
				$pattern = '/^' . str_replace('/', '\/', $pattern) . '$/';
				if(preg_match($pattern, $Uri, $params)) {
					global $_PAGE;
					
					$_PAGE = $page;

                    foreach($this->replace as $key => $value) {
                        $_PAGE['module'] = str_replace($key, $value, $_PAGE['module']);
                    }

					$_PAGE += ['params' => $params, 'url' => $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'], 'get' => $this->getGetParams()];

                    $pathFile = '../application/pages/' . $_PAGE['module'] . '.php';
					if(!file_exists($pathFile)) {
						die('Не найден модуль: ' . $pathFile);
					}
					
					array_shift($params);
					return require($pathFile);
				}
			}
			
			/* Если маршрут не найден, вызываем метод pageNotFound */
			$this->pageNotFound();
		}
		
		/*
			Удаляем GET параметры из URI
		*/
		private function removeGetParams($uri) {
			return parse_url($uri, PHP_URL_PATH);
		}

		/*
			Преобразуем $_GET в массив
		*/
		private function getGetParams() {
			return $_GET;
		}
		
		/*
			Выводим, что страница не найдена
		*/
		public function pageNotFound() {
			$this->loadPage('errors/page_not_found', 'Страница не найдена', [
				'{meta_name}' => 'Страница не найдена',
				'{meta_description}' => 'запрашиваемая страница не найдена',
				'{meta_keywords}' => 'запрашиваемая страница не найдена'
			]);
		}
		
		/*
			Загрузка кастомной страницы
		*/
		public function loadPage($name, $title = null, $meta = null) {
			$tpl = new Template;
			
			$tpl->Start('sample');
			
			if(!empty($title)) {
				$tpl->AddTitle($title);
			}
			
			$tpl->Content(tpl()->Get($name));
			
			if(!empty($meta)) {
				$tpl->Set($meta);
			}
			
			$tpl->Show();
		}
	}