<?PHP
	class Pays {
		var $links = [
			'ym' => 'https://yoomoney.ru/quickpay/confirm.xml',
			'fk' => 'https://pay.freekassa.com',
			'payeer' => 'https://kz.payeer.com/merchant/'
		];
		
		/* Создаем платёжную ссылку */
		public function onInsert($merchant, $data) {
			try {
				$sth = pdo()->prepare('INSERT INTO money__orders(merchant, data, amount, date) VALUES (:merchant, :data, :amount, :date)');
				$sth->execute([
					':merchant' => $merchant,
					':data' => json_encode($data),
					':amount' => $data['amount'],
					':date' => time()
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Создаем платеж */
		public function onCreate($merchant, $data) {
			try {
				$dataMerchant = Merchants();
				
				if(!$dataMerchant[$merchant]['enable']) {
					throw new Exception(getLang('library.pays.errors.0'));
				}
				
				if(isset($dataMerchant[$merchant]['min'])) {
					if($dataMerchant[$merchant]['min'] > $data['amount']) {
						throw new Exception(getLang('library.pays.errors.1', [$dataMerchant[$merchant]['min'], conf()->currency->html]));
					}
				}
				
				$insertId = $this->onInsert($merchant, $data);
				
				switch($merchant) {
					/* Freekassa */
					case 'fk': {
						$hash = md5(sprintf('%s:%s:%s:RUB:%s', $dataMerchant[$merchant]['merchant_id'], $data['amount'], $dataMerchant[$merchant]['public_key'], $insertId));
						$payLink = sprintf('%s?m=%s&oa=%s&o=%s&s=%s&currency=RUB', $this->links[$merchant], $dataMerchant[$merchant]['merchant_id'], $data['amount'], $insertId, $hash);
						Result(['Alert' => 'Success', 'Message' => getLang('system.redirect'), 'payLink' => $payLink]);
						
						break;
					}
					
					/* ЮMoney */
					case 'ym': {
						$payId = str_pad($insertId, 7, "0", STR_PAD_LEFT) . substr($insertId, 5);
						
						$params = [
							'receiver' => $dataMerchant[$merchant]['wallet_id'],
							'formcomment' => '',
							'short-dest' => '',
							'label' => $payId,
							'quickpay-form' => 'donate',
							'targets' => getLang('library.pays.message', [$_SERVER['SERVER_NAME']]),
							'sum' => $data['amount'],
							'comment' => '',
							'need-fio' => 'false',
							'need-email' => 'false',
							'need-phone' => 'false',
							'need-address' => 'false',
							'paymentType' => '',
							'successURL' => $this->getLinkResult('success')
						];
						
						$postLink = $this->doPay($this->links[$merchant], $params);
						Result(['Alert' => 'Success', 'Message' => getLang('system.redirect'), 'payForm' => $postLink]);
						
						break;
					}
					
					/* Merchant001 */
					case 'merchant001': {
						$payId = str_pad($insertId, 7, "0", STR_PAD_LEFT) . substr($insertId, 5);
						
						try {
							$Merchant001 = new Merchant001($dataMerchant[$merchant]['apiKey']);
							$dataTransaction = $Merchant001->onCreate(
								$data['amount'],
								$dataMerchant[$merchant]['currency'],
								true,
								$dataMerchant[$merchant]['provider'],
								$payId,
								$this->getLinkResult('success'),
								$this->getLinkResult('fail')
							);
							
							$Merchant001->errorProcessing($dataTransaction);
							
							switch($dataTransaction['status']) {
								case 'CREATED': {
									Result(['Alert' => 'Success', 'Message' => getLang('system.redirect'), 'payLink' => $dataTransaction['paymentUrl']]);
									
									break;
								}
							}
						}
						catch(Exception $e) {
							throw new Exception($e->getMessage());
						}
						
						break;
					}

					/* Nicepay */
					case 'nicepay': {
						try {
							$classNicepay = new Nicepay;
							Result(['Alert' => 'Success', 'Message' => getLang('system.redirect'), 'payLink' => $classNicepay->onCreate($insertId, $data['amount'])]);
						}
						catch(Exception $e) {
							throw new Exception($e->getMessage());
						}
						
						break;
					}

					/* LinePay */
					case 'linepay': {
						try {
							$classLinePay = new LinePay;
							Result(['Alert' => 'Success', 'Message' => getLang('system.redirect'), 'payLink' => $classLinePay->onCreate($insertId, $data['amount'])]);
						}
						catch(Exception $e) {
							throw new Exception($e->getMessage());
						}

						break;
					}

					/* Skinify */
					case 'skinify': {
						try {
							$classSkinify = new Skinify;
							Result(['Alert' => 'Success', 'Message' => getLang('system.redirect'), 'payLink' => $classSkinify->onCreate($insertId, $data['amount'])]);
						}
						catch(Exception $e) {
							throw new Exception($e->getMessage());
						}

						break;
					}

					/* Payeer */
					case 'payeer': {
						try {
							$dataUser = users()->Get($_SESSION['id']);
							$mDesc = getLang('library.pays.message_payeer', $insertId, $dataUser->email, $_SERVER['SERVER_NAME']);

							$params = [
								'm_shop' => $dataMerchant[$merchant]['shop_id'],
								'm_orderid' => $insertId,
								'm_amount' => number_format($data['amount'], 2, '.', ''),
								'm_curr' => $dataMerchant[$merchant]['currency'],
								'm_desc' => base64_encode($mDesc),
								'm_sign' => strtoupper(hash('sha256', implode(':', [
									$dataMerchant[$merchant]['shop_id'],
									$insertId,
									number_format($data['amount'], 2, '.', ''),
									$dataMerchant[$merchant]['currency'],
									base64_encode($mDesc),
									$dataMerchant[$merchant]['secret_key']
								])))
							];

							$postLink = $this->doPay($this->links[$merchant], $params);
							Result(['Alert' => 'Success', 'Message' => getLang('system.redirect'), 'payForm' => $postLink]);
						}
						catch(Exception $e) {
							throw new Exception($e->getMessage());
						}

						break;
					}

					/* NowPayments */
					case 'nowpayments': {
						$apiKey = $dataMerchant[$merchant]['apiKey'];
						$publicKey = $dataMerchant[$merchant]['publicKey'];

						$nowPayments = new NowPayments($publicKey, $apiKey);
						$invoiceData = $nowPayments->createInvoice($data['amount'], $dataMerchant[$merchant]['currency'], $insertId, '');

						if(isset($invoiceData['invoice_url'])) {
							Result([
								'Alert' => 'Success',
								'Message' => getLang('system.redirect'),
								'payLink' => $invoiceData['invoice_url']
							]);
						}
						else {
							throw new Exception($invoiceData['message']);
						}
						
						break;
					}

					/* Cryptomus */
					case 'cryptomus': {
						$merchantData = $dataMerchant[$merchant];
						
						try {
							$Cryptomus = new Cryptomus($merchantData['apiKey'], $merchantData['merchant_id']);
							$result = $Cryptomus->onPay($data['amount'], $insertId);

							Result([
								'Alert' => 'Success',
								'Message' => getLang('system.redirect'),
								'payLink' => $result['url']
							]);
						}
						catch(Exception $e) {
							throw $e;
						}

						break;
					}
				}
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Получение ссылки на возврат */
		public function getLinkResult($type) {
			return sprintf('https://%s/pay/index?type=%s', $_SERVER['SERVER_NAME'], $type);
		}
		
		/* Создаем переход на платеж */
		public function doPay($uri, $params) {
			$tmp = sprintf('<form id="payForm" method="POST" action="%s">', $uri);
			
			foreach($params as $name => $value) {
				$tmp .= sprintf('<input type="hidden" name="%s" value="%s">', $name, $value);
			}
			
			$tmp .= '</form>';
			$tmp .= '<script>document.getElementById("payForm").submit();</script>';
			
			return $tmp;
		}
		
		/* Получаем данные об заказе */
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM money__orders WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/* Изменяем статус заказа */
		public function UpdateStatus($id, $status) {
			try {
				$sth = pdo()->prepare('UPDATE money__orders SET status=:status WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id,
					':status' => $status
				]);
				
				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Проверка на проведение платежа */
		public function IsPayed($orderId) {
			try {
				$dataOrder = $this->Get($orderId);
				
				if(is_null($dataOrder)) {
					return null;
				}
				
				if($dataOrder->status == 1 || $dataOrder->status == 3) {
					return true;
				}
				
				return false;
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Получаем модальное окно */
		public function getModal() {
			tpl()->ClearCell('merchant_items');

			foreach(Merchants() as $key => $merchant) {
				if(empty($merchant['is_view']) && $merchant['enable']) {
					tpl()->AddCell('merchant_items', tpl()->Set([
						'{key}' => $key,
						'{name}' => $merchant['name'],
						'{desc}' => isset($merchant['desc']) ? $merchant['desc'] : '',
						'{currency}' => isset($merchant['currency']) ? $merchant['currency'] : 'rub',
						'{min_price_info}' => getLang('pay.modal.step.one.minimal_payments', [$merchant['min'], conf()->currency->html])
					], tpl()->Get('/elements/modals/pay/item')));
				}
			}

			return tpl()->Execute(tpl()->Set([
				'{items}' => tpl()->GetCell('merchant_items')
			], tpl()->Get('/elements/modals/pay')));
		}
	}