<?PHP
	require_once(__DIR__ . '/cryptomus/Client.php');
	require_once(__DIR__ . '/cryptomus/Payment.php');
	require_once(__DIR__ . '/cryptomus/RequestBuilder.php');
	require_once(__DIR__ . '/cryptomus/RequestBuilderException.php');

	class Cryptomus {
		private $payment;
		private $merchantData;

		public function __construct($apiKey, $merchantId) {
			$this->payment = \Cryptomus\Api\Client::payment($apiKey, $merchantId);
			$this->merchantData = Merchants()['cryptomus'];
		}

		public function getServices() {
			return $this->payment->services();
		}

		public function getMinAmount() {
			$data = $this->getData($this->merchantData['convert'], $this->merchantData['network']);
			return $data['limit']['min_amount'];
		}

		public function getData($cryptoName, $network) {
			$data = $this->getServices();

			foreach($data as $key => $val) {
				if($val['currency'] == $cryptoName && $val['network'] == $network) {
					return $val;
				}
			}

			return null;
		}

		public function onPay($amount, $insertId) {
			$Currency = new Currency;
			$amount = $Currency->cryptoConvert($amount, $this->merchantData['convert']);

			if($amount == false) {
				throw new Exception(getLang('library.pays.errors.2'));
			}

			if($amount < $this->getMinAmount()) {
				throw new Exception(getLang('library.pays.errors.1', [
					$amount, $this->merchantData['convert']
				]));
			}

			$Pays = new Pays;

			$data = [
				'amount' => (string)$amount,
				'currency' => $this->merchantData['convert'],
				'network' => $this->merchantData['network'],
				'order_id' => (string)$insertId,
				'url_return' => 'https://' . $_SERVER['SERVER_NAME'],
				'url_callback' => $Pays->getLinkResult('confirm'),
				'url_success' => $Pays->getLinkResult('success'),
				'is_payment_multiple' => false,
				'lifetime' => '7200',
				'to_currency' => 'USDT'
			];

			return $this->payment->create($data);
		}
	}