<?PHP
	class Merchant001 {
		private $links = ['https://api.merchant001.io/v1/transaction/merchant'];
		private $apiKey;
		
		public function __construct($token) {
			$this->apiKey = $token;
		}
		
		/* Удаляем ненужные переменные */
		public function isNullClear(array $data, array $params) {
			foreach($params as $key) {
				if(is_null($data[$key])) {
					unset($data[$key]);
				}
			}
			
			return $data;
		}
		
		/* Создаем транзакцию */
		public function onCreate($amount = 10, $currency = 'RUB', $isPartnerFee = true, $selectedProvider = null, $invoiceId = null, $redirectUrl = null, $cancelUrl = null) {
			$data = [
				'isPartnerFee' => $isPartnerFee,
				'pricing' => [
					'local' => [
						'amount' => $amount,
						'currency' => $currency
					]
				],
				'selectedProvider' => $selectedProvider,
				'invoiceId' => $invoiceId,
				'redirectUrl' => $redirectUrl,
				'cancelUrl' => $cancelUrl
			];
			
			$this->isNullClear($data, ['selectedProvider', 'invoiceId', 'redirectUrl', 'cancelUrl']);
			$headers = ['Authorization: Bearer ' . $this->apiKey, 'Content-Type: application/x-www-form-urlencoded'];
			
			$ch = curl_init($this->links[0]);
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data));
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			
			$response = curl_exec($ch);
			
			if(curl_errno($ch)) {
				throw new Exception('Error: ' . curl_error($ch));
			}
			
			curl_close($ch);
			return json_decode($response, true);
		}
		
		/* Обработка ошибки */
		public function errorProcessing($result) {
			if(isset($result['code'])) {
				switch($result['code']) {
					case 'INVOICE_ID_DUPLICATED': {
						throw new Exception($result['message']);
					}
				}
			}
			
			if(isset($result['statusCode'])) {
				switch($result['statusCode']) {
					case 400: {
						throw new Exception($result['message']);
					}
					
					case 500: {
						throw new Exception($result['message']);
					}
				}
			}
		}
		
		/* Результат */
		public function sendResult($status, $message) {
			http_response_code($status);
			die(json_encode(['message' => $message]));
		}
		
		public function isSignature($data) {
			if(empty($data['signature'])) {
				throw new Exception('The signature is missing');
			}
			
			$payLoad = str_replace(['-', '_'], ['+', '/'], $data['signature']);
			$payLoad = base64_decode($payLoad);
			$payLoad = json_decode($this->getTextWithBraces($payLoad), true);
			
			if($data['transaction']['id'] == $payLoad['id'] && $this->apiKey == $payLoad['apiKey']) {
				return true;
			}
			
			return false;
		}
		
		public function getTextWithBraces($in) {
			$startPos = strpos($in, '{');
			if($startPos == false) {
				return '';
			}
			
			$endPos = strrpos($in, '}');
			if($endPos == false) {
				return '';
			}
			
			$result = substr($in, $startPos, $endPos - $startPos + 1);
			$lengthToRemove = strlen('{"alg":"HS256","typ":"JWT"}');
			$output = substr_replace($result, '', 0, $lengthToRemove);
			
			return $output;
		}
	}