<?PHP
	class Nicepay {
		private $dataMerchant;
		private $link = 'https://nicepay.io/public/api/payment';
		private $ips = [
			'121.127.46.166'
		];

		public function __construct() {
			$this->dataMerchant = Merchants()['nicepay'];
		}

		/* Создаем платежную ссылку */
		public function onCreate($orderId, $amount) {
			try {
				$classPays = new Pays;

				$dataTransaction = [
					'merchant_id' => $this->dataMerchant['merchant_id'],
					'secret' => $this->dataMerchant['secret'],
					'order_id' => $orderId,
					'customer' => 'user id ' . $_SESSION['id'],
					'amount' => intval($amount * 100),
					'currency' => $this->dataMerchant['currency'],
					'description' => getLang('library.pays.message', $_SERVER['SERVER_NAME']),
					'success_url' => $classPays->getLinkResult('success'),
					'fail_url' => $classPays->getLinkResult('fail')
				];
				
				$postResult = $this->onPost($dataTransaction);
				$jsonResult = json_decode($postResult);

				if(json_last_error() == JSON_ERROR_NONE) {
					switch($jsonResult->status) {
						case 'success': return $jsonResult->data->link;
						default: throw new Exception($jsonResult->data->message);
					}
				}
				else {
					throw new Exception(json_last_error_msg());
				}
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}

		/* Выполнение поручения */
		public function onExecute($params) {
			if(!in_array(users()->getRemote(), $this->ips)) {
				throw new Exception('Direct call attempt');
			}

			$hash = $params['hash'];
			unset($params['hash'], $params['/pay/index'], $params['merchant'], $params['type']);
			ksort($params, SORT_STRING);

			array_push($params, $this->dataMerchant['secret']);
			$hash_string = implode('{np}', $params);
			$hash_sha256 = hash('sha256', $hash_string);

			if($hash != $hash_sha256) {
				throw new Exception('Invalid hash');
			}

			switch($params['result']) {
				case 'success': {
					$orderId = $params['order_id'];

					$classPays = new Pays;
					$isPayed = $classPays->IsPayed($orderId);

					if(is_null($isPayed)) {
						throw new Exception('The order was not found');
					}
					
					if($isPayed) {
						throw new Exception('The operation has already been performed');
					}

					$dataOrder = $classPays->Get($orderId);
			
					if(is_null($dataOrder)) {
						throw new Exception('The payment was not found');
					}

					$dataOrder = json_decode($dataOrder->data);

					$classUsersBalance = new UsersBalance;
					$classUsersTransactions = new UsersTransactions;

					if($classUsersBalance->addBalance($dataOrder->userid, $dataOrder->amount)) {
						$classUsersTransactions->Add($dataOrder->userid, ['amount' => $dataOrder->amount, 'type' => 1, 'status' => 1]);
						$classPays->UpdateStatus($orderId, 1);
					}
					else {
						$classUsersTransactions->Add($dataOrder->userid, ['amount' => $dataOrder->amount, 'type' => 1, 'status' => 3]);
						$classPays->UpdateStatus($orderId, 3);
					}
					
					return true;
					break;
				}

				case 'error': {
					throw new Exception('Error');
					break;
				}

				default: {
					throw new Exception('Empty');
				}
			}
		}

		/* Отправка POST запроса */
		public function onPost($PostFields) {
			$ch = curl_init($this->link);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($PostFields));
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($ch, CURLOPT_HEADER, false);
			curl_setopt($ch, CURLOPT_HTTPHEADER, ['Content-Type: application/json']);
			$result = curl_exec($ch);
			curl_close($ch);
	
			return $result;
		}
	}