<?PHP
	class Skinify {
		private $dataMerchant;
		private $linkPay = 'https://skinify.io/api/create-deposit';

		public function __construct() {
			$this->dataMerchant = Merchants()['skinify'];
		}
		
		public function onCreate($orderId, $amount) {
			try {
				$result = $this->onPost($this->linkPay, ['deposit_id' => $orderId, 'min_amount' => $amount, 'currency' => $this->dataMerchant['currency']], $this->dataMerchant['token']);
				$result = json_decode($result, true);

				if(empty($result['status'])) {
					throw new Exception('Contact support');
				}

				switch($result['status']) {
					case 'success': {
						return $result['url'];
						break;
					}

					default: throw new Exception($result['error_message']);
				}
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}

		public function onExecute($params) {
			try {
				if(empty($params['status']) || $params['status'] == 'failed') {
					throw new Exception('Payment error, reset to zero');
				}

				if($params['token_md5'] != md5($this->dataMerchant['token'])) {
					throw new Exception('Invalid hash');
				}

				$classPays = new Pays;
				$isPayed = $classPays->IsPayed($params['deposit_id']);

				if(is_null($isPayed)) {
					throw new Exception('The order was not found');
				}

				if($isPayed) {
					throw new Exception('The operation has already been performed');
				}

				$dataOrder = $classPays->Get($params['deposit_id']);

				if(is_null($dataOrder)) {
					throw new Exception('The payment was not found');
				}

				$dataOrder = json_decode($dataOrder->data);

				$classUsersBalance = new UsersBalance;
				$classUsersTransactions = new UsersTransactions;
				
				if($dataOrder->amount > $params['amount']) {
					throw new Exception('The payment amount is lower than the stated amount');
				}

				if($classUsersBalance->addBalance($dataOrder->userid, $dataOrder->amount)) {
					$classUsersTransactions->Add($dataOrder->userid, ['amount' => $dataOrder->amount, 'type' => 1, 'status' => 1]);
					$classPays->UpdateStatus($params['deposit_id'], 1);
				}
				else {
					$classUsersTransactions->Add($dataOrder->userid, ['amount' => $dataOrder->amount, 'type' => 1, 'status' => 3]);
					$classPays->UpdateStatus($params['deposit_id'], 3);
				}

				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/* Отправляем POST запрос */
		public function onPost($site, $PostFields, $token) {
			$ch = curl_init($site);
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $PostFields);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			$headers = ['Token: ' . $token];
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			$result = curl_exec($ch);
			curl_close($ch);
		
			return $result;
		}

		/* Получаем currency системы */
		public function onGet($site, $token) {
			$ch = curl_init($site);
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
			$headers = ['Token: ' . $token];
			curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
			$result = curl_exec($ch);
			curl_close($ch);
		
			return $result;
		}
	}