<?PHP
	class Users {
		/*
			** Лимиты
			* $_Sizes['login'][0] - минимальная длина
			* $_Sizes['login'][1] - максимальная длина
		*/
		protected $_Sizes = [
			'login' => [4, 64]
		];
		
		/*
			Минимальный возраст регистрации на сайте
		*/
		protected $minYears = 14;
		
		/*
			Регистрация учётной записи
		*/
		public function onRegister($_ARRAY = []) {
			try {
				/* Начало::Проверка логина */
				if(empty($_ARRAY['login'])) {
					throw new Exception(getLang('library.users.errors.0'));
				}
				
				$_ARRAY['login'] = $this->replaceSpecial($_ARRAY['login'], '/[^\p{L}\d]/u');
				$strLen = mb_strlen($_ARRAY['login']);
				
				if($strLen < $this->_Sizes['login'][0]) {
					throw new Exception(getLang('library.users.errors.1', [$this->_Sizes['login'][0]]));
				}
				
				if($strLen > $this->_Sizes['login'][1]) {
					throw new Exception(getLang('library.users.errors.2', [$this->_Sizes['login'][1]]));
				}
				
				if($this->isLogin($_ARRAY['login'])) {
					throw new Exception(getLang('library.users.errors.3'));
				}
				/* Конец::Проверка логина */
				
				/* Начало::Проверка почты */
				if(empty($_ARRAY['email'])) {
					throw new Exception(getLang('library.users.errors.4'));
				}
				
				if(!filter_var($_ARRAY['email'], FILTER_VALIDATE_EMAIL)) {
					throw new Exception(getLang('library.users.errors.5'));
				}
				
				if($this->isEmail($_ARRAY['email'])) {
					throw new Exception(getLang('library.users.errors.6'));
				}
				/* Конец::Проверка почты */
				
				$lTime = time();
				
				$sth = pdo()->prepare('INSERT INTO `users`(`login`, `email`, `password`, `rights`, `language`, `roleid`, `balance`, `date_online`, `date_register`, `settings`) VALUES (:login, :email, :password, :rights, :language, :roleid, :balance, :date_online, :date_register, :settings)');
				$sth->execute([
					':login' => $_ARRAY['login'],
					':email' => $_ARRAY['email'],
					':password' => password_hash($_ARRAY['password'], PASSWORD_DEFAULT),
					':rights' => $this->Rows() ? 'z' : 'abcdefghijklmnopqrstuvwxyz',
					':language' => empty($_SESSION['language']) ? conf()->language : $_SESSION['language'],
					':roleid' => $this->Rows() ? 4 : 1,
					':balance' => 0,
					':date_online' => $lTime,
					':date_register' => $lTime,
					':settings' => json_encode([
						'view_orders' => '1',
						'messages' => '1'
					])
				]);

				$_SESSION['id'] = pdo()->lastInsertId();
				$SafetyLogin = new SafetyLogin;
				$SafetyLogin->Add();

				if(conf()->mailer->enabled) {
					$pathFile = pathAsynchronous('mail/register.php');
				
					if($pathFile === false) {
						return true;
					}
					
					$scriptPath = pathAsynchronous('mail/register.php');
					AsyncPHP::run($scriptPath, [$_ARRAY['email']]);
				}				

				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Авторизация учётной записи
		*/
		public function onAuth($_ARRAY = []) {
			try {
				$sth = pdo()->prepare('SELECT * FROM `users` WHERE `login`=:login OR `email`=:login LIMIT 1');
				$sth->execute([
					':login' => $_ARRAY['login']
				]);
				
				if(!$sth->rowCount()) {
					throw new Exception(getLang('library.users.errors.7'));
				}
				
				$dataUser = $sth->fetch(PDO::FETCH_OBJ);
				if(!password_verify($_ARRAY['password'], $dataUser->password)) {
					throw new Exception(getLang('library.users.errors.8'));
				}
				
				$_SESSION['id'] = $dataUser->id;
				$this->updateOnline($_SESSION['id']);
				
				$SafetyLogin = new SafetyLogin;
				$SafetyLogin->Add();

				return true;
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Получение данных
		*/
		public function Get($userid) {
			try {
				if(!$this->isValid($userid)) {
					return null;
				}
				
				$sth = pdo()->prepare('SELECT * FROM `users` WHERE `id`=:id LIMIT 1');
				$sth->execute([
					':id' => $userid
				]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/*
			Выход из учётной записи
		*/
		public function onLogout() {
			if(isset($_SESSION['id'])) {
				$this->updateOnline($_SESSION['id'], true);
				unset($_SESSION['id']);
			}
			
			return true;
		}
		
		/*
			Проверка наличия логина
		*/
		public function isLogin($login) {
			try {
				$sth = pdo()->prepare('SELECT * FROM `users` WHERE `login`=:login LIMIT 1');
				$sth->execute([
					':login' => $login
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Проверка наличия E-mail адреса
		*/
		public function isEmail($email) {
			try {
				$sth = pdo()->prepare('SELECT * FROM `users` WHERE `email`=:email LIMIT 1');
				$sth->execute([
					':email' => $email
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Проверка наличия пользователя
		*/
		public function isValid($userid) {
			try {
				$sth = pdo()->prepare('SELECT * FROM `users` WHERE `id`=:id LIMIT 1');
				$sth->execute([
					':id' => $userid
				]);
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Количество учётных записей
		*/
		public function Rows() {
			try {
				$sth = pdo()->query('SELECT * FROM `users`');
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
		/*
			Внутренние функции
		*/
		public function replaceSpecial($string, $preg = null) {
			/* Чистим специальные символы и пробелы */
			return preg_replace(isset($preg) ? $preg : '/[^\p{L}]/u', '', $string);
		}
		
		/*
			Обновление Активности
		*/
		public function updateOnline($userid, $logout = false) {
			try {
				if(!$this->isValid($userid)) {
					return false;
				}
				
				$sth = pdo()->prepare('UPDATE `users` SET `date_online`=:date_online WHERE `id`=:userid LIMIT 1');
				$sth->execute([
					':userid' => $userid,
					':date_online' => $logout ? time() : strtotime("+15 minutes")
				]);
				
				return true;
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Проверка активности пользователя
		*/
		public function isOnline($userid) {
			$dataUser = $this->Get($userid);
			
			if(empty($dataUser)) {
				return false;
			}
			
			if($dataUser->date_online <= time()) {
				return false;
			}
			
			return true;
		}
		
		/*
			Получаем IP пользователя
		*/
		public function getRemote() {
			if(isset($_SERVER['HTTP_CLIENT_IP']) && !empty($_SERVER['HTTP_CLIENT_IP'])) {
				$ip = $_SERVER['HTTP_CLIENT_IP'];
			}
			elseif(isset($_SERVER['HTTP_X_FORWARDED_FOR']) && !empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
				$ipList = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
				$ip = trim(end($ipList));
			}
			elseif(isset($_SERVER['HTTP_X_FORWARDED']) && !empty($_SERVER['HTTP_X_FORWARDED'])) {
				$ip = $_SERVER['HTTP_X_FORWARDED'];
			}
			elseif(isset($_SERVER['HTTP_X_CLUSTER_CLIENT_IP']) && !empty($_SERVER['HTTP_X_CLUSTER_CLIENT_IP'])) {
				$ip = $_SERVER['HTTP_X_CLUSTER_CLIENT_IP'];
			}
			elseif(isset($_SERVER['HTTP_FORWARDED_FOR']) && !empty($_SERVER['HTTP_FORWARDED_FOR'])) {
				$ip = $_SERVER['HTTP_FORWARDED_FOR'];
			}
			elseif(isset($_SERVER['HTTP_FORWARDED']) && !empty($_SERVER['HTTP_FORWARDED'])) {
				$ip = $_SERVER['HTTP_FORWARDED'];
			}
			elseif(isset($_SERVER['HTTP_X_REAL_IP']) && !empty($_SERVER['HTTP_X_REAL_IP'])) {
				$ip = $_SERVER['HTTP_X_REAL_IP'];
			}
			elseif(isset($_SERVER['REMOTE_ADDR']) && !empty($_SERVER['REMOTE_ADDR'])) {
				$ip = $_SERVER['REMOTE_ADDR'];
			}
			else {
				$ip = 'UNKNOWN';
			}

			return $ip;
		}

		/*
			Изменение аватара
		*/
		public function onUpdateAvatar($userid, $image) {
			try {
				if(!$this->isValid($userid)) {
					throw new Exception(getLang('library.users.errors.9'));
				}

				$sth = pdo()->prepare('UPDATE users SET image=:image WHERE id=:id LIMIT 1');
				$sth->execute([
					':image' => $image,
					':id' => $userid
				]);

				return true;
			}
			catch(Exception $e) {
				return $e->getMessage();
			}
		}

		/*
			Получаем конфигурации пользователя
		*/
		public function getSettings($userId) {
			try {
				$data = $this->Get($userId);
				
				if(is_null($data)) {
					return null;
				}

				$settings = json_decode($data->settings, true);
				return $settings;
			}
			catch(Exception $e) {
				return null;
			}
		}

		/*
			Изменяем конфигурации
		*/
		public function updateSettings($userId, $data) {
			try {
				$configs = $this->getSettings($userId);
				if(is_null($configs)) {
					$configs = [];
				}

				foreach($data as $key => $value) {
					$configs[$key] = $value;
				}

				$jsonConfigs = json_encode($configs, JSON_UNESCAPED_UNICODE);
				$this->Update($userId, ['settings' => $jsonConfigs]);
			}
			catch (Exception $e) {
				return null;
			}
		}		

		/*
			Обновляем данные пользователя
		*/
		public function Update($userId, $data) {
			try {
				$setPart = [];
				$params = [];
				
				foreach($data as $key => $value) {
					if($value == null) {
						$setPart[] = "`$key` = NULL";
					}
					else {
						$setPart[] = "`$key` = :$key";
						$params[":$key"] = $value;
					}
				}
		
				$sql = "UPDATE `users` SET " . implode(", ", $setPart) . " WHERE `id` = :userId";
				$params[':userId'] = $userId;
				
				$stmt = pdo()->prepare($sql);
				$stmt->execute($params);
		
				return true;
			}
			catch (Exception $e) {
				error_log("Ошибка обновления пользователя с ID $userId: " . $e->getMessage());
				return false;
			}
		}

		/* Блокировка пользоваетля */
		public function setBanned($user_id) {
			try {
				// Выставялем роль
				$this->Update($user_id, [
					'roleid' => 5,
					'date_online' => time()
				]);

				// Отключаем весь товар
				$sth = pdo()->prepare('UPDATE `product` SET `enable`=2 WHERE `author_id`=:user_id');
				$sth->execute([':user_id' => $user_id]);

				return true;
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return false;
			}
		}

		/* Генератор пароля */
		function generatePassword($length = 12) {
			$chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()-_=+';
			$password = '';
			$maxIndex = strlen($chars) - 1;
			
			for($i = 0; $i < $length; $i++) {
				$password .= $chars[random_int(0, $maxIndex)];
			}

			return $password;
		}
	}