<?PHP
	class UsersBalance extends Users {
		/*
			Получение баланса пользователя
		*/
		public function getBalance($userid) {
			if(!$this->isValid($userid)) {
				return 0;
			}
			
			$dataUser = $this->Get($userid);
			return $dataUser->balance;
		}
		
		/*
			Установка баланса
		*/
		public function setBalance($userid, $balance) {
			if(!$this->isValid($userid)) {
				return false;
			}
			
			try {
				$sth = pdo()->prepare('UPDATE `users` SET `balance`=:balance WHERE `id`=:id LIMIT 1');
				$sth->execute([
					':balance' => $balance,
					':id' => $userid
				]);
				
				return true;
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Добавление к балансу
		*/
		public function addBalance($userid, $balance) {
			if(!$this->isValid($userid)) {
				return false;
			}
			
			try {
				$balance = reFormatPrice($balance);
				$dataUserBalance = $this->getBalance($userid);
				$dataUserBalance += $balance;
				
				$this->setBalance($userid, $dataUserBalance);
				return true;
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Минусуем баланс
		*/
		public function minBalance($userid, $amount) {
			if(!$this->isValid($userid)) {
				return false;
			}
		
			try {
				$amount = reFormatPrice($amount);
				if(!$this->isBalance($userid, $amount)) {
					return false;
				}
		
				$dataUserBalance = $this->getBalance($userid);
				
				if($dataUserBalance - $amount < 0) {
					return false;
				}
		
				$newBalance = $dataUserBalance - $amount;
				$this->setBalance($userid, $newBalance);
		
				return true;
			}
			catch (Exception $e) {
				error_log("Error in minBalance for user {$userid}: " . $e->getMessage());
				return false;
			}
		}
		
		/*
			Проверяем достаточно ли у пользователя баланса
		*/
		public function isBalance($userid, $balance) {
			if(!$this->isValid($userid)) {
				return false;
			}
			
			if($balance < 0) {
				return false;
			}
			
			try {
				$dataUserBalance = $this->getBalance($userid);
				
				if($dataUserBalance >= $balance) {
					return true;
				}
				
				return false;
			}
			catch(Exception $e) {
				return false;
			}
		}

		/*
			Удаляем чейку из заморозки
		*/
		public function removeFrozen($userid, $trans_id) {
			try {
				$user = users()->Get($userid);

				if(is_null($user->balance_frozen)) {
					return true;
				}

				$frozen_balance = json_decode($user->balance_frozen, true);
				$balance_modified = false;

				foreach($frozen_balance as $key => $value) {
					if($value['trans_id'] == $trans_id) {
						unset($frozen_balance[$key]);
						$balance_modified = true;
					}
				}
				
				if(!$balance_modified) {
					return false;
				}

				$frozen_balance = count($frozen_balance) ? json_encode($frozen_balance) : null;
				$sth = pdo()->prepare('UPDATE `users` SET balance_frozen=:balance_frozen WHERE id=:id LIMIT 1');
				$sth->execute([
					':balance_frozen' => $frozen_balance,
					':id' => $userid
				]);

				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/*
			Добавление замороженного баланса
		*/
		public function addFrozen($userid, $trans_id, $price) {
			try {
				$user = Users()->Get($userid);

				$unfrozenDays = conf()->unfrozen_days;
				$unfrozenDateUnix = strtotime("+$unfrozenDays days");

				if(is_null($user->balance_frozen)) {
					$frozen_balance[] = [
						'trans_id' => $trans_id,
						'price' => reFormatPrice($price),
						'date_end' => $unfrozenDateUnix
					];
				}
				else {
					$frozen_balance = json_decode($user->balance_frozen, true);

					$frozen_balance[] = [
						'trans_id' => $trans_id,
						'price' => reFormatPrice($price),
						'date_end' => $unfrozenDateUnix
					];
				}
		
				$sth = pdo()->prepare('UPDATE users SET balance_frozen = :balance_frozen WHERE id = :id LIMIT 1');
				$sth->execute([':balance_frozen' => json_encode($frozen_balance), ':id' => $userid]);
				
				return pdo()->lastInsertId();
			} catch (Exception $e) {
				throw $e;
			}
		}
		
		/*
			Получение кол-во замороженных средств
		*/
		public function getFrozen($userid) {
			try {
				$user = Users()->Get($userid);
				$frozen_balance = 0;

				if(is_null($user->balance_frozen)) {
					return $frozen_balance;
				}

				$json_balance = json_decode($user->balance_frozen, true);
				foreach($json_balance as $key => $value) {
					$frozen_balance += $value['price'];
				}

				return $frozen_balance;
			}
			catch(Exception $e) {
				return 0;
			}
		}

		/* Получаем замороженные средства по номеру транзакции */
		public function getFrozenFromTransId($user_id, $trans_id) {
			try {
				$User = Users()->Get($user_id);

				if(is_null($User->balance_frozen)) {
					return null;
				}

				$data = json_decode($User->balance_frozen, true);
				foreach($data as $key => $value) {
					if($value['trans_id'] == $trans_id) {
						return $data[$key];
						break;
					}
				}

				return null;
			}
			catch(Exception $e) {
				return null;
			}
		}

		/* Получаем дату авто-закрытия сделки */
		public function getFrozenEndTime($data) {
			if(empty($data)) {
				return [
					'label' => getLang('getFrozenEndTime.0'),
					'class' => 'text-success'
				];
			}

			$now = time();

			if(isset($data['dispute']) && $data['dispute'] == true) {
				return [
					'label' => getLang('getFrozenEndTime.1'),
					'class' => 'text-danger'
				];
			}
			elseif($now < $data['date_end']) {
				return [
					'label' => date('d.m.Y H:i', $data['date_end']),
					'class' => ''
				];
			}

			return [
				'label' => getLang('getFrozenEndTime.2'),
				'class' => 'text-warning'
			];
		}
	}