<?PHP
	class UsersMessages {
		/*
			Получаем количество непрочитанных сообщений
		*/
		public function rowNotReady($user_dialog_id = null) {
			if(empty($_SESSION['id'])) {
				return 0;
			}

			try {
				if(isset($user_dialog_id)) {
					$sth = pdo()->prepare('SELECT * FROM users__messages WHERE recipient_id=:recipient_id AND sender_id=:sender_id AND ready=0');
					$sth->execute([':recipient_id' => $_SESSION['id'], ':sender_id' => $user_dialog_id]);
				}
				else {
					$sth = pdo()->prepare('SELECT * FROM users__messages WHERE recipient_id=:recipient_id AND ready=0');
					$sth->execute([':recipient_id' => $_SESSION['id']]);
				}
				
				return $sth->rowCount();
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
		/*
			Отправляем сообщение
		*/
		public function Send($_ARRAY = []) {
			if(empty($_ARRAY['recipient_id'])) {
				throw new Exception(getLang('library.users.messages.errors.0'));
			}
			
			if(!users()->isValid($_ARRAY['recipient_id'])) {
				throw new Exception(getLang('library.users.messages.errors.1'));
			}
			
			if(empty($_ARRAY['content']) || mb_strlen($_ARRAY['content']) < 1) {
				throw new Exception(getLang('library.users.messages.errors.2'));
			}
			
			try {
				$sth = pdo()->prepare('INSERT INTO `users__messages`(`sender_id`, `recipient_id`, `content`, `system`, `date`) VALUES (:sender_id, :recipient_id, :content, :system, :date)');
				$sth->execute([
					':sender_id' => isset($_ARRAY['user_id']) ? $_ARRAY['user_id'] : $_SESSION['id'],
					':recipient_id' => $_ARRAY['recipient_id'],
					':content' => $_ARRAY['content'],
					':system' => isset($_ARRAY['system']) ? $_ARRAY['system'] : 0, ':date' => time()
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/*
			Отправляем картинку
		*/
		public function SendFile($recipient_id, $data) {
			try {
				if(empty($recipient_id)) {
					throw new Exception(getLang('library.users.messages.errors.3'));
				}

				if(!users()->isValid($recipient_id)) {
					throw new Exception(getLang('library.users.messages.errors.4'));
				}

				$sth = pdo()->prepare('INSERT INTO users__messages(sender_id, recipient_id, content, filename, date) VALUES (:sender_id, :recipient_id, :content, :filename, :date)');
				$sth->execute([':sender_id' => $_SESSION['id'], ':recipient_id' => $recipient_id, ':content' => $data['message'], ':filename' => $data['filename'], ':date' => time()]);

				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw $e;
			}
		}
		
		/*
			Получаем сообщение по ID
		*/
		public function Get($message_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM users__messages WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $message_id]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/*
			Получаем диалоги
		*/
		public function getDialogs($dialog_id = null) {
			try {
				$sth = pdo()->prepare('
					SELECT sub.user1, sub.user2, sub.last_message, sub.last_update, sub.message_id
					FROM (
						SELECT 
							LEAST(sender_id, recipient_id) AS user1,
							GREATEST(sender_id, recipient_id) AS user2,
							content AS last_message,
							last_update,
							id AS message_id
						FROM users__messages
						WHERE sender_id = :userid OR recipient_id = :userid
					) AS sub
					INNER JOIN (
						SELECT 
							LEAST(sender_id, recipient_id) AS grouped_user1,
							GREATEST(sender_id, recipient_id) AS grouped_user2,
							MAX(last_update) AS max_last_update,
							MAX(id) AS max_message_id
						FROM users__messages
						WHERE sender_id = :userid OR recipient_id = :userid
						GROUP BY LEAST(sender_id, recipient_id), GREATEST(sender_id, recipient_id)
					) AS grouped
					ON sub.user1 = grouped.grouped_user1
					AND sub.user2 = grouped.grouped_user2
					AND sub.last_update = grouped.max_last_update
					AND sub.message_id = grouped.max_message_id
					ORDER BY sub.last_update DESC, sub.message_id DESC');
				
				$sth->execute([':userid' => $_SESSION['id']]);
				
				if(!$sth->rowCount()) {
					return '<div class="No">' . getLang('library.users.messages.no_dialogs') . '</div>';
				}
				
				$tpl = new Template;
				while ($dialog = $sth->fetch(PDO::FETCH_OBJ)) {
					$dataMessage = $this->Get($dialog->message_id);
					
					if (empty($dataMessage)) {
						continue;
					}
					
					$tpl->AddCell('dialogs', $tpl->Set([
						'{recipient_id}' => ($dialog->user2 == $_SESSION['id']) ? $dialog->user1 : $dialog->user2,
						'{name}' => ($dialog->user2 == $_SESSION['id']) ? getName($dialog->user1) : getName($dialog->user2),
						'{last_message}' => strip_tags($dialog->last_message),
						'{last_name}' => getName($dataMessage->sender_id),
						'{avatar}' => ($dialog->user2 == $_SESSION['id']) ? getAvatar($dialog->user1) : getAvatar($dialog->user2),
						'{my}' => ($dataMessage->sender_id == $_SESSION['id']) ? '1' : '0',
						'{active}' => ($dialog->user2 == $_SESSION['id']) ? (($dialog->user1 == $dialog_id) ? 'Active' : '') : (($dialog->user2 == $dialog_id) ? 'Active' : '')
					], $tpl->Get('elements/messenger/dialog')));
				}
				
				return $tpl->Execute($tpl->GetCell('dialogs'));
			}
			catch (Exception $e) {
				return $e->getMessage();
			}
		}		
		
		/*
			Получаем сообщения
		*/
		public function getMessages($recipient_id) {
			if (!users()->isValid($recipient_id)) {
				throw new Exception(getLang('library.users.messages.errors.5'));
			}
			
			try {
				$sth = pdo()->prepare("
					SELECT 
						FROM_UNIXTIME(`date`, '%Y-%m-%d') AS `date`,
						`sender_id`,
						`recipient_id`,
						`content`,
						`system`,
						`filename`,
						`id`
					FROM `users__messages`
					WHERE 
						(sender_id = :sender_id AND recipient_id = :recipient_id) 
						OR 
						(sender_id = :recipient_id AND recipient_id = :sender_id)
					ORDER BY `date` ASC
				");
				$sth->execute([
					':sender_id' => $_SESSION['id'],
					':recipient_id' => $recipient_id
				]);
				
				if (!$sth->rowCount()) {
					return '<div class="No">' . getLang('library.users.messages.no_messages') . '</div>';
				}
				
				$this->setReady($recipient_id);
				$tpl = new Template;
		
				$tpl->ClearCell('getMessages');
				$lastDate = null;
				while ($message = $sth->fetch(PDO::FETCH_OBJ)) {
					$messageDate = date('d.m.Y', strtotime($message->date));
		
					if($messageDate != $lastDate) {
						$tpl->AddCell('getMessages', '<div class="date_start">' . $messageDate . '</div>');
					}
					
					$tpl->AddCell('getMessages', $tpl->Set([
						'{id}' => $message->id,
						'{sender_id}' => $message->sender_id,
						'{recipient_id}' => $message->recipient_id,
						'{my}' => ($message->sender_id == $_SESSION['id']) ? '1' : '0',
						'{content}' => htmlspecialchars_decode($message->content),
						'{date}' => date('d.m в H:i', strtotime($message->date)),
						'{system}' => $message->system,
						'{filename}' => isset($message->filename) ? $message->filename : null
					], $tpl->Get('elements/messenger/message')));
		
					$lastDate = $messageDate;
				}
				
				return $tpl->Execute($tpl->GetCell('getMessages'));
			}
			catch (Exception $e) {
				return $e->getMessage();
			}
		}		
		
		/*
			Получаем сообщения в Диапазоне
		*/
		public function getMessagesSince($recipient_id, $last_id) {
			if(!users()->isValid($recipient_id)) {
				throw new Exception(getLang('library.users.messages.errors.5'));
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM `users__messages` WHERE ((sender_id=:sender_id AND recipient_id=:recipient_id) OR (sender_id=:recipient_id AND recipient_id=:sender_id)) AND id > :last_id ORDER BY id ASC');
				$sth->execute([
					':sender_id' => $_SESSION['id'],
					':recipient_id' => $recipient_id,
					':last_id' => $last_id,
				]);
				
				if(!$sth->rowCount()) {
					throw new Exception(getLang('library.users.messages.errors.6'));
				}
				
				$this->setReady($recipient_id);
				$tpl = new Template;
				$tpl->ClearCell('getMessagesSince');

				while($message = $sth->fetch(PDO::FETCH_OBJ)) {
					$tpl->AddCell('getMessagesSince', $tpl->Set([
						'{id}' => $message->id,
						'{sender_id}' => $message->sender_id,
						'{recipient_id}' => $message->recipient_id,
						'{my}' => ($message->sender_id == $_SESSION['id']) ? '1': '0',
						'{content}' => htmlspecialchars_decode($message->content),
						'{date}' => date('d.m в H:i', $message->date),
						'{system}' => $message->system,
						'{filename}' => isset($message->filename) ? $message->filename : null
					], $tpl->Get('elements/messenger/message')));
				}
				
				return $tpl->Execute($tpl->GetCell('getMessagesSince'));
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Устанавливаем прочитанность сообщений
		*/
		public function setReady($senderId) {
			try {
				$sth = pdo()->prepare('UPDATE users__messages SET ready=1 WHERE sender_id=:sender_id');
				$sth->execute([':sender_id' => $senderId]);
				
				return true;
			}
			catch(Exception $e) {
				return false;
			}
		}
		
		/*
			Получаем последний ID между чатами
		*/
		public function lastId($recipient_id) {
			if(!users()->isValid($recipient_id)) {
				return 0;
			}
			
			try {
				$sth = pdo()->prepare('SELECT * FROM `users__messages` WHERE (sender_id=:sender_id AND recipient_id=:recipient_id) OR (sender_id=:recipient_id AND recipient_id=:sender_id) ORDER BY id DESC LIMIT 1');
				$sth->execute([
					':sender_id' => $_SESSION['id'],
					':recipient_id' => $recipient_id,
				]);
				
				if(!$sth->rowCount()) {
					return 0;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ)->id;
			}
			catch(Exception $e) {
				return 0;
			}
		}
		
		/*
			Отправка сообщения об покупке
		*/
		public function InfoBuy($recipient_id, $product_id, $count, $price) {
			$dataProduct = Product()->Get($product_id);
			$formatex = getLang('library.users.messages.confirm', [$_SESSION['id'], getName($_SESSION['id']), $product_id, $dataProduct['product']['name'], $count, $price, conf()->currency->html]);
			
			try {
				$this->Send(['recipient_id' => $recipient_id, 'content' => $formatex, 'system' => 1]);
			}
			catch(Exception $e) {
				throw $e;
			}
		}

		/* Отправляем данные товара из авто-выдачи */
		public function InfoAutoBuy($recipient_id, $data) {
			try {
				$message = getLang('library.users.messages.auto_delivery', $data);
				$this->Send([
					'recipient_id' => $recipient_id,
					'content' => $message,
					'system' => '1'
				]);

				return true;
			}
			catch(Exception $e) {
				throw $e;
			}
		}
	}