<?PHP
	class UsersRecovery {
		// Отправляем почту
		public function sendMail($email) {
			if(empty($email)) {
				throw new Exception(getLang('recovery.labels.0'));
			}

			try {
				if(!$this->IsSMTP()) {
					throw new Exception(getLang('recovery.labels.1'));
				}
		
				$userId = $this->getUserId($email);

				if(is_null($userId)) {
					throw new Exception(getLang('recovery.labels.2'));
				}

				$mail = PHPMailer();
				$mail->addAddress($email);
				
				$key = generateApiKey(14);
				$link = 'https://' . $_SERVER['SERVER_NAME'] . '/recovery/confirm/' . $key;

				$sth = pdo()->prepare('INSERT INTO `users__recovery`(`user_id`, `code`, `date`) VALUES (:user_id, :code, :date)');
				$sth->execute([
					':user_id' => $userId,
					':code' => $key,
					':date' => time()
				]);

				$tpl = new Template('system');
				$body = $tpl->Execute($tpl->Set([
					'{url}' => $link
				], $tpl->Get('mailer/password_recovery')));

				$mail->setContent(getLang('recovery.mail.title'), $body);
				$mail->Send();
			}
			catch(Exception $e) {
				error_log('UsersRecovery: ' . $e->getMessage());
				throw new Exception($e->getMessage());
			}
		}

		// Проверяем работу SMTP
		public function IsSMTP() {
			if(!conf()->mailer->enabled) {
				return false;
			}

			$mail = PHPMailer();
			return $mail->checkSMTP(
				conf()->mailer->hostname,
				conf()->mailer->port,
				conf()->mailer->username,
				conf()->mailer->password
			);
		}

		// Проверяем наличие профиля на сайте
		public function getUserId($email) {
			try {
				$sth = pdo()->prepare('SELECT `id` FROM `users` WHERE `email`=:email LIMIT 1');
				$sth->execute([':email' => $email]);
				
				if($sth->rowCount() == 0) {
					return null;
				}

				return $sth->fetchColumn();
			}
			catch (Exception $e) {
				error_log('UsersRecovery: ' . $e->getMessage());
				return null;
			}
		}		
	}