<?PHP
	class UsersTransactions extends Users {
		var $dataStatus = [];
		
		public function __construct() {
			$this->dataStatus = [
				["name" => getLang('library.transactions.status.0')],
				["name" => getLang('library.transactions.status.1')],
				["name" => getLang('library.transactions.status.2')],
				["name" => getLang('library.transactions.status.3')],
				["name" => getLang('library.transactions.status.4')],
				["name" => getLang('library.transactions.status.5')]
			];
			
			if(json_last_error() != JSON_ERROR_NONE) {
				die(json_last_error_msg());
			}
		}
		
		/*
			Привязываем и добавляет транзакцию пользователя
		*/
		public function Add($user_id, $data) {
			try {
				$sth = pdo()->prepare('INSERT INTO money__transactions(user_id, amount, data, type, status, date) VALUES (:user_id, :amount, :data, :type, :status, :date)');
				$sth->execute([
					':user_id' => $user_id,
					':amount' => reFormatPrice($data['amount']),
					':data' => isset($data['data']) ? json_encode($data['data']) : null,
					':type' => $data['type'],
					':status' => $data['status'],
					':date' => time()
				]);
				
				return pdo()->lastInsertId();
			}
			catch(Exception $e) {
				throw new Exception($e->getMessage());
			}
		}
		
		/*
			Получаем визуальные транзакции пользователя
		*/
		public function getTableGUI($user_id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM money__transactions WHERE user_id=:user_id ORDER BY id DESC');
				$sth->execute([':user_id' => $user_id]);
				
				if(!$sth->rowCount()) {
					return tpl()->Execute(tpl()->Get('elements/transactions/no_data'));
				}
				
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$preType = $this->getTypeFromId($row->type);
					$preStatus = $this->dataStatus[$row->status];
					
					tpl()->AddCell('listTransactions', tpl()->Set([
						'{id}' => generateUniqueId($row->id),
						'{date}' => getLang('getTimeAgo.date', [date('d.m.Y', $row->date), date('H:i', $row->date)]),
						'{type}' => isset($preType) ? $preType->name : getLang('system.errors.undefined'),
						'{status}' => isset($preStatus) ? $preStatus['name'] : getLang('system.errors.undefined'),
						'{status_id}' => $row->status,
						'{amount}' => $row->amount
					], tpl()->Get('elements/transactions/item')));
				}
				
				return tpl()->Execute(tpl()->GetCell('listTransactions'));
			}
			catch(Exception $e) {
				return tpl()->Execute(tpl()->Get('elements/transactions/no_data'));
			}
		}
		
		/*
			Получаем тип транзакции
		*/
		public function getTypeFromId($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM money__types WHERE id=:id LIMIT 1');
				$sth->execute([
					':id' => $id
				]);
				
				if(!$sth->rowCount()) {
					return null;
				}

				$data = $sth->fetch(PDO::FETCH_OBJ);
				$data->name = getLang($data->name);
				
				return $data;
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/*
			Получение типа транзакции по коду
		*/
		public function getTypeFromCode($string) {
			try {
				$sth = pdo()->prepare('SELECT * FROM money__types WHERE target=:target LIMIT 1');
				$sth->execute([
					':target' => $string
				]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				$data = $sth->fetch(PDO::FETCH_OBJ);
				$data->name = getLang($data->name);
				
				return $data;
			}
			catch(Exception $e) {
				return null;
			}
		}
		
		/*
			Получаем список покупок пользователя
		*/
		public function getUserBuys($user_id) {
			$settings = users()->getSettings($user_id);
			if (!is_null($settings)) {
				$errorMessage = getLang('library.users.trans.errors.1');

				if ($settings['view_orders'] == 2 && empty($_SESSION['id'])) {
					return getNotData($errorMessage);
				}
				elseif ($settings['view_orders'] == 0 && (empty($_SESSION['id']) || $_SESSION['id'] != $user_id)) {
					return getNotData($errorMessage);
				}
			}

			try {
				$sth = pdo()->prepare('SELECT * FROM money__transactions WHERE user_id=:user_id AND type=3 AND data IS NOT NULL ORDER BY id DESC');
				$sth->execute([
					':user_id' => $user_id
				]);
				
				if(!$sth->rowCount()) {
					return getNotData(getLang('library.users.trans.errors.0'));
				}
				
				$SellerProduct = new SellerProduct;
				
				tpl()->ClearCell('listOrders');
				while($row = $sth->fetch(PDO::FETCH_OBJ)) {
					$trans_data = json_decode($row->data, true);

					if(isset($trans_data['product_id'])) {
						$product = Product()->Get($trans_data['product_id'])['product'];
						$price = formatPrice($product['id']);

						tpl()->AddCell('listOrders', tpl()->Set([
							'{id}' => $product['id'],
							'{image}' => $product['image'],
							'{price}' => $price->total,
							'{discount}' => $price->discount,
							'{oldprice}' => $price->oldprice,
							'{name}' => htmlspecialchars_decode($product['name']),
							'{author_id}' => $product['author_id'],
							'{currency}' => conf()->currency->html,
							'{row_sells}' => $SellerProduct->rowSells($product['author_id']),
							'{date}' => date('d.m.Y', $row->id),
							'{seller_id}' => $product['author_id'],
							'{seller_image}' => getAvatar($product['author_id']),
							'{seller_name}' => getName($product['author_id']),
							'{stars_count}' => Reviews()->getStarsProduct($product['id']),
							'{stars_reviews}' => Reviews()->rowProduct($product['id']),
							'{filters}' => ''
						], tpl()->Get('elements/product/item')));
					}
				}
				
				return tpl()->Execute(tpl()->GetCell('listOrders'));
			}
			catch(Exception $e) {
				return tpl()->Set(['{Content}' => $e->getMessage()], tpl()->Get('elements/product/no_data'));
			}
		}
		
		/*
			Получение транзакции по Индексу
		*/
		public function Get($id) {
			try {
				$sth = pdo()->prepare('SELECT * FROM money__transactions WHERE id=:id LIMIT 1');
				$sth->execute([':id' => $id]);
				
				if(!$sth->rowCount()) {
					return null;
				}
				
				return $sth->fetch(PDO::FETCH_OBJ);
			}
			catch(Exception $e) {
				return null;
			}
		}

		/*
			Получаем покупку пользователя
		*/
		public function getProduct($trans_id) {
			try {
				$transData = $this->Get($trans_id);

				if(is_null($transData) || is_null($transData->data)) {
					return null;
				}

				$transData = json_decode($transData->data, true);
				if(empty($transData) || empty($transData['product_id'])) {
					return null;
				}

				return $transData['product_id'];
			}
			catch(Exception $e) {
				error_log($e->getMessage());
				return null;
			}
		}
	}