function setDarkMode() {
	if(!$('html').hasClass('dark')) {
		localStorage.setItem('darkMode', '1');
		$('#changeMode').prop('checked', true);
		$('html').addClass('dark');
		imageLogo();
	}
}

function setLightMode() {
	if($('html').hasClass('dark')) {
		localStorage.removeItem('darkMode');
		$('#changeMode').prop('checked', false);
		$('html').removeClass('dark');
		imageLogo();
	}
}

function imageLogo() {
	$('#Page > .Header .Navigation > .Start > .Brand > a > img').attr('src', function(index, currentSrc) {
		if(currentSrc.includes('_dark')) {
		  return currentSrc.replace('_dark', '');
		}
		else {
		  return currentSrc.replace(/(\.png|\.jpg|\.jpeg|\.gif|\.svg)$/i, '_dark$1');
		}
	});	  
}

if(localStorage.getItem('darkMode')) {
	setDarkMode();
}

$(function() {
	$('#changeMode').on('click', function() {
		if($('html').hasClass('dark')) {
			setLightMode();
		}
		else {
			setDarkMode();
		}
	});

	if($('html').hasClass('dark')) {
		$('#changeMode').prop('checked', true);
	}

    $('.Profile .dropdown-menu').on('click', function(e) {
        e.stopPropagation();
    });
	
	$('#modalCatalog').on('show.bs.modal', function() {
		if(!$(this).find('[data-target="onCatalog"].Active').length) {
			$('[data-target="onCatalog"]:first-child').click();
		}
	});
	
	/* Сортировка Маркета */
	var inSearch = $('#seller_online');
	if(inSearch.length) {
		inSearch.on('change', function() {
			$isOnlineSeller = $(this).is(':checked');
			sortSellerOnline();
		});
	}
	
	var inSearch = $('#onFilter');
	if(inSearch.length) {
		inSearch.on('change', function() {
			sortPrice($(this).val());
		});
	}

	/* Изменение аватара */
	$('#changeImage > button').on('click', function() {
		$('#changeImage > form > input').click();
	});

	$('#changeImage > form > input').on('change', function() {
		$('#changeImage > form').submit();
	});

	$('#changeImage > form').submit(function(Event) {
		Event.preventDefault();

		SendPost('/api/v1/users/editAvatar', {
			this: this
		}, (Result) => {
			ShowToasty(Result.Message, Result.Alert);
			
			switch(Result.Alert) {
				case 'Info':
				case 'Success':
					setTimeout('location.reload();', 300);
				break;
			}
		});
	});
});

var $isOnlineSeller = false;

function sortSellerOnline() {
    var inSearch = $('#seller_online');
    
    if (inSearch.length) {
        if ($isOnlineSeller) {
            $('#Products .Item > .Info > li > a.Online').closest('.col-lg-3').prependTo('#Products > .row');
        }
    }
}

function sortPrice(type) {
    var items = $('#Products .col-lg-3').get();
    
    items.sort(function(a, b) {
		if($isOnlineSeller) {
			var onlineSellerA = $(a).find('.Info > li > a.Online').length > 0;
			var onlineSellerB = $(b).find('.Info > li > a.Online').length > 0;
			
			if(onlineSellerA !== onlineSellerB) {
				return (onlineSellerB ? 1 : -1) - (onlineSellerA ? 1 : -1);
			}
		}
		
        var priceA = parseInt($(a).find('.Info > li.Bottom > .Price').text().replace('₽', '').trim());
        var priceB = parseInt($(b).find('.Info > li.Bottom > .Price').text().replace('₽', '').trim());
        
        if(type === 'down') {
            return priceB - priceA;
        }
		else {
            return priceA - priceB;
        }
    });
    
    $.each(items, function(index, item) {
        $('#Products > .row').append(item);
    });
}

function updatePrice() {
    let count = parseFloat($('#product_count').val().replace(/[\s,]/g, ''));
    let price = parseFloat($('#product_price_count').val().replace(/[\s,]/g, ''));

    if(isNaN(count) || isNaN(price)) {
        return;
    }

    let endPrice = count * price;
    endPrice = parseFloat(endPrice.toFixed(2));

    $('#result_price').text(endPrice);
}

$(window).on('load', function() {
	$('.preLoader').fadeOut('slow', function() {
		$(this).css('display', 'none');
	});
});

$(function() {
	if($('html').hasClass('dark')) {
		imageLogo();
	}

	$('button[data-action]').on('click', function(e) {
		var action = $(this).data('action');
		
		switch(action) {
			case 'goToStep': {
				var step_id = $(this).data('step_id');

				SendPost('/api/v1/market/add', {goto_step: 1, step_id: step_id}, (Result) => {
					if(Result.Content) {
						$('#step-content').html(Result.Content);
					}
					else {
						ShowToasty(Result.Message, Result.Alert);
					}
				});

				break;
			}
		}
	});

	/* Меню профиля */
	$('[data-target="open-menu"]').on('click', function() {
		if(!$('[data-toggle="open-menu"]').hasClass('Show')) {
			$('[data-toggle="open-menu"]').fadeIn(300, function() {
				$('[data-toggle="open-menu"]').addClass('Show');
			});
		}
		else {
			$('[data-toggle="open-menu"]').fadeOut(300, function() {
				$('[data-toggle="open-menu"]').removeClass('Show');
			});
		}
	});

	/* Открываем информацию о товаре */
	if($('[data-target="open-full"]').length) {
		$('[data-target="open-full"]').on('click', function() {
			$(this).remove();
			$('.Page-Product > .End .product_desc .content').css('max-height', '100%');
		});

		$('.product_desc .content').each(function() {
			const content = $(this);
			const description = content.text().trim();
			const lineHeight = parseFloat(content.css('line-height'));
			const containerHeight = content.height();
		
			if(!lineHeight || !containerHeight) {
				console.warn('Line height или высота контейнера не определены.');
				return;
			}
		
			const lineCount = Math.floor(containerHeight / lineHeight);

			if(lineCount < 4) {
				content.siblings('[data-target="open-full"]').hide();
			}
		});
		
	}

	/* Отправляем изображение */
	$('.onMessengerSendImage').on('click', function() {
		if($('#onMessengerSendImage > input[type="file"]').length) {
			$('#onMessengerSendImage > input[type="file"]').click();
		}
		
	});

	if($('#onMessengerSendImage > input[type="file"]').length) {
		$('#onMessengerSendImage > input[type="file"]').on('change', function() {
			const file = this.files[0];

			if(file) {
				$('#onMessengerSendImage').submit();
			}
		});
	}

	if($('[data-premium]').length) {
		$('[data-premium]').on('click', function() {
			var productId = $(this).data('premium');
			$('[data-action="buy_boost"]').data('id', productId);

			$('#modalBuyPremium').modal('show');
		});
	}

	$('#product_count').on('input', function() {
		let val = $(this).val().replace(/,/g, '');
		$(this).val(Number(val).toLocaleString('ru-RU'));
		updatePrice();
	});
	
	$('[data-minus], [data-plus]').on('click', function() {
		let o = $(this).data('minus') || $(this).data('plus');
		let count = parseInt($(o).val().replace(/\s/g, ''));

		if($(this).data('minus') && count <= 1) {
			$.alert({
				title: Languages('buy.alert.title'),
				content: Languages('buy.alert.min_count')
			});
			return;
		}
	
		count += $(this).data('minus') ? -1 : 1;
		$(o).val(count.toLocaleString('ru-RU'));
		updatePrice();
	});
	
	$('#product_count').on('input', function() {
		updatePrice();
	});

	$('[toggle="product"]').on('click', function() {
		var data = $(this).data('toggle');
		var product_id = $(this).data('product_id');

		if([1, 4].includes(data)) {
			SendPost('/api/v1/product/toggle', {
				product_id: product_id
			}, (Result) => {
				if(Result.Alert == 'Success') {
					if(Result.EnableId === 1) {
						$('[toggle="product"][data-product_id="' + product_id + '"] > i').removeClass('bi bi-toggle-off');
						$('[toggle="product"][data-product_id="' + product_id + '"] > i').addClass('bi bi-toggle-on');
					}
					else {
						$('[toggle="product"][data-product_id="' + product_id + '"] > i').removeClass('bi bi-toggle-on');
						$('[toggle="product"][data-product_id="' + product_id + '"] > i').addClass('bi bi-toggle-off');
					}

					$(this).data('toggle', Result.EnableId);
					return;
				}

				ShowToasty(Result.Message, Result.Alert);
			});
		}
	});
});

// Если мы на странице маркета
if(window.location.pathname.startsWith('/market')) {
	$(function() {
		$('#filter_hidden').on('click', function() {
			let filter = $('#filter_hidden').closest('.filter');

			if(!filter.hasClass('hidden')) {
				filter.addClass('hidden');
				filter.find('.content').fadeOut(100);
				$(this).find('i').removeClass('bi-x').addClass('bi-list');
			}
			else {
				filter.removeClass('hidden');
				filter.find('.content').fadeIn(500);
				$(this).find('i').removeClass('bi-list').addClass('bi-x');
			}
		});
		
		if(isMobile.any()) {
			$('#filter_hidden').click();
		}

		new Swiper('#Subcategory > .swiper', {
			slidesPerView: 2,
			spaceBetween: 10,
			loop: false,
			breakpoints: {
				768: { slidesPerView: 4, spaceBetween: 10 },
				1024: { slidesPerView: 10, spaceBetween: 10 }
			}
		});
		
		function filterProducts() {
			$(".product-item").each(function() {
				let item = $(this);
				let show = true;
		
				$("[data-filter]").each(function() {
					let filterType = $(this).data("filter");
					let filterValue = $(this).val();
		
					if(filterType === "price-min" || filterType === "price-max") {
						let price = parseFloat(item.data("price").toString().replace(",", "."));
						
						if(filterType === "price-min" && filterValue && price < parseFloat(filterValue)) {
							show = false;
						}
						
						if(filterType === "price-max" && filterValue && price > parseFloat(filterValue)) {
							show = false;
						}
					}
					else if(filterType === "name") {
						let name = item.data("name");
						
						if(typeof name !== "string") {
							name = String(name || "").toLowerCase();
						} else {
							name = name.toLowerCase();
						}
		
						if(filterValue && !name.includes(filterValue.toLowerCase())) {
							show = false;
						}
					}
					else if(filterType === "online-only") {
						let isOnline = item.find(".status").hasClass("online");
		
						if($(this).is(":checked") && !isOnline) {
							show = false;
						}
					} 
					else if(filterType === "discount-only") {
						let discount = item.data("discount");
						if($(this).is(":checked") && discount === 0) {
							show = false;
						}
					} 
					else {
						let itemValue = item.data(filterType);
		
						if(filterValue && itemValue && !itemValue.toString().toLowerCase().includes(filterValue.toString().toLowerCase())) {
							show = false;
						}
					}
				});
				
				item.toggle(show);
			});
		
			if(!$("#Products > .row .product-item:visible").length && !$("#Products > .row .noData").length) {
				$('#Products > .noData').css('display', 'flex');
			}
			else {
				$('#Products > .noData').css('display', 'none');
			}
		}
		
		filterProducts();
		$("[data-filter]").on("input change", filterProducts);
		
		const path = window.location.pathname;
		let categoryId = path.startsWith('/market') ? path : null;

		if(categoryId.length > 0) {
			$("#Subcategory .Item[href='" + categoryId + "']").addClass('Active');
		}
		else {
			$("#Subcategory .Item[data-id='none']").addClass('Active');
		}
	});
}